@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: di_bootagent.c,v 6.21 2003/03/21 15:21:00 brbarret Exp $
+ * $Id: di_bootagent.c,v 6.23 2003/11/15 13:17:15 jsquyres Exp $
  *
  *	Function:	- obtains link information from boot agent
  *	Accepts:	- link array (value returned)
@@ -60,7 +60,7 @@ extern OPT *lam_daemon_optd;
  */
 int
 di_bootagent(struct dilink **pdilinks, struct diinfo **pdiinfo, int4 *pndil, 
-	     int argc, char* argv[])
+	     int4 *mynode, int argc, char* argv[])
 {
   int4 i;
   int dli_port = 0;
@@ -79,9 +79,15 @@ di_bootagent(struct dilink **pdilinks, struct diinfo **pdiinfo, int4 *pndil,
 
   /* Open the boot SSI */
 
-  if (lam_ssi_boot_open(lam_daemon_optd, LAM_SSI_BOOT_LOCATION_LEAF) != 0)
+  if (lam_ssi_base_open(lam_daemon_optd) != 0 ||
+      lam_ssi_boot_open(lam_daemon_optd) != 0 ||
+      lam_ssi_boot_select(LAM_SSI_BOOT_LOCATION_LEAF) != 0)
     lampanic("di_bootagent: lam_ssi_boot_open failed");
 
+  /* Ask the boot module what our node ID is */
+
+  *mynode = lam_ssi_boot.lsba_get_nodeid(lam_daemon_optd);
+
   /* Call back to the booting agent and send it our info. */
 
   if (lam_ssi_boot.lsba_send_lamd_info(lam_daemon_optd, dli_port) != 0)
@@ -123,13 +129,13 @@ di_bootagent(struct dilink **pdilinks, struct diinfo **pdiinfo, int4 *pndil,
     (*pdilinks)[i].dil_addr = universe[i].lnd_addr;
     (*pdiinfo)[i].dii_num_cpus = universe[i].lnd_ncpus;
     (*pdiinfo)[i].dii_node_type = universe[i].lnd_type;
-    lamlog("Link %d: node: %d, cpus: %d, type: %d, ip: %s\n",
+    lamlog("Link %d: node: %d, cpus: %d, type: %d, ip: %s, port %d\n",
 	   i, (*pdilinks)[i].dil_link,  (*pdiinfo)[i].dii_num_cpus,
 	   (*pdiinfo)[i].dii_node_type, 
-	   inet_ntoa((*pdilinks)[i].dil_addr.sin_addr));
+	   inet_ntoa((*pdilinks)[i].dil_addr.sin_addr),
+           ntohs((*pdilinks)[i].dil_addr.sin_port));
   }
   free(universe);
 
-
   return (dli_sd);
 }
