@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -36,6 +36,7 @@
 #include <terror.h>
 #include <typical.h>
 #include <lam_network.h>
+#include <lam-ssi-boot.h>
 
 
 /*
@@ -53,7 +54,7 @@ extern OPT* lam_daemon_optd;
 
 int
 di_bootcmd(struct dilink **pdilinks, struct diinfo **pdiinfo, int4 *pndil,
-	   int argc, char **argv)
+	   int *mynodeid, int argc, char **argv)
 {
 	int		dli_port;	/* datalink input port */
 	int		dli_sd;		/* datalink input socket */
@@ -68,6 +69,11 @@ di_bootcmd(struct dilink **pdilinks, struct diinfo **pdiinfo, int4 *pndil,
 		errno = EUSAGE;
 		lampanic("dli_inet: <host> <link> <cpus> go in tuples");
 	}
+
+        /* Find out what my node ID is */
+
+        *mynodeid = lam_ssi_boot_base_get_nodeid(lam_daemon_optd);
+
 /*
  * Get my dli port number.
  */
