@@ -11,11 +11,8 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	GDB
  *
- *	$Id: fqsimplex.c,v 6.4 2002/10/09 20:56:36 brbarret Exp $
+ *	$Id: fqsimplex.c,v 6.5 2003/06/30 05:19:30 brbarret Exp $
  * 
  *	Function:	- simple filed requests
  *	Accepts:	- filed request
@@ -50,6 +47,7 @@ void			fqftrunc();
 void			fqgetwd();
 void			fqincr();
 void			fqmkdir();
+void			fqrename();
 void			fqrmdir();
 void			fqrmfd();
 void			fqstat();
@@ -61,6 +59,7 @@ void			fqunlink();
  */
 extern int4		fmychdir();
 extern int4		fmyaccess();
+extern int4		fmyrename(CONST char *from, CONST char *to);
 extern int4		fmymkdir();
 extern int4		fmyrmdir();
 extern int4		fmyunlink();
@@ -188,6 +187,26 @@ struct freq		*fq;
 			INT4_NIL);
 }
 
+/*
+ * fqrename
+ *
+ * Function: - rename a file
+ */
+void
+fqrename(struct freq* fq)
+{
+  int4 ret;
+  struct fqrename data;
+  
+  memcpy(&data, fbuf, sizeof(struct fqrename));
+
+  ret = fmyrename(data.source, data.dest);
+  fsendr(fq->fq_src_node, fq->fq_src_event,
+	 (ret == LAMERROR) ? errno : 0, ret, INT4_NIL,
+	 INT4_NIL);
+}
+
+
 /*
  *	fqrmdir
  *
