@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,10 +11,8 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
  *
- *	$Id: haltd.c,v 1.24.2.1 2003/09/27 05:10:40 brbarret Exp $
+ * $Id: haltd.c,v 1.25 2003/09/27 05:16:16 brbarret Exp $
  * 
  *	Function:	- halt server
  *			- if receive local message, rebroadcast to all 
@@ -237,12 +235,15 @@ diediedie(void)
 
 /*
  * Oops -- execvp failed.  It shouldn't, because tkill should be in
- * the path.  Oh well -- kill the parent.
+ * the path.  Oh well -- kill the parent (don't kill init by
+ * accident!).
  */
-    if (kill(getppid(), SIGHUP))
-      if (kill(getppid(), SIGINT))
-	if (kill(getppid(), SIGKILL))
-	  lampanic("haltd (kill)");
+    pid = getppid();
+    if (pid > 1) 
+      if (kill(getppid(), SIGHUP))
+        if (kill(getppid(), SIGINT))
+          if (kill(getppid(), SIGKILL))
+            lampanic("haltd (kill)");
     exit(1);
   }
   
