@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,11 +11,6 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB/NJN
- *
- *	$Id: pqcreate.c,v 6.20 2003/04/09 03:37:46 brbarret Exp $
  *
  *	Function:	- OTB kenyad
  *			- creates a process
@@ -44,6 +39,7 @@
 #include "terror.h"
 #include "typical.h"
 #include "lamdebug.h"
+#include "etc_misc.h"
 
 /*
  * private functions
@@ -101,7 +97,7 @@ pqcreate(void)
  */
     if (pq.pq_argv) {
 	get_argv_env(&argv, &envv);
-	argc = argvcount(argv);
+	argc = sfh_argv_count(argv);
     }
 /*
  * If flag set to try parents working dir, check if it exists.
@@ -121,13 +117,13 @@ pqcreate(void)
       if (!LAM_isNullStr(pq.pq_wrkdir)) {
 	lamlog("kenyad: looking for executable \"%s\" in directory \"%s\"", 
 	       pq.pq_name, pq.pq_wrkdir);
-	loadname = sfh_path_env_findv(pq.pq_name, R_OK | X_OK,
+	loadname = sfh_path_env_findv(pq.pq_name, X_OK,
 				      envv, pq.pq_wrkdir);
       } else {
 	cwd = getworkdir();
 	lamlog("kenyad: looking for executable \"%s\" in directory \"%s\"", 
 	       pq.pq_name, cwd);
-	loadname = sfh_path_env_findv(pq.pq_name, R_OK | X_OK,
+	loadname = sfh_path_env_findv(pq.pq_name, X_OK,
 				      envv, cwd);
 	if (cwd) 
 	  free(cwd);
@@ -146,9 +142,9 @@ pqcreate(void)
     if (!pq.pq_argv) {
       if (loadname) {
 	argc = 1;
-	argv = argvbreak(loadname, 0xa);
+	argv = sfh_argv_break(loadname, 0xa);
 	if (argv == 0) 
-	  lampanic("kenyad (argvbreak)");
+	  lampanic("kenyad (sfh_argv_break)");
       } else {
 	argv = 0;
       }
@@ -431,17 +427,17 @@ get_argv_env(char ***argv, char ***envv)
  */
 	if (pq.pq_envsize) {
 		len = strlen(args);
-		*envv = argvbreak(args + len - pq.pq_envsize, 0xa);
-		if (*envv == 0) lampanic("kenyad (argvbreak)");
+		*envv = sfh_argv_break(args + len - pq.pq_envsize, 0xa);
+		if (*envv == 0) lampanic("kenyad (sfh_argv_break)");
 
 		if (pq.pq_envsize < len) {
 			args[len - pq.pq_envsize] = 0;
-			*argv = argvbreak(args, 0xa);
-			if (*argv == 0) lampanic("kenyad (argvbreak)");
+			*argv = sfh_argv_break(args, 0xa);
+			if (*argv == 0) lampanic("kenyad (sfh_argv_break)");
 		}
 	} else {
-		*argv = argvbreak(args, 0xa);
-		if (*argv == 0) lampanic("kenyad (argvbreak)");
+		*argv = sfh_argv_break(args, 0xa);
+		if (*argv == 0) lampanic("kenyad (sfh_argv_break)");
 	}
 
 	free(args);
