@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,40 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1995 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Log: pqstate.c,v $
- *	Revision 6.2.2.1  2002/10/09 19:48:45  brbarret
- *	* Updated license header to match current requirements
- *	
- *	Revision 6.2  1999/06/12 17:11:33  kmeyer1
- *	added copyright
- *	
- *	Revision 6.1  1996/11/23 18:43:18  nevin
- *	Ohio Release
- *	
- * Revision 6.0  96/02/29  13:31:46  gdburns
- * Ohio Release
- * 
- * Revision 5.2.1.1  96/02/27  20:11:31  gdburns
- * general modernization
- * 
- * Revision 5.2  94/08/22  13:50:31  gdburns
- * Ohio Release
- * 
- * Revision 5.1.1.1  94/08/18  10:12:24  gdburns
- * Always use buffers so as not to confuse ksr() in kenyad.
- * 
- * Revision 5.1  94/05/18  10:42:20  gdburns
- * Ohio Release
- * 
- * Revision 2.3.1.1  94/05/11  20:25:13  gdburns
- * Clean up abnormally terminated non-child clients.
- * 
- * Revision 2.3  94/04/22  12:27:41  gdburns
- * Ohio Release
+ * $Id: pqstate.c,v 6.5 2003/05/31 22:28:48 jsquyres Exp $
  * 
  *	Function:	- OTB kenyad
  *			- assembles status of processes
@@ -68,6 +37,7 @@
 #include <t_types.h>
 #include <terror.h>
 #include <typical.h>
+#include <etc_misc.h>
 
 /*
  * external variables
@@ -127,14 +97,16 @@ pqstate()
  * Fill in argv[0], usually the program name, if it exists.
  */
 		if (pstate->p_argc != 0) {
-			strncpy(reply_state.ps_name, *(pstate->p_argv),
+			lam_strncpy(reply_state.ps_name, *(pstate->p_argv),
 					PSMAXNAME);
 		} else {
-			strncpy(reply_state.ps_name, "<unknown>", PSMAXNAME);
+			lam_strncpy(reply_state.ps_name, "<unknown>", 
+                                    PSMAXNAME);
 		}
 
 		reply_state.ps_name[PSMAXNAME - 1] = '\0';
-		strncpy(reply_state.ps_loadpt, pstate->p_loadpt, PSMAXNAME);
+		lam_strncpy(reply_state.ps_loadpt, pstate->p_loadpt, 
+                            PSMAXNAME);
 		reply_state.ps_loadpt[PSMAXNAME - 1] = '\0';
 /*
  * Fill in the kernel info.
@@ -143,8 +115,8 @@ pqstate()
 /*
  * Is the process attached to kenyad, but not yet a known kernel process?
  */
-			if ((pstate->p_rtf & RTF_KENYA) &&
-					!(pstate->p_rtf & RTF_KERNEL)) {
+			if ((pstate->p_rtf & RTF_KENYA_ATTACH) &&
+			    !(pstate->p_rtf & RTF_KERNEL)) {
 				reply_state.ps_kernel.ks_state = ltot(KSFREE);
 				reply_state.ps_kernel.ks_event = ltot(0);
 				reply_state.ps_kernel.ks_type = ltot(0);
@@ -179,7 +151,7 @@ pqstate()
 		reply_state.ps_nodeid = ltot(NOTNODEID);
 		reply_state.ps_event = ltot(NOTEVENT);
 
-		strncpy(reply_state.ps_name, "<unknown>", PSMAXNAME);
+		lam_strncpy(reply_state.ps_name, "<unknown>", PSMAXNAME);
 		reply_state.ps_name[PSMAXNAME - 1] = '\0';
 		reply_state.ps_loadpt[0] = '\0';
 
