@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- *	$Id: kernelio.c,v 6.29.2.3 2003/10/11 19:04:01 brbarret Exp $
+ *	$Id: kernelio.c,v 6.33 2003/12/09 13:56:46 jsquyres Exp $
  * 
  *	Function:	- receives kernel requests and send replies
  *			- moves messages between clients
@@ -195,7 +195,9 @@ kio_init()
         addr_file = strrchr(addr_dir, '/');
         if (addr_file != NULL) {
           *addr_file = '\0';
-          chdir(addr_dir);
+          if (chdir(addr_dir) != 0) {
+            lampanic("chdir failed!");
+          }
           ++addr_file;
         } else {
           addr_file = addr_dir;
@@ -225,8 +227,14 @@ kio_init()
 	    fprintf(stderr, "lamd kernel: \"%s\"\n", kernel_un.sun_path);
 	    fprintf(stderr, "lamd kernel: check the unix permissions");
 	    lampanic("lamd kernel");
-	  } else
-	    lampanic("lamd kernel: problem with bind()");
+	  } else {
+            char name[4096];
+            snprintf(name, sizeof(name) - 1,
+                     "lamd kernel: problem with bind (%s/%s)",
+                     addr_dir, kernel_un.sun_path);
+            name[sizeof(name) - 1] = '\0';
+            lampanic(name);
+          }
 	}
 /*
  * Establish backlog queue.
