@@ -1,38 +1,18 @@
 /*
- * Copyright 1998-2001, University of Notre Dame.
- * Authors: Jeffrey M. Squyres, Arun Rodrigues, and Brian Barrett with
- *          Kinis L. Meyer, M. D. McNally, and Andrew Lumsdaine
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ *                         All rights reserved.
+ * Copyright (c) 1998-2001 University of Notre Dame. 
+ *                         All rights reserved.
+ * Copyright (c) 1994-1998 The Ohio State University.  
+ *                         All rights reserved.
  * 
- * This file is part of the Notre Dame LAM implementation of MPI.
+ * This file is part of the LAM/MPI software package.  For license
+ * information, see the LICENSE file in the top level directory of the
+ * LAM/MPI source distribution.
  * 
- * You should have received a copy of the License Agreement for the Notre
- * Dame LAM implementation of MPI along with the software; see the file
- * LICENSE.  If not, contact Office of Research, University of Notre
- * Dame, Notre Dame, IN 46556.
- * 
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted subject to the conditions specified in the
- * LICENSE file.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
- * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
- * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
- * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- * 
- * Additional copyrights may follow.
- * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
+ * $HEADER$
  *
- *	$Id: kkill.c,v 6.5 2000/09/14 22:45:59 jsquyres Exp $
+ * $Id: kkill.c,v 6.9 2003/05/21 19:16:15 jsquyres Exp $
  * 
  *	Function:	- supports client cleanup by the kernel and by tkill(1)
  *			- OTB specific code
@@ -53,11 +33,12 @@
 /*
  * global functions
  */
-void			kkillall();
-int			kkillclose();
-int			kkillopen();
-int			kkillrewrite();
-int			kkillwrite();
+void kkillall(void);
+int kkillclose(void);
+int kkillopen(char *f_kill);
+int kkillrewrite(struct kproc kptable[]);
+int kkillwrite(int4 pid);
+
 
 /*
  * external variables
@@ -65,22 +46,22 @@ int			kkillwrite();
 extern struct kproc	*pready;
 extern struct kproc	*pblock;
 
+
 /*
  * local variables
  */
 static FILE		*fp_kill;	/* kill file ptr */
 static int		fd_kill;	/* kill file desc. */
 
+
 /*
  *	kkillall
  *
  *	Function:	- catches SIGTERM and SIGHUP
  *			- kills all attached processes
- *			- exits
  */
 void
-kkillall()
-
+kkillall(void)
 {
 	struct kproc	*p;
 
@@ -99,10 +80,9 @@ kkillall()
 			kill(p->kp_pid, SIGHUP);
 		}
 	}
-
-	exit(0);
 }
 
+
 /*
  *	kkillopen
  *
@@ -116,41 +96,45 @@ kkillall()
  *	Returns:	- 0 or LAMERROR
  */
 int
-kkillopen(f_kill)
-
-char			*f_kill;
-
+kkillopen(char *f_kill)
 {
 	fd_kill = open(f_kill, O_WRONLY | O_CREAT | O_EXCL, 0600);
+	if (fd_kill < 0)
+	  return(LAMERROR);
 
-	if (fd_kill < 0) {
-		return(LAMERROR);
-	}
+	/* Make this be close-on-exec so that forked children don't
+	   inherit it.  Here's a corner case where it matters: the LAM
+	   session directory is actually located on NFS (e.g.,
+	   /tmp/lam-username@hostname is on NFS).  When the user
+	   invokes lamhalt, which invokes the haltd, which forks off
+	   tkill, tkill will go unlink all files in the session
+	   directory.  Unfortunately, tkill will have inherited this
+	   open file, which is now just removed.  Hence, when tkill
+	   tries to remove the directory itself, NFS will complain
+	   that the directory is not empty because NFS will
+	   automatically create a .nfsXXXXX cache/placeholder file in
+	   the directory when the kill file was unlinked but someone
+	   (tkill) still had it open.  End result: the session
+	   directory is not removed.  And this is bad.  I'm annoyed
+	   because it took me so long to track this down! */
+
+	if (fcntl(fd_kill, F_SETFD, 1) != 0)
+	  return LAMERROR;
 
 	fp_kill = fdopen(fd_kill, "w");
 
-	if (! fp_kill) {
-		return(LAMERROR);
-	}
-
+	if (! fp_kill)
+	  return(LAMERROR);
+	
 	setvbuf(fp_kill, (char *) 0, _IOLBF, 0);
-
-	if (fprintf(fp_kill, "%d\n", (int) getpid()) == EOF) {
-		return(LAMERROR);
-	}
-
-#if 0
-	/* I'm not sure why this is here.  It doesn't seem harmful,
-           but it also just seems weird, in a security sense... Hence,
-           I'm commenting it out. */
-	if (chmod(f_kill, 0644)) {
-		return(LAMERROR);
-	}
-#endif
-
+	
+	if (fprintf(fp_kill, "%d\n", (int) getpid()) == EOF)
+	  return(LAMERROR);
+	
 	return(0);
 }
 
+
 /*
  *	kkillwrite
  *
@@ -159,10 +143,7 @@ char			*f_kill;
  *	Returns:	- 0 or LAMERROR
  */
 int
-kkillwrite(pid)
-
-int4			pid;
-
+kkillwrite(int4 pid)
 {
 	if (fprintf(fp_kill, "%d\n", pid) == EOF) {
 		return(LAMERROR);
@@ -171,6 +152,7 @@ int4			pid;
 	}
 }
 
+
 /*
  *	kkillrewrite
  *
@@ -179,10 +161,7 @@ int4			pid;
  *	Returns:	- 0 or LAMERROR
  */
 int
-kkillrewrite(kptable)
-
-struct kproc		kptable[];
-
+kkillrewrite(struct kproc kptable[])
 {
 	int		i;
 /*
@@ -213,16 +192,16 @@ struct kproc		kptable[];
  * Truncate file to new length.
  */
 #ifdef SCO
-	if (chsize(fd_kill, ftell(fp_kill))) {
+	if (chsize(fd_kill, ftell(fp_kill)))
 #else
-	if (ftruncate(fd_kill, ftell(fp_kill))) {
+	if (ftruncate(fd_kill, ftell(fp_kill)))
 #endif
-		return(LAMERROR);
-	}
+	  return(LAMERROR);
 
 	return(0);
 }
 
+
 /*
  *	kkillclose
  *
@@ -230,8 +209,7 @@ struct kproc		kptable[];
  *	Returns:	- 0 or LAMERROR
  */
 int
-kkillclose()
-
+kkillclose(void)
 {
 	if (fclose(fp_kill) == EOF) {
 		return(LAMERROR);
