@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: asc_run.c,v 6.16 2003/05/31 22:28:49 jsquyres Exp $
+ * $Id: asc_run.c,v 6.17 2003/11/04 04:00:49 pkambadu Exp $
  *
  *	Function:	- application execution
  */
@@ -214,9 +214,7 @@ asc_run(LIST *appd, int nparent, int4 rtf, int verbose, int prerror,
     niod = 0;
     if (rtf & RTF_IO) {
 	for (pp = al_top(appd); pp; pp = al_next(appd, pp)) {
-	    if (pp->asc_node != LOCAL && pp->asc_node != getnodeid()) {
-		niod++;
-	    }
+	  niod++;
 	}
     }
 /*
@@ -233,12 +231,7 @@ asc_run(LIST *appd, int nparent, int4 rtf, int verbose, int prerror,
  * is suppported locally.
  */
 	remote = (pp->asc_node != LOCAL && pp->asc_node != getnodeid());
-
-#if LAM_HAVE_FD_PASSING
-	pfdio = ((rtf & RTF_IO) && !remote) ? RTF_PFDIO : 0;
-#else
 	pfdio = 0;
-#endif
 /*
  * Start the process and store its pid.
  */
@@ -271,8 +264,8 @@ asc_run(LIST *appd, int nparent, int4 rtf, int verbose, int prerror,
 	pgps[i].gps_idx = idx;
 	pgps[i].gps_grank = i;
 
-	if ((rtf & RTF_IO) && remote) {
-	    niod--;
+	if (rtf & RTF_IO) {
+	  niod--;
 	}
 
 	if (verbose) {
