@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: bhostparse.c,v 6.21.2.1 2004/01/01 23:59:16 jsquyres Exp $
+ * $Id: bhostparse.c,v 6.22 2004/01/01 23:59:08 jsquyres Exp $
  * 
  *	Function:	- vanilla boot schema parser
  *			- host file syntax version
@@ -452,7 +452,7 @@ bhostparseline(char *linebuf, int lineno, char *linedata, LIST *list)
     quotes = 0;
 
     /* If *p is whitespace, just continue. */
-    if (isspace(*p)) {
+    if (isspace((int) *p)) {
       q = p + 1;
       continue;
     }
@@ -483,7 +483,7 @@ bhostparseline(char *linebuf, int lineno, char *linedata, LIST *list)
     }
     else {
       token = p;
-      for (q = p; *q && !isspace(*q); ++q)
+      for (q = p; *q && !isspace((int) *q); ++q)
 	continue;
 
       if (*q != '\0') {
@@ -533,7 +533,7 @@ bhostparseline(char *linebuf, int lineno, char *linedata, LIST *list)
 	/* not a valid key-value pair */
 	show_help("bhost-parse", "invalid-key", lineno_str, linedata, NULL);
 	return;
-      } else if (isspace(*(equals - 1)) || isspace(*(equals + 1))) {
+      } else if (isspace((int) *(equals - 1)) || isspace((int) *(equals + 1))) {
 	/* not a valid key-value pair, cannot take whitespace before or after 
 	   the equals sign -- this can happen only when enclosed in quotes */
 	show_help("bhost-parse", "invalid-key", lineno_str, linedata, NULL);
