@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: lamnet.c,v 6.12.2.1 2004/02/16 22:26:14 jsquyres Exp $
+ * $Id: lamnet.c,v 6.13 2004/02/16 22:20:52 jsquyres Exp $
  * 
  *	Function:	- LAM network functions
  */
@@ -190,8 +190,23 @@ getifaddr(uint4 **ifaddr)
 /*
  * Get network interface configuration.
  */
+        /*
+         * Some notes on the behavior of ioctl(..., SIOCGIFCONF,...)
+         * when not enough space is allocated for all the entries.
+         *
+         * - Solaris returns -1, errno EINVAL if there is not enough
+         *   space 
+         * - OS X returns 0, sets .ifc_len to the space used by the
+         *   by the entries that did fit.
+         * - Linux returns 0, sets .ifc_len to the space required to
+         *   hold all the entries (although it only writes what will
+         *   fit in the buffer of .ifc_len passed to the function).
+         * - FreeBSD returns 0, sets .ifc_len to 0.
+         *
+         * Everyone else seems to do one of the four.
+         */
         lastlen = 0;
-        ifc_len = sizeof(req) * 2;
+        ifc_len = sizeof(req) * 10;
         do {
           config.ifc_len = ifc_len;
           config.ifc_req = malloc(ifc_len);
@@ -200,19 +215,24 @@ getifaddr(uint4 **ifaddr)
             return LAMERROR;
           }
           if (ioctl(sock, SIOCGIFCONF, &config) < 0) {
+            /* if we got an einval, we probably don't have enough
+               space.  so we'll fall down and try to expand our
+               space */
 	    if (errno != EINVAL && lastlen != 0) {
 	      close(sock);
 	      return LAMERROR;
 	    } 
           } else {
-	    if (config.ifc_len == lastlen) {
+            /* if config.ifc_len is 0, we obviously aren't done.  try
+               again... */
+	    if (config.ifc_len == lastlen && config.ifc_len > 0) {
 	      /* we didn't expand.  we're done */
 	      break;
 	    }
 	    lastlen = config.ifc_len;
 	  }
 
-          /* Yes, we overflowed (or had an EINVAL on the ioctl.
+          /* Yes, we overflowed (or had an EINVAL on the ioctl).
           .  Loop back around and try again */
           free(config.ifc_req);
           ifc_len *= 2;
