@@ -12,9 +12,9 @@
  * 
  * $HEADER$
  *
- * $Id: kill.c,v 6.34.2.3 2004/03/12 13:22:50 jsquyres Exp $
+ *  $Id: kill.c,v 6.41 2004/03/12 13:21:49 jsquyres Exp $
  *
- *	Function:	- kill file useful routines
+ *  Function: - kill file useful routines
  */
 
 #include <stdio.h>
@@ -25,7 +25,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
-#include <netdb.h>			/* MAXHOSTNAMELEN in Solaris */
+#include <netdb.h>      /* MAXHOSTNAMELEN in Solaris */
 #include <unistd.h>
 #include <errno.h>
 #include <dirent.h>
@@ -56,12 +56,12 @@ static char lam_tmpdir_prefix[MAXPATHLEN];
 
 
 /*
- *	lam_tmpdir_init
+ *  lam_tmpdir_init
  *
- *	Function:	- set up tmpdir for LAM to store stuff
- *			- creates directory, with "good" perms
- *	Accepts:	- suffix to use
- *	Returns:	- 0 or LAMERROR
+ *  Function: - set up tmpdir for LAM to store stuff
+ *      - creates directory, with "good" perms
+ *  Accepts:  - suffix to use
+ *  Returns:  - 0 or LAMERROR
  *      Notes:          - see lam_tmpdir_init()
  *                      - see lam_tmpdir_setsuffix()
  *                      - directory in form of:
@@ -81,7 +81,7 @@ lam_tmpdir_init_opt(char *prefix, char *suffix, int create)
   char *tmp;
   char host[MAXHOSTNAMELEN + 1];
   char *user;
-  struct passwd	*pwdent;
+  struct passwd *pwdent;
   size_t len;
 
   /* We only do this once.  see, if we do it multiple times, then we
@@ -96,7 +96,7 @@ lam_tmpdir_init_opt(char *prefix, char *suffix, int create)
 
   /* In case something goes wrong... */
   snprintf(lam_tmpdir, sizeof(lam_tmpdir), "%s/lam-failure@failure",
-	   LAM_DEFAULT_TMPDIR);
+     LAM_DEFAULT_TMPDIR);
 
   /* set up the prefix */
   if (prefix != NULL) {
@@ -159,10 +159,10 @@ lam_tmpdir_init_opt(char *prefix, char *suffix, int create)
   /* we now have a beginning, middle, and end - set everything up. */
   if (tmpsuffix == NULL) {
     snprintf(lam_tmpdir, sizeof(lam_tmpdir), "%s/lam-%s@%s",
-	     tmpprefix, user, host);
+       tmpprefix, user, host);
   } else {
     snprintf(lam_tmpdir, sizeof(lam_tmpdir), "%s/lam-%s@%s-%s",
-	     tmpprefix, user, host, tmpsuffix);
+       tmpprefix, user, host, tmpsuffix);
   }
 
   /* make the freaking directory (or check perms or whatever) */
@@ -177,14 +177,14 @@ lam_tmpdir_init_opt(char *prefix, char *suffix, int create)
   /* set up the storage for the get_{suffix,prefix} */
   if (tmpprefix != NULL) {
     snprintf(lam_tmpdir_prefix, sizeof(lam_tmpdir_prefix), 
-	     "%s", tmpprefix);
+             "%s", tmpprefix);
   } else {
     lam_tmpdir_prefix[0] = '\0';
   }
 
   if (tmpsuffix != NULL) {
     snprintf(lam_tmpdir_suffix, sizeof(lam_tmpdir_suffix),
-	     "%s", tmpsuffix);
+             "%s", tmpsuffix);
   } else {
     lam_tmpdir_suffix[0] = '\0';
   }
@@ -204,8 +204,8 @@ lam_tmpdir_init_opt(char *prefix, char *suffix, int create)
 /*
  * lam_get_tmpdir
  *
- *	Function:	- get path to internal LAM tmpdir
- *	Returns:	- NULL or pathname (must be freed by caller)
+ *  Function: - get path to internal LAM tmpdir
+ *  Returns:  - NULL or pathname (must be freed by caller)
  */
 char*
 lam_get_tmpdir(void)
@@ -225,8 +225,8 @@ lam_get_tmpdir(void)
 /*
  * lam_get_tmpdir_prefix
  *
- *	Function:	- 
- *	Returns:	- NULL or pathname (must be freed by caller)
+ *  Function: - 
+ *  Returns:  - NULL or pathname (must be freed by caller)
  */
 char*
 lam_get_tmpdir_prefix(void)
@@ -249,8 +249,8 @@ lam_get_tmpdir_prefix(void)
 /*
  * lam_get_tmpdir_suffix
  *
- *	Function:	- 
- *	Returns:	- NULL or pathname (must be freed by caller)
+ *  Function: - 
+ *  Returns:  - NULL or pathname (must be freed by caller)
  */
 char*
 lam_get_tmpdir_suffix(void)
@@ -273,8 +273,8 @@ lam_get_tmpdir_suffix(void)
 /*
  * lam_get_killfile
  *
- *	Function:	- get full path name to killfile
- *	Returns:	- NULL or filename (must be freed by caller)
+ *  Function: - get full path name to killfile
+ *  Returns:  - NULL or filename (must be freed by caller)
  */
 char*
 lam_get_killfile(void)
@@ -295,8 +295,8 @@ lam_get_killfile(void)
 /*
  * lam_get_sockname
  *
- *	Function:	- get full path name to kernel socket name
- *	Returns:	- NULL or filename (must be freed by caller)
+ *  Function: - get full path name to kernel socket name
+ *  Returns:  - NULL or filename (must be freed by caller)
  */
 char*
 lam_get_sockname(void)
@@ -316,8 +316,8 @@ lam_get_sockname(void)
 /*
  * lam_get_iosockname
  *
- *	Function:	- get full path name to I/O socket name
- *	Returns:	- NULL or filename (must be freed by caller)
+ *  Function: - get full path name to I/O socket name
+ *  Returns:  - NULL or filename (must be freed by caller)
  */
 char*
 lam_get_iosockname(void)
@@ -335,14 +335,14 @@ lam_get_iosockname(void)
 
 
 /*
- *	lam_mktmpid
+ *  lam_mktmpid
  *
- *	Function:	- create temporary file name based on an id
- *			- returns error if insufficient space
- *	Accepts:	- id number to make name unique
- *			- temporary file name (out)
- *			- size of storage for temporary name
- *	Returns:	- 0 or LAMERROR
+ *  Function: - create temporary file name based on an id
+ *      - returns error if insufficient space
+ *  Accepts:  - id number to make name unique
+ *      - temporary file name (out)
+ *      - size of storage for temporary name
+ *  Returns:  - 0 or LAMERROR
  */
 int
 lam_mktmpid(int id, char *name, unint n)
@@ -355,7 +355,7 @@ lam_mktmpid(int id, char *name, unint n)
   }
 
   snprintf(name, n, "%s/lam-%d", lam_tmpdir, id);
-	
+  
   return 0;
 }
 
@@ -375,12 +375,12 @@ lam_tmpdir_remove(void)
     ret = chdir(lam_tmpdir);
     if (ret == -1) {
       if (errno == EINTR)
-	continue;
+        continue;
       else if (errno == ENOENT) {
-	return;
+        return;
       } else {
-	show_help(NULL, "unable-rmdir", "tkill", lam_tmpdir, NULL);
-	return;
+        show_help(NULL, "unable-rmdir", "tkill", lam_tmpdir, NULL);
+        return;
       }
     }
   } while (ret != 0);
@@ -394,16 +394,16 @@ lam_tmpdir_remove(void)
   badness = 0;
   while ((entry = readdir(dp)) != NULL) {
     if (strcmp(entry->d_name, ".") == 0 ||
-	strcmp(entry->d_name, "..") == 0)
+        strcmp(entry->d_name, "..") == 0)
       continue;
 
     do {
       ret = remove(entry->d_name);
       if (ret == -1) {
-	if (errno == EINTR)
-	  continue;
-	else
-	  badness++;
+        if (errno == EINTR)
+          continue;
+        else
+          badness++;
       }
     } while (ret != 0);
   }
@@ -420,10 +420,10 @@ lam_tmpdir_remove(void)
     ret = chdir("..");
     if (ret == -1) {
       if (errno == EINTR)
-	continue;
+        continue;
       else {
-	show_help(NULL, "unable-rmdir", "tkill", lam_tmpdir, NULL);
-	return;
+        show_help(NULL, "unable-rmdir", "tkill", lam_tmpdir, NULL);
+        return;
       }
     }
   } while (ret != 0);
@@ -433,10 +433,10 @@ lam_tmpdir_remove(void)
     ret = rmdir(lam_tmpdir);
     if (ret == -1) {
       if (errno == EINTR)
-	continue;
+        continue;
       else {
-	show_help(NULL, "unable-rmdir", "tkill", lam_tmpdir, NULL);
-	return;
+        show_help(NULL, "unable-rmdir", "tkill", lam_tmpdir, NULL);
+        return;
       }
     }
   } while (ret != 0);
@@ -458,11 +458,11 @@ check_dir_perms(void)
     statret = stat(lam_tmpdir, &s);
     if (statret == -1) {
       if (errno == EINTR)
-	continue;
+        continue;
       else if (errno == ENOENT)
-	break;
+        break;
       else if (errno == EACCES)
-	return (LAMERROR);
+        return (LAMERROR);
     }
   } while (statret != 0);
 
@@ -481,33 +481,33 @@ check_dir_perms(void)
 
       mode = (S_IRWXU | S_IRWXG | S_IRWXO);
       if ((s.st_mode & mode) != S_IRWXU)
-	baddir = 1;
+        baddir = 1;
 
       /* Check owner */
 
       if (s.st_uid != getuid())
-	baddir = 1;
+        baddir = 1;
     }
 
     /* If we need to remove something, do it */
 
     if (badfile == 1 || baddir == 1) {
       do {
-	if (badfile == 1)
-	  ret = unlink(lam_tmpdir);
-	else
-	  ret = rmdir(lam_tmpdir);
+        if (badfile == 1)
+          ret = unlink(lam_tmpdir);
+        else
+          ret = rmdir(lam_tmpdir);
 
-	if (ret == -1) {
-	  if (errno == EINTR)
-	    continue;
-	  else if (errno == ENOENT)
-	    /* shouldn't happen, but... */
-	    break;
-	  else
-	    /* If we couldn't remove it, badness */
-	    return (LAMERROR);
-	}
+        if (ret == -1) {
+          if (errno == EINTR)
+            continue;
+          else if (errno == ENOENT)
+            /* shouldn't happen, but... */
+            break;
+          else
+            /* If we couldn't remove it, badness */
+            return (LAMERROR);
+        }
       } while (ret != 0);
     } else
       /* Otherwise, we have goodness */
@@ -517,6 +517,8 @@ check_dir_perms(void)
   /* Make the directory */
   mode = umask(077);
   ret = mkdir(lam_tmpdir, 0700);
+  if (ret != 0)
+    perror("mkdir");
   umask(mode);
 
   if (ret == 0)
@@ -527,16 +529,16 @@ check_dir_perms(void)
 
 
 /*
- *	lam_get_batchid
+ *  lam_get_batchid
  *
- *	Function:	- get batch job ID
- *			- only checks for PBS & LSF at the moment
- *			- this prevents LAM from using the same socket
- *			  name on multiple jobs in a batch environment,
- *			  where the scheduler may put multiple LAM/MPI
- *			  jobs on the same host
+ *  Function: - get batch job ID
+ *      - only checks for PBS, LSF, SGE, and SLURM at the moment
+ *      - this prevents LAM from using the same socket
+ *        name on multiple jobs in a batch environment,
+ *        where the scheduler may put multiple LAM/MPI
+ *        jobs on the same host
  *
- *	Returns:	- job ID in a string (which must be free()ed) or
+ *  Returns:  - job ID in a string (which must be free()ed) or
  *                        NULL
  */
 static char*
@@ -555,9 +557,16 @@ lam_get_batchid(void)
     snprintf(retstr, retstrlen, "pbs-%s", tmp);
   } 
 
+  /* Are we running under a SLURM job? */
+  else if ((tmp = getenv("SLURM_JOBID")) != NULL) {
+    retstrlen = strlen(tmp) + strlen("slurm-") + 2;
+    retstr = malloc(retstrlen);
+    snprintf(retstr, retstrlen, "slurm-%s", tmp);
+  }
+
   /* Are we running under an LSF job? */
   else if ((tmp = getenv("LSB_JOBID")) != NULL && 
-	   (tmp2 = getenv("LSB_JOBINDEX")) != NULL) {
+           (tmp2 = getenv("LSB_JOBINDEX")) != NULL) {
     retstrlen = strlen(tmp) + strlen(tmp2) + strlen("lsf--") + 2;
     retstr = malloc(retstrlen);
     snprintf(retstr, retstrlen, "lsf-%s-%s", tmp, tmp2);
@@ -579,3 +588,83 @@ lam_get_batchid(void)
 
   return retstr;
 }
+
+
+
+/*
+ *  lam_mktmpid_stdin
+ *
+ *  Function: - create temporary file name based on an id
+ *      - returns error if insufficient space
+ *  Accepts:  - id number to make name unique
+ *      - temporary file name (out)
+ *      - size of storage for temporary name
+ *  Returns:  - 0 or LAMERROR
+ */
+int
+lam_mktmpid_stdin(int id, char *name, unint n)
+{
+  if (check_dir_perms() == LAMERROR) {
+    /* try never to leave a directory around... */
+    lam_tmpdir_remove();
+    memset(name, 0, n);
+    return LAMERROR;
+  }
+
+  snprintf(name, n, "%s/lam-%d-stdin", lam_tmpdir, id);
+  
+  return 0;
+}
+
+
+/*
+ *  lam_mktmpid_stdout
+ *
+ *  Function: - create temporary file name based on an id
+ *      - returns error if insufficient space
+ *  Accepts:  - id number to make name unique
+ *      - temporary file name (out)
+ *      - size of storage for temporary name
+ *  Returns:  - 0 or LAMERROR
+ */
+int
+lam_mktmpid_stdout(int id, char *name, unint n)
+{
+  if (check_dir_perms() == LAMERROR) {
+    /* try never to leave a directory around... */
+    lam_tmpdir_remove();
+    memset(name, 0, n);
+    return LAMERROR;
+  }
+
+  snprintf(name, n, "%s/lam-%d-stdout", lam_tmpdir, id);
+  
+  return 0;
+}
+
+
+
+/*
+ *  lam_mktmpid_stderr
+ *
+ *  Function: - create temporary file name based on an id
+ *      - returns error if insufficient space
+ *  Accepts:  - id number to make name unique
+ *      - temporary file name (out)
+ *      - size of storage for temporary name
+ *  Returns:  - 0 or LAMERROR
+ */
+int
+lam_mktmpid_stderr(int id, char *name, unint n)
+{
+  if (check_dir_perms() == LAMERROR) {
+    /* try never to leave a directory around... */
+    lam_tmpdir_remove();
+    memset(name, 0, n);
+    return LAMERROR;
+  }
+
+  snprintf(name, n, "%s/lam-%d-stderr", lam_tmpdir, id);
+  
+  return 0;
+}
