@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: registry.c,v 1.1.2.3 2002/10/09 19:48:52 brbarret Exp $
+ * $Id: registry.c,v 1.14 2003/05/31 19:39:55 brbarret Exp $
  *
  *	Function:	- object registry
  *			- mechanism for registering semi-persistent
@@ -33,63 +31,55 @@
 #else
 #include <varargs.h>
 #endif
-
 #include <errno.h>
 #include <fcntl.h>
+#include <lamdebug.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#if LAM_HAVE_SYSV_SHMEM
 #include <sys/ipc.h>
 #include <sys/shm.h>
+#endif
 #if LAM_HAVE_SYSV_SEMAPHORES
 #include <sys/sem.h>
-#endif
+#endif 
 #include <unistd.h>
 
 #include <terror.h>
 #include <typical.h>
 #include <args.h>
 #include <laminternal.h>
+#include <etc_misc.h>
 
 #ifndef MAXPATHLEN
 #define MAXPATHLEN	1024
 #endif
 
-/*
- * public functions
- */
-#if __STDC__
-void			lam_register_objects(int, ...);
-#else
-void			lam_register_objects();
-#endif
-void			lam_deregister_object();
-void			lam_cleanup_objects();
 
 /*
  * local constants
  */
 #define	FORMBUFLEN	(MAXPATHLEN + 16)	/* size of formatting buffer */
 
-/*
- * external functions
- */
-extern char		*killname();
 
 /*
  * private functions
  */
-static char		*lam_reg_filename();
-static char		*lam_reg_tmpname();
-static char		*lam_reg_lockname();
-static void		clean_warning();
-static void		del_warning();
+static char		*lam_reg_filename(void);
+static char		*lam_reg_tmpname(void);
+static char		*lam_reg_lockname(void);
+static void		clean_warning(char *msg);
+static void		del_warning(char *msg);
 static int              set_names(char *msg);
 static int              lock_registry(char *name);
 static void             unlock_registry(void);
-static int		filelock();
+static int              filelock(int fd, int cmd, int type, off_t offset, 
+				 int whence, off_t len);
+static void             free_filenames(void);
+
 
 /*
  * private variables
@@ -115,7 +105,8 @@ static union {
 } semctl_arg;
 #endif
 
-#endif /* #if LAM_HAVE_SYSV_SEMAPHORES */
+#endif /* LAM_HAVE_SYSV_SEMAPHORES */
+
 
 /*
  *	lam_register_objects
@@ -133,11 +124,9 @@ lam_register_objects(int nobjs, ...)
 #else
 void
 lam_register_objects(nobjs, va_alist)
-
 int			nobjs;
 va_dcl
 #endif
-
 {
 	char		buf[FORMBUFLEN];	/* formatting buffer */
 	char		type;			/* object type */
@@ -156,15 +145,14 @@ va_dcl
 	  return;
 	if (lock_registry("lam_register_objects") == LAMERROR)
 	  return;
-
 /*
  * Open the registry file and add the objects.
  */
 	fd = open(reg_fname, O_WRONLY | O_APPEND | O_CREAT, 0600);
 
 	if (fd < 0) {
-	        unlock_registry();
-		return;
+	  unlock_registry();
+	  return;
 	}
 
 	for (i = 0; i < nobjs; i++) {
@@ -188,6 +176,7 @@ va_dcl
 
 	close(fd);
 	unlock_registry();
+        free_filenames();
 	va_end(arglist);
 }
 
@@ -202,11 +191,7 @@ va_dcl
  *			- name of the object to deregister
  */
 void
-lam_deregister_object(deltype, delobj)
-
-int			deltype;
-char			*delobj;
-
+lam_deregister_object(int deltype, char *delobj)
 {
     struct stat		fstatus;		/* registry file status */
     int			fd;			/* registry file descriptor */
@@ -217,7 +202,6 @@ char			*delobj;
       return;
     if (lock_registry("lam_deregister_object") == LAMERROR)
       return;
-
 /*
  * Open the registry and temporary files.
  */
@@ -227,7 +211,6 @@ char			*delobj;
     }
 
     fd = fileno(f);
-
     if (fstat(fd, &fstatus) == -1) {
 	del_warning("lam_deregister_object (fstat)");
 	return;
@@ -264,6 +247,7 @@ char			*delobj;
     }
 
     unlock_registry();
+    free_filenames();
 }
 
 
@@ -274,8 +258,7 @@ char			*delobj;
  *	Returns:	- 0 or LAMERROR
  */
 void
-lam_cleanup_objects()
-
+lam_cleanup_objects(void)
 {
     int			sysvid;			/* SYSV IPC shared id */
     char		type;			/* shared structure type */
@@ -292,19 +275,18 @@ lam_cleanup_objects()
 	if (errno != ENOENT) {
 	    clean_warning("lam_cleanup_objects (fopen)");
 	}
-	unlock_registry();
-	return;
-    }
+    } else {
+      while (fscanf(f, "%c %s\n", &type, obj) == 2) {
+
+	/* Don't check for errors here as structures may not exist. */
 
-    while (fscanf(f, "%c %s\n", &type, obj) == 2) {
-/*
- * Don't check for errors here as structures may not exist.
- */
 	switch (type) {
+#if LAM_HAVE_SYSV_SHMEM
 	case 'm':
 	    sysvid = atoi(obj);
 	    shmctl(sysvid, IPC_RMID, (struct shmid_ds *) 0);
 	    break;
+#endif
 #if LAM_HAVE_SYSV_SEMAPHORES
 	case 's':
 	    sysvid = atoi(obj);
@@ -317,26 +299,16 @@ lam_cleanup_objects()
 	default:
 	    break;
 	}
+      }
+
+      fclose(f);
+      f = 0;
+
+      unlink(reg_fname);
     }
 
-    fclose(f);
-    f = 0;
-
-    unlink(reg_fname);
     unlock_registry();
-
-    if (reg_fname) {
-	free(reg_fname);
-	reg_fname = 0;
-    }
-    if (reg_lockname) {
-	free(reg_lockname);
-	reg_lockname = 0;
-    }
-    if (reg_tmpname) {
-	free(reg_tmpname);
-	reg_tmpname = 0;
-    }
+    free_filenames();
 }
 
 
@@ -348,15 +320,14 @@ lam_cleanup_objects()
  *	Returns:	- registry filename
  */
 static char *
-lam_reg_filename()
-
+lam_reg_filename(void)
 {
     	char		*fname;
 	
-	fname = killname();
+	fname = lam_get_tmpdir();
 	if (fname == 0) return(0);
 
-	strcat(fname, "-reg");
+	strcat(fname, "/lam-registry");
 	return(fname);
 }
 
@@ -368,15 +339,14 @@ lam_reg_filename()
  *	Returns:	- temporary registry filename
  */
 static char *
-lam_reg_tmpname()
-
+lam_reg_tmpname(void)
 {
     	char		*fname;
 
-	fname = killname();
+	fname = lam_get_tmpdir();
 	if (fname == 0) return(0);
 
-	strcat(fname, "-tmp");
+	strcat(fname, "/lam-registry-tmp");
 	return(fname);
 }
 
@@ -389,15 +359,14 @@ lam_reg_tmpname()
  *	Returns:	- registry lock filename
  */
 static char *
-lam_reg_lockname()
-
+lam_reg_lockname(void)
 {
     	char		*fname;
 
-	fname = killname();
+	fname = lam_get_tmpdir();
 	if (fname == 0) return(0);
 
-	strcat(fname, "-registry-lock");
+	strcat(fname, "/lam-registry-lock");
 	return(fname);
 }
 
@@ -411,10 +380,7 @@ lam_reg_lockname()
  *	Accepts:	- message
  */
 static void
-clean_warning(msg)
-
-char			*msg;
-
+clean_warning(char *msg)
 {
 	char		errmsg[256];		/* error message */
 
@@ -450,18 +416,18 @@ char			*msg;
  *	Accepts:	- message
  */
 static void
-del_warning(msg)
-
-char			*msg;
-
+del_warning(char *msg)
 {
     char		errmsg[128];		/* error message */
 
     lam_errorstr(errmsg, sizeof(errmsg));
     if (errno) {
 	fprintf(stderr,
-	    "%s: Warning: %s: deleting shared memory id's from registry.\n",
-	    msg, errmsg);
+		"%s: Warning: %s: deleting shared memory id's from registry.\n",
+		msg, errmsg);
+	fprintf(stderr,
+		"%s: Warning: \"%s\"\n",
+		msg, reg_fname);
     } else {
 	fprintf(stderr, "%s\n", msg);
     }
@@ -519,7 +485,7 @@ set_names(char *name)
 
     reg_mylockname = malloc(MAXPATHLEN);
     snprintf(reg_mylockname, MAXPATHLEN - 1, "%s.%d", 
-	     reg_lockname, getpid());
+	     reg_lockname, lam_getpid());
     fd = open(reg_mylockname, O_RDWR | O_CREAT | O_EXCL, 0600);
     if (fd == -1) {
       oexcl_works = 0;
@@ -561,9 +527,6 @@ lock_registry(char *name)
   if (oexcl_works == 1) {
     lockfd = open(reg_lockname, O_RDWR | O_CREAT | O_EXCL, 0600);
     
-    if (lockfd == -1 && errno == EEXIST)
-      lockfd = open(reg_lockname, O_RDWR, 0600);
-
     if (lockfd == -1) {
       return LAMERROR;
     }
@@ -648,15 +611,7 @@ unlock_registry(void)
  *	Returns:	- fcntl return value
  */
 static int
-filelock(fd, cmd, type, offset, whence, len)
-
-int			fd;
-int			cmd;
-int			type;
-off_t			offset;
-int			whence;
-off_t			len;
-
+filelock(int fd, int cmd, int type, off_t offset, int whence, off_t len)
 {
 	struct flock	lock;
 
@@ -667,3 +622,34 @@ off_t			len;
 
 	return(fcntl(fd, cmd, &lock));
 }
+
+
+/*
+ *	free_filenames
+ *
+ *	Function:	- free the filenames allocated by the session 
+ *                        directory API
+ *	Accepts:	- void
+ *	Returns:	- void
+ */
+static void
+free_filenames(void)
+{
+
+    if (reg_fname) {
+	free(reg_fname);
+	reg_fname = 0;
+    }
+    if (reg_lockname) {
+	free(reg_lockname);
+	reg_lockname = 0;
+    }
+    if (reg_mylockname) {
+	free(reg_mylockname);
+	reg_mylockname = 0;
+    }
+    if (reg_tmpname) {
+	free(reg_tmpname);
+	reg_tmpname = 0;
+    }
+}
