@@ -10,11 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1996 The Ohio State University
  *	GDB
  *
- *	$Id: rfstate.c,v 6.5.2.1 2002/10/09 19:48:55 brbarret Exp $
+ *	$Id: rfstate.c,v 6.7 2002/10/09 20:57:14 brbarret Exp $
  * 
  *	Function:	- gathers state from a remote filed
  *			- receives file descriptors from
@@ -37,6 +38,7 @@
 #include <terror.h>
 #include <typical.h>
 #include <t_types.h>
+#include <etc_misc.h>
 
 /*
  * local variables
@@ -62,7 +64,7 @@ lam_rfstate(int nodeid, struct fstate ftbl[], int maxcount)
 	reply = (struct freply *) nhead.nh_data;
 
 	request->fq_src_node = (nodeid == LOCAL) ? nodeid : getnodeid();
-	request->fq_src_event = -getpid();
+	request->fq_src_event = -lam_getpid();
 	request->fq_req = FQSTATE;
 
 	nhead.nh_node = nodeid;
@@ -79,7 +81,7 @@ lam_rfstate(int nodeid, struct fstate ftbl[], int maxcount)
 		return(LAMERROR);
 	}
 
-	nhead.nh_event = -getpid();
+	nhead.nh_event = -lam_getpid();
 	nhead.nh_msg = hole;
 
 	do {
