@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- *	$Id: MPISYSF.h,v 6.12.2.1 2003/11/06 18:22:22 jsquyres Exp $
+ *	$Id: MPISYSF.h,v 6.14 2003/11/06 18:22:48 jsquyres Exp $
  *
  *	Function:	- F77 wrappers for MPI library
  */
@@ -47,6 +47,7 @@
 #define mpi_allreduce_			pmpi_allreduce
 #define mpi_alltoall_			pmpi_alltoall
 #define mpi_alltoallv_			pmpi_alltoallv
+#define mpi_alltoallw_                  pmpi_alltoallw
 #define mpi_attr_delete_		pmpi_attr_delete
 #define mpi_attr_get_			pmpi_attr_get
 #define mpi_attr_put_			pmpi_attr_put
@@ -103,6 +104,7 @@
 #define mpi_errhandler_set_		pmpi_errhandler_set
 #define mpi_error_class_		pmpi_error_class
 #define mpi_error_string_		pmpi_error_string
+#define mpi_exscan                      pmpi_exscan
 #define mpi_free_mem_			pmpi_free_mem
 #define mpi_finalize_			pmpi_finalize
 #define mpi_finalized_			pmpi_finalized
@@ -267,6 +269,7 @@
 #define mpi_allreduce_			mpi_allreduce
 #define mpi_alltoall_			mpi_alltoall
 #define mpi_alltoallv_			mpi_alltoallv
+#define mpi_alltoallw_                  mpi_alltoallw         
 #define mpi_attr_delete_		mpi_attr_delete
 #define mpi_attr_get_			mpi_attr_get
 #define mpi_attr_put_			mpi_attr_put
@@ -323,6 +326,7 @@
 #define mpi_errhandler_set_		mpi_errhandler_set
 #define mpi_error_class_		mpi_error_class
 #define mpi_error_string_		mpi_error_string
+#define mpi_exscan_                     mpi_exscan
 #define mpi_free_mem_			mpi_free_mem
 #define mpi_finalize_			mpi_finalize
 #define mpi_finalized_			mpi_finalized
@@ -522,6 +526,7 @@
 #define mpi_allreduce_			pmpi_allreduce__
 #define mpi_alltoall_			pmpi_alltoall__
 #define mpi_alltoallv_			pmpi_alltoallv__
+#define mpi_alltoallw_                  pmpi_alltoallw__
 #define mpi_attr_delete_		pmpi_attr_delete__
 #define mpi_attr_get_			pmpi_attr_get__
 #define mpi_attr_put_			pmpi_attr_put__
@@ -578,6 +583,7 @@
 #define mpi_errhandler_set_		pmpi_errhandler_set__
 #define mpi_error_class_		pmpi_error_class__
 #define mpi_error_string_		pmpi_error_string__
+#define mpi_exscan_                     pmpi_exscan__  
 #define mpi_free_mem_			pmpi_free_mem__
 #define mpi_finalize_			pmpi_finalize__
 #define mpi_finalized_			pmpi_finalized__
@@ -745,6 +751,7 @@
 #define mpi_allreduce_			mpi_allreduce__
 #define mpi_alltoall_			mpi_alltoall__
 #define mpi_alltoallv_			mpi_alltoallv__
+#define mpi_alltoallw_                  mpi_alltoallw__
 #define mpi_attr_delete_		mpi_attr_delete__
 #define mpi_attr_get_			mpi_attr_get__
 #define mpi_attr_put_			mpi_attr_put__
@@ -801,6 +808,7 @@
 #define mpi_errhandler_set_		mpi_errhandler_set__
 #define mpi_error_class_		mpi_error_class__
 #define mpi_error_string_		mpi_error_string__
+#define mpi_exscan_                     mpi_exscan__
 #define mpi_free_mem_			mpi_free_mem__
 #define mpi_finalize_			mpi_finalize__
 #define mpi_finalized_			mpi_finalized__
@@ -1000,6 +1008,7 @@
 #define mpi_allreduce_			PMPI_ALLREDUCE
 #define mpi_alltoall_			PMPI_ALLTOALL
 #define mpi_alltoallv_			PMPI_ALLTOALLV
+#define mpi_alltoallw_                  PMPI_ALLTOALLW
 #define mpi_attr_delete_		PMPI_ATTR_DELETE
 #define mpi_attr_get_			PMPI_ATTR_GET
 #define mpi_attr_put_			PMPI_ATTR_PUT
@@ -1056,6 +1065,7 @@
 #define mpi_errhandler_set_		PMPI_ERRHANDLER_SET
 #define mpi_error_class_		PMPI_ERROR_CLASS
 #define mpi_error_string_		PMPI_ERROR_STRING
+#define mpi_exscan_                     PMPI_EXSCAN
 #define mpi_free_mem_			PMPI_FREE_MEM
 #define mpi_finalize_			PMPI_FINALIZE
 #define mpi_finalized_			PMPI_FINALIZED
@@ -1223,6 +1233,7 @@
 #define mpi_allreduce_			MPI_ALLREDUCE
 #define mpi_alltoall_			MPI_ALLTOALL
 #define mpi_alltoallv_			MPI_ALLTOALLV
+#define mpi_alltoallw_                  MPI_ALLTOALLW
 #define mpi_attr_delete_		MPI_ATTR_DELETE
 #define mpi_attr_get_			MPI_ATTR_GET
 #define mpi_attr_put_			MPI_ATTR_PUT
@@ -1279,6 +1290,7 @@
 #define mpi_errhandler_set_		MPI_ERRHANDLER_SET
 #define mpi_error_class_		MPI_ERROR_CLASS
 #define mpi_error_string_		MPI_ERROR_STRING
+#define mpi_exscan_                     MPI_EXSCAN 
 #define mpi_free_mem_			MPI_FREE_MEM
 #define mpi_finalize_			MPI_FINALIZE
 #define mpi_finalized_			MPI_FINALIZED
@@ -1510,6 +1522,7 @@
 #define mpi_allreduce_			pmpi_allreduce_
 #define mpi_alltoall_			pmpi_alltoall_
 #define mpi_alltoallv_			pmpi_alltoallv_
+#define mpi_alltoallw_                  pmpi_alltoallw_
 #define mpi_attr_delete_		pmpi_attr_delete_
 #define mpi_attr_get_			pmpi_attr_get_
 #define mpi_attr_put_			pmpi_attr_put_
@@ -1566,6 +1579,7 @@
 #define mpi_errhandler_set_		pmpi_errhandler_set_
 #define mpi_error_class_		pmpi_error_class_
 #define mpi_error_string_		pmpi_error_string_
+#define mpi_exscan_                     pmpi_exscan_
 #define mpi_free_mem_			pmpi_free_mem_
 #define mpi_finalize_			pmpi_finalize_
 #define mpi_finalized_			pmpi_finalized_
@@ -1733,6 +1747,7 @@
 #define mpi_allreduce_			mpi_allreduce_
 #define mpi_alltoall_			mpi_alltoall_
 #define mpi_alltoallv_			mpi_alltoallv_
+#define mpi_alltoallw_                  mpi_alltoallw_
 #define mpi_attr_delete_		mpi_attr_delete_
 #define mpi_attr_get_			mpi_attr_get_
 #define mpi_attr_put_			mpi_attr_put_
@@ -1789,6 +1804,7 @@
 #define mpi_errhandler_set_		mpi_errhandler_set_
 #define mpi_error_class_		mpi_error_class_
 #define mpi_error_string_		mpi_error_string_
+#define mpi_exscan_                     mpi_exscan_
 #define mpi_free_mem_			mpi_free_mem_
 #define mpi_finalize_			mpi_finalize_
 #define mpi_finalized_			mpi_finalized_
