@@ -10,8 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- *      $Id: hreq.h,v 1.2.2.1 2002/10/09 19:48:59 brbarret Exp $
+ *      $Id: hreq.h,v 1.4 2002/12/05 01:53:49 jsquyres Exp $
  *	Function:	- defines constants and structures for use
  *			  with the halt daemon
  */
@@ -24,8 +25,9 @@
 #include <net.h>
 
 struct hreq {
-	int4		hq_node;	/* source node */
-	int4		hq_event;	/* ack/reply event */
+  int4 hq_node;  /* source node */
+  int4 hq_event; /* ack/reply event */
+  int4 hq_index; /* reply index */
 };
 
 typedef enum {
@@ -33,4 +35,12 @@ typedef enum {
   LAM_HALT_DIE
 } halt_types;
 
+
+/*
+ * Function to shut down a set of LAM daemons
+ */
+
+int rhreq(int *nodes, int nnodes, int timeout, int verbose);
+
+
 #endif
