@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- *	$Id: mpi.h,v 6.37.2.1 2003/08/21 15:33:43 brbarret Exp $
+ *	$Id: mpi.h,v 6.41 2004/03/09 16:49:39 adharurk Exp $
  *
  *	Function:	- MPI header file
  */
@@ -98,6 +98,7 @@ typedef int	(MPI_Win_delete_attr_function)();
 #define MPI_ANY_SOURCE		-1		/* match any source rank */
 #define MPI_PROC_NULL		-2		/* rank of null process */
 #define MPI_CANCEL_SOURCE	-3		/* successful cancel */
+#define MPI_ROOT                -4
 #define MPI_ANY_TAG		-1		/* match any message tag */
 #define MPI_GER_TAG		-2		/* used for GER protocol */
 #define MPI_MERGE_TAG		-3		/* used to merge inter-comm */
@@ -144,6 +145,8 @@ typedef int	(MPI_Win_delete_attr_function)();
 #define LAM_MPI_SSI_COLL_CROSSOVER          16
 #define LAM_MPI_SSI_COLL_ASSOCIATIVE        17
 #define LAM_MPI_SSI_COLL_REDUCE_CROSSOVER   18
+#define LAM_MPI_SSI_COLL_SHMEM_MESSAGE_POOL_SIZE 19
+#define LAM_MPI_SSI_COLL_SHMEM_NUM_SEGMENTS 20
 
 
 #define MPI_BOTTOM		((void *) 0)	/* base reference address */
@@ -329,6 +332,9 @@ extern struct _dtype		lam_mpi_float_int, lam_mpi_double_int;
 extern struct _dtype		lam_mpi_long_int, lam_mpi_2int;
 extern struct _dtype		lam_mpi_short_int, lam_mpi_dblcplex;
 extern struct _dtype		lam_mpi_integer, lam_mpi_real;
+extern struct _dtype            lam_mpi_integer1, lam_mpi_integer2;
+extern struct _dtype            lam_mpi_integer4, lam_mpi_integer8;
+extern struct _dtype            lam_mpi_real4, lam_mpi_real8, lam_mpi_real16;
 extern struct _dtype		lam_mpi_dblprec, lam_mpi_character;
 extern struct _dtype		lam_mpi_2real, lam_mpi_2dblprec;
 extern struct _dtype		lam_mpi_2integer, lam_mpi_longdbl_int;
@@ -388,6 +394,7 @@ extern MPI_Fint			*MPI_F_STATUSES_IGNORE;
 #define MPI_UB			((MPI_Datatype) &lam_mpi_ub)
 #define MPI_LB			((MPI_Datatype) &lam_mpi_lb)
 #define MPI_WCHAR		((MPI_Datatype) &lam_mpi_wchar)
+#define MPI_LONG_LONG		((MPI_Datatype) &lam_mpi_long_long_int)
 #define MPI_LONG_LONG_INT	((MPI_Datatype) &lam_mpi_long_long_int)
 #define MPI_UNSIGNED_LONG_LONG	((MPI_Datatype) &lam_mpi_unsigned_long_long)
 
@@ -422,6 +429,9 @@ extern int MPI_Alltoall __ARGS((void *, int, MPI_Datatype, void *, int,
 				MPI_Datatype, MPI_Comm));
 extern int MPI_Alltoallv __ARGS((void *, int *, int *, MPI_Datatype, void *, 
 				 int *, int *, MPI_Datatype, MPI_Comm));
+extern int MPI_Alltoallw __ARGS((void *, int *, int *, MPI_Datatype *, void *,
+				 int *, int *, MPI_Datatype *, MPI_Comm));
+
 extern int MPI_Attr_delete __ARGS((MPI_Comm, int));
 extern int MPI_Attr_get __ARGS((MPI_Comm, int, void *, int *));
 extern int MPI_Attr_put __ARGS((MPI_Comm, int, void *));
@@ -491,6 +501,8 @@ extern int MPI_Errhandler_get __ARGS((MPI_Comm, MPI_Errhandler *));
 extern int MPI_Errhandler_set __ARGS((MPI_Comm, MPI_Errhandler));
 extern int MPI_Error_class __ARGS((int, int *));
 extern int MPI_Error_string __ARGS((int, char *, int *));
+extern int MPI_Exscan __ARGS((void *, void *, int, MPI_Datatype, MPI_Op,
+			      MPI_Comm));
 extern int MPI_Finalize __ARGS((void));
 extern int MPI_Finalized __ARGS((int *flag));
 extern int MPI_Free_mem __ARGS((void *base));
@@ -710,7 +722,6 @@ extern double MPI_Wtime __ARGS((void));
 /*
  * profiling prototypes
  */
-
 extern int PMPI_Abort __ARGS((MPI_Comm, int));
 extern int PMPI_Accumulate __ARGS((void *, int, MPI_Datatype, int, MPI_Aint, 
 				   int, MPI_Datatype, MPI_Op, MPI_Win));
@@ -727,6 +738,8 @@ extern int PMPI_Alltoall __ARGS((void *, int, MPI_Datatype, void *, int,
 				 MPI_Datatype, MPI_Comm));
 extern int PMPI_Alltoallv __ARGS((void *, int *, int *, MPI_Datatype, void *, 
 				  int *, int *, MPI_Datatype, MPI_Comm));
+extern int PMPI_Alltoallw __ARGS((void *, int *, int *, MPI_Datatype *, void *,
+				  int *, int *, MPI_Datatype *, MPI_Comm));
 extern int PMPI_Attr_delete __ARGS((MPI_Comm, int));
 extern int PMPI_Attr_get __ARGS((MPI_Comm, int, void *, int *));
 extern int PMPI_Attr_put __ARGS((MPI_Comm, int, void *));
@@ -796,6 +809,8 @@ extern int PMPI_Errhandler_get __ARGS((MPI_Comm, MPI_Errhandler *));
 extern int PMPI_Errhandler_set __ARGS((MPI_Comm, MPI_Errhandler));
 extern int PMPI_Error_class __ARGS((int, int *));
 extern int PMPI_Error_string __ARGS((int, char *, int *));
+extern int PMPI_Exscan __ARGS((void *, void *, int, MPI_Datatype, MPI_Op,
+			       MPI_Comm));
 extern int PMPI_Finalize __ARGS((void));
 extern int PMPI_Finalized __ARGS((int *flag));
 extern int PMPI_Free_mem __ARGS((void *base));
@@ -867,10 +882,10 @@ extern int PMPI_Keyval_create __ARGS((MPI_Copy_function *,
 				      MPI_Delete_function *, int *, void *));
 extern int PMPI_Keyval_free __ARGS((int *));
 extern int PMPI_Lookup_name __ARGS((char *, MPI_Info, char *));
-extern MPI_Fint MPI_Op_c2f __ARGS((MPI_Op op));
+extern MPI_Fint PMPI_Op_c2f __ARGS((MPI_Op op));
 extern int PMPI_Op_create __ARGS((MPI_User_function *, int, MPI_Op *));
 extern int PMPI_Open_port __ARGS((MPI_Info, char *));
-extern MPI_Op MPI_Op_f2c __ARGS((MPI_Fint f_handle));
+extern MPI_Op PMPI_Op_f2c __ARGS((MPI_Fint f_handle));
 extern int PMPI_Op_free __ARGS((MPI_Op *));
 extern int PMPI_Pack __ARGS((void *, int, MPI_Datatype, void *, int, int *, 
 			     MPI_Comm));
@@ -1047,6 +1062,7 @@ extern int XMPI_Coloroff __ARGS((void));
  * backward compatibility
  */
 extern int MPI_Comm_get_contextid __ARGS((MPI_Comm, int *));
+extern int PMPI_Comm_get_contextid __ARGS((MPI_Comm, int *));
 
 
 /*
@@ -1082,4 +1098,13 @@ extern int MPI_Comm_get_contextid __ARGS((MPI_Comm, int *));
 #endif
 #endif
 
+/*
+ * This function should only be used if you are using external memory
+ * management.  Please read the LAM/MPI Installation Guide for more
+ * details.
+ */
+#include <stdlib.h>
+void lam_handle_free(void *buf, size_t length);
+
+
 #endif	/* _MPI_H */
