@@ -141,10 +141,15 @@ kinit(int priority)
 	     mojo that is pd priority assignments. */
 
 	  if (rversion(&version_string, NULL, NULL, NULL, NULL, 
-		       NULL) == LAMERROR || errno == EFAULT) {
-	    show_help(NULL, "version-mismatch", LAM_VERSION, version_string,
-		      NULL);
-	    kexit(1);
+		       NULL) == LAMERROR) {
+            if (errno == EFAULT) {
+              show_help(NULL, "version-mismatch", LAM_VERSION, version_string,
+                        NULL);
+              kexit(1);
+            } else {
+              show_help(NULL, "unable-to-check-version", LAM_VERSION, NULL);
+              kexit(1);
+            }
 	  }
 	}
 
