@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: accept.c,v 6.16.2.1 2003/08/21 06:32:00 brbarret Exp $
+ * $Id: accept.c,v 6.17 2003/08/22 02:33:18 brbarret Exp $
  *
  *	Function:	- accept a connection on an MPI port
  *	Accepts:	- port name
@@ -167,7 +167,7 @@ MPI_Comm_accept(char *port_name, MPI_Info info, int root, MPI_Comm comm,
  * Accept a connection from a client.
  */
 		err = accept_on_port(port_name, info,
-					comm, cid, &cltcid, &cltsize, &cltgps);
+				     comm, cid, &cltcid, &cltsize, &cltgps);
 		if (err != MPI_SUCCESS) {
 			msg[0] = -1;
 			msg[1] = err;
@@ -289,6 +289,11 @@ MPI_Comm_accept(char *port_name, MPI_Info info, int root, MPI_Comm comm,
 
         LAM_TRACE(lam_tr_cffend(BLKMPICOMMACCEPT, root, comm, 0, 0));
         lam_resetfunc_m(BLKMPICOMMACCEPT);
+
+	/* Free memory */
+	if (cltgps != NULL)
+	  free((char*) cltgps);
+
 	return(MPI_SUCCESS);
 }
 
@@ -426,7 +431,8 @@ accept_on_port(char *port_name, MPI_Info info, MPI_Comm comm, int cid,
 		return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
 
-	free((char *) svrgps);
+	if (svrgps != NULL)
+	  free((char *) svrgps);
 	return(MPI_SUCCESS);
 }
 
