@@ -118,7 +118,8 @@ int MPI_Comm_dup(MPI_Comm comm, MPI_Comm* newcomm)
  */
 	*newcomm = 0;
 	if (lam_comm_new(cid, comm->c_group, comm->c_rgroup,
-				comm->c_flags & ~LAM_PREDEF, newcomm)) {
+                         comm->c_flags & ~LAM_PREDEF & ~LAM_RBASIC,
+                         newcomm)) {
 	    return(lam_errfunc(comm, BLKMPICOMMDUP,
 				lam_mkerr(MPI_ERR_OTHER, errno)));
 	}
