@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: cgetattr.c,v 1.4.2.1 2002/10/09 19:49:06 brbarret Exp $
+ * $Id: cgetattr.c,v 1.6 2002/12/11 19:15:10 jsquyres Exp $
  *
  *	Function:	- get communicator attribute value
  *	Accepts:	- communicator
@@ -66,21 +64,22 @@ MPI_Comm_get_attr(MPI_Comm comm, int key, void *value, int *found)
 * Check the arguments.
 */
 	if (comm == MPI_COMM_NULL) {
-		return(lam_err_comm(MPI_COMM_WORLD, MPI_ERR_COMM, 0,
+		return(lam_err_comm(MPI_COMM_WORLD, MPI_ERR_COMM, EINVAL,
 					"null handle"));
 	}
 
 	if ((value == 0) || (found == 0)) {
-		return(lam_err_comm(comm, MPI_ERR_ARG, 0, "null pointer"));
+		return(lam_err_comm(comm, MPI_ERR_ARG, EINVAL, 
+				    "null pointer"));
 	}
 	
 	pk = lam_getattr(key);
 	if (pk == 0) {
-		return(lam_err_comm(comm, MPI_ERR_KEYVAL, 0, ""));
+		return(lam_err_comm(comm, MPI_ERR_KEYVAL, EINVAL, ""));
 	}
 
 	if (!(pk->ak_flags & LAM_AKCOMM)) {
-		return(lam_err_comm(comm, MPI_ERR_KEYVAL, 0,
+		return(lam_err_comm(comm, MPI_ERR_KEYVAL, EINVAL,
 					"not a communicator key"));
 	}
 /*
