@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: errcreate.c,v 6.4.2.1 2002/10/09 19:49:07 brbarret Exp $
+ * $Id: errcreate.c,v 6.6 2002/12/11 19:15:10 jsquyres Exp $
  *
  *	Function:	- create error handler
  *	Accepts:	- error function
@@ -52,7 +50,7 @@ int MPI_Errhandler_create(MPI_Handler_function *errfunc,
  */
 	if ((errfunc == 0) || (errhdl == 0)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIERRCREATE, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIERRCREATE, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Create the handler.
