@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN/RBD
+ * $HEADER$
  *
- *	$Id: getcount.c,v 6.5.2.2 2002/10/09 19:49:08 brbarret Exp $
+ * $Id: getcount.c,v 6.8 2002/12/11 19:15:10 jsquyres Exp $
  *
  *	Function:	- get # top-level elements
  *	Accepts:	- status
@@ -66,21 +64,23 @@ int MPI_Get_count(MPI_Status *stat, MPI_Datatype dtype,
  */
 	if ((count == 0) || (stat == 0) || (stat->st_length < 0)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIGETCOUNT, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIGETCOUNT, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	if (dtype == MPI_DATATYPE_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIGETCOUNT, lam_mkerr(MPI_ERR_TYPE, 0)));
+			BLKMPIGETCOUNT, lam_mkerr(MPI_ERR_TYPE, EINVAL)));
 	}
 /*
  * Find the count.
  */
 	if (dtype->dt_size == 0) {
 		*count = (stat->st_length == 0) ? 0 : MPI_UNDEFINED;
-	} else if (stat->st_length % dtype->dt_size != 0) {
+	}
+	else if (stat->st_length % dtype->dt_size != 0) {
 		*count = MPI_UNDEFINED;
-	} else {
+	}
+	else {
 		*count = stat->st_length / dtype->dt_size;
 	}
 
