@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD/NJN
+ * $HEADER$
  *
- *	$Id: graphcreate.c,v 6.4.2.1 2002/10/09 19:49:08 brbarret Exp $
+ * $Id: graphcreate.c,v 6.6 2002/12/11 19:15:11 jsquyres Exp $
  *
  *	Function:	- create a communicator with graph mapping
  *	Accepts:	- old communicator
@@ -80,17 +78,17 @@ int MPI_Graph_create(MPI_Comm comm, int nnodes, int *index,
  */
 	if (comm == MPI_COMM_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIGRAPHCREATE, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPIGRAPHCREATE, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if (LAM_IS_INTER(comm)) {
 		return(lam_errfunc(comm,
-			BLKMPIGRAPHCREATE, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPIGRAPHCREATE, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if ((nnodes < 1) || (index == 0) || (edges == 0) || (newcomm == 0)) {
 		return(lam_errfunc(comm,
-			BLKMPIGRAPHCREATE, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIGRAPHCREATE, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	LAM_TRACE(lam_tr_cffstart(BLKMPIGRAPHCREATE));
@@ -114,7 +112,7 @@ int MPI_Graph_create(MPI_Comm comm, int nnodes, int *index,
 		if (*p < 0 || *p >= nnodes) {
 			free((char *) topo);
 			return(lam_errfunc(comm, BLKMPIGRAPHCREATE,
-					lam_mkerr(MPI_ERR_TOPOLOGY, 0)));
+					lam_mkerr(MPI_ERR_TOPOLOGY, EINVAL)));
 		}
 	}
 /*
@@ -129,7 +127,7 @@ int MPI_Graph_create(MPI_Comm comm, int nnodes, int *index,
 	if (nnodes > size) {
 		free((char *) topo);
 		return(lam_errfunc(comm, BLKMPIGRAPHCREATE,
-				lam_mkerr(MPI_ERR_ARG, 0)));
+				lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	if (nnodes == size) {
