@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: infodup.c,v 6.5.2.1 2002/10/09 19:49:09 brbarret Exp $
+ * $Id: infodup.c,v 6.7 2002/12/11 19:15:11 jsquyres Exp $
  *
  *	Function:	- duplicate an info object
  *	Accepts:	- info
@@ -67,7 +65,7 @@ int MPI_Info_dup(MPI_Info info, MPI_Info *newinfo)
 
 	if (info == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIINFODUP,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	err = MPI_Info_create(newinfo);
