@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: infogetvlen.c,v 6.5.2.1 2002/10/09 19:49:09 brbarret Exp $
+ * $Id: infogetvlen.c,v 6.9 2002/12/11 19:15:11 jsquyres Exp $
  *
  *	Function:	- get length of info value associated with key
  *	Accepts:	- info
@@ -48,8 +46,7 @@ Output Parameters:
 Notes:
 
 The length returned in C and C++ does not include the end-of-string
-character.  If the 'key' is not found on 'info', 'valuelen' is set to 
-'MPI_UNDEFINED'.
+character.  If the 'key' is not found on 'info', 'valuelen' is left alone.
 
 .N fortran
 
@@ -72,13 +69,13 @@ int MPI_Info_get_valuelen(MPI_Info info, char *key,
 
 	if (info == 0 || valuelen == 0 || flag == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIINFOGETLEN,
-					lam_mkerr(MPI_ERR_ARG, 0)));
+					lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	keylen = (key) ? strlen(key) : 0;
 	if (keylen == 0 || keylen > MPI_MAX_INFO_KEY) {
 		return(lam_errfunc(MPI_COMM_WORLD, BLKMPIINFOGETLEN,
-					lam_mkerr(MPI_ERR_INFO_KEY, 0)));
+					lam_mkerr(MPI_ERR_INFO_KEY, EINVAL)));
 	}
 
 	strcpy(search.ie_key, key);
@@ -88,7 +85,6 @@ int MPI_Info_get_valuelen(MPI_Info info, char *key,
 		*valuelen = strlen(found->ie_value);
 	} else {
 		*flag = 0;
-		*valuelen = MPI_UNDEFINED;
 	}
 
 	lam_resetfunc_m(BLKMPIINFOGETLEN);
