@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: lambuf.c,v 6.3.2.1 2002/10/09 19:49:10 brbarret Exp $
+ * $Id: lambuf.c,v 6.7 2003/04/10 15:40:01 jsquyres Exp $
  *
  *	Function:	- buffer management functions
  */
@@ -29,27 +27,12 @@
 #include <t_types.h>
 #include <terror.h>
 
+
 /*
  * macros
  */
 #define HDRSIZE		(sizeof(struct _bsndhdr))
 
-/*
- * public functions
- */
-int			lam_bufinitbsend_();
-int			lam_bufinit();
-int			lam_buffreebsend_();
-int			lam_bufattach();
-int			lam_bufdetach();
-
-/*
- * external functions
- */
-extern int		lam_mkerr();
-extern int		_mpi_req_advance();
-extern void		_mpi_req_blkclr();
-extern void		_mpi_req_blkset();
 
 /*
  * local variables
@@ -60,13 +43,15 @@ static struct _bsndhdr	*buftop = 0;
 static struct _bsndhdr	*bufend = 0;
 static struct _bsndhdr	*buffree = 0;
 
+
 /*
  * local functions
  */
-static void		buf_append();
-static void		buf_free();
-static void		buf_unlink();
-static struct _bsndhdr	*buf_alloc();
+static void buf_append(struct _bsndhdr *pbuf);
+static void buf_free(struct _bsndhdr *pchunk);
+static void buf_unlink(struct _bsndhdr *pbuf);
+static struct _bsndhdr *buf_alloc(int size);
+
 
 /*
  *	lam_bufinitbsend_
@@ -76,10 +61,7 @@ static struct _bsndhdr	*buf_alloc();
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_bufinitbsend_(rq)
-
-MPI_Request		rq;
-
+lam_bufinitbsend_(MPI_Request rq)
 {
 /*
  * Handle zero length messages.
@@ -112,6 +94,7 @@ MPI_Request		rq;
 	return(MPI_SUCCESS);
 }
 
+
 /*
  *	lam_bufinit
  *
@@ -120,10 +103,7 @@ MPI_Request		rq;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_bufinit(rq)
-
-MPI_Request		rq;
-
+lam_bufinit(MPI_Request rq)
 {
 	MPI_Datatype	dtype;			/* datatype */
 	double		local_rep;		/* local data representation */
@@ -147,30 +127,29 @@ MPI_Request		rq;
 	rq->rq_packsize = rq->rq_count * dtype->dt_size;
 	rq->rq_marks |= LAM_RQFCHAR;
 
-	if ((dtype->dt_flags & LAM_DTNOPACK)
-			&& ((dtype->dt_flags & LAM_DTNOXADJ) || rq->rq_count==1)
-			&& ((local_rep == net_rep) || lam_homog)) {
-		rq->rq_packbuf = (char *) rq->rq_buf;
-/*
- * Check for bad buffer.
- */
-		if (rq->rq_packbuf == 0) {
-			return(lam_mkerr(MPI_ERR_BUFFER, 0));
-		}
+	if ((dtype->dt_flags & LAM_DTNOPACK) &&
+            ((dtype->dt_flags & LAM_DTNOXADJ) || rq->rq_count==1) &&
+            ((local_rep == net_rep) || lam_homog)) {
+          rq->rq_packbuf = (char *) rq->rq_buf;
+
+          /* Check for bad buffer. */
+
+          if (rq->rq_packbuf == 0)
+            return(lam_mkerr(MPI_ERR_BUFFER, EINVAL));
 	}
-/*
- * Otherwise allocate a buffer.
- */
+
+        /* Otherwise allocate a buffer. */
+
 	else {
-		rq->rq_packbuf = malloc((unsigned) rq->rq_packsize);
-		if (rq->rq_packbuf == 0) {
-			return(lam_mkerr(MPI_ERR_OTHER, errno));
-		}
+          rq->rq_packbuf = malloc((unsigned) rq->rq_packsize);
+          if (rq->rq_packbuf == 0)
+            return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
 
 	return(MPI_SUCCESS);
 }
 
+
 /*
  *	lam_buffreebsend_
  *
@@ -179,10 +158,7 @@ MPI_Request		rq;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_buffreebsend_(req)
-
-MPI_Request		req;
-
+lam_buffreebsend_(MPI_Request req)
 {
 /*
  * Handle zero length messages.
@@ -198,6 +174,7 @@ MPI_Request		req;
 	return(MPI_SUCCESS);
 }
 
+
 /*
  *	lam_bufattach
  *
@@ -207,11 +184,7 @@ MPI_Request		req;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_bufattach(buf, size)
-
-void			*buf;
-int			size;
-
+lam_bufattach(void *buf, int size)
 {
 	unsigned long	inc;			/* alignment increment */
 
@@ -247,6 +220,7 @@ int			size;
 	}
 }
 
+
 /*
  *	lam_bufdetach
  *
@@ -257,11 +231,7 @@ int			size;
  *	Returns:	- MPI_SUCCESS or error code
  */
 int
-lam_bufdetach(pbuf, psize)
-
-void			*pbuf;
-int			*psize;
-
+lam_bufdetach(void *pbuf, int *psize)
 {
 	int		err;			/* error code */
 	struct _bsndhdr	*p;			/* favourite pointer */
@@ -296,6 +266,7 @@ int			*psize;
 	return(MPI_SUCCESS);
 }
 
+
 /*
  *	buf_alloc
  *
@@ -305,10 +276,7 @@ int			*psize;
  *	Returns:	- ptr allocated buffer header or NULL
  */
 static struct _bsndhdr *
-buf_alloc(size)
-
-int			size;
-
+buf_alloc(int size)
 {
 	struct _bsndhdr	*pchunk;		/* ptr allocated chunk */
 	struct _bsndhdr	*pcur;			/* ptr current block */
@@ -356,6 +324,7 @@ int			size;
 	return(pchunk);
 }
 
+
 /*
  *	buf_free
  *
@@ -365,10 +334,7 @@ int			size;
  *	Accepts:	- ptr chunk
  */
 static void
-buf_free(pchunk)
-
-struct _bsndhdr		*pchunk;
-
+buf_free(struct _bsndhdr *pchunk)
 {
 	struct _bsndhdr	*pcur;			/* ptr current chunk */
 	struct _bsndhdr	*pprev;			/* ptr previous chunk */
@@ -414,6 +380,7 @@ struct _bsndhdr		*pchunk;
 	}
 }
 
+
 /*
  *	buf_append
  *
@@ -421,10 +388,7 @@ struct _bsndhdr		*pchunk;
  *	Accepts:	- ptr buffer
  */
 static void
-buf_append(pbuf)
-
-struct _bsndhdr		*pbuf;
-
+buf_append(struct _bsndhdr *pbuf)
 {
 	pbuf->bsh_prev = bufend;
 	pbuf->bsh_next = 0;
@@ -435,6 +399,7 @@ struct _bsndhdr		*pbuf;
 	bufend = pbuf;
 }
 
+
 /*
  *	buf_unlink
  *
@@ -442,10 +407,7 @@ struct _bsndhdr		*pbuf;
  *	Accepts:	- ptr buffer entry
  */
 static void
-buf_unlink(pbuf)
-
-struct _bsndhdr		*pbuf;
-
+buf_unlink(struct _bsndhdr *pbuf)
 {
 	struct _bsndhdr	*pprev;			/* ptr previous buffer */
 	struct _bsndhdr	*pnext;			/* ptr next buffer */
