@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: laminit.c,v 6.62.2.1 2003/09/11 20:17:07 jsquyres Exp $
+ * $Id: laminit.c,v 6.63 2003/09/11 19:52:02 jsquyres Exp $
  *
  *	Function:	- LAM specific initialization for MPI
  */
@@ -77,7 +77,10 @@ struct _dtype		lam_mpi_ub, lam_mpi_lb;
 struct _dtype		lam_mpi_float_int, lam_mpi_double_int;
 struct _dtype		lam_mpi_long_int, lam_mpi_2int;
 struct _dtype		lam_mpi_short_int, lam_mpi_dblcplex;
-struct _dtype		lam_mpi_integer, lam_mpi_real;
+struct _dtype		lam_mpi_integer, lam_mpi_integer1;
+struct _dtype           lam_mpi_integer2, lam_mpi_integer4;
+struct _dtype           lam_mpi_integer8, lam_mpi_real;
+struct _dtype           lam_mpi_real4, lam_mpi_real8, lam_mpi_real16;
 struct _dtype		lam_mpi_dblprec, lam_mpi_character;
 struct _dtype		lam_mpi_2real, lam_mpi_2dblprec;
 struct _dtype		lam_mpi_2integer, lam_mpi_longdbl_int;
