@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: lammpiinit.c,v 6.32.2.2 2003/09/11 20:17:36 jsquyres Exp $
+ * $Id: lammpiinit.c,v 6.42 2004/03/09 18:01:12 adharurk Exp $
  *
  *	Function:	- initialize the MPI session
  */
@@ -49,6 +49,7 @@
 #if LAM_WANT_IMPI
 #include <impi.h>
 #endif
+#include <malloc_wrapper.h>
 
 
 /*
@@ -205,7 +206,6 @@ lam_mpi_init(int *pargc, char ***pargv, int requested, int *pprovided,
   int tmp_nprocs;
   int argc;
   char **argv;
-  char *bogus;
 
   /* Check if we have been initialized or finalized. */
 
@@ -240,11 +240,6 @@ lam_mpi_init(int *pargc, char ***pargv, int requested, int *pprovided,
   
   lam_tv_load_type_defs();
 
-  /* Bogus usage of variable so that compilers don't complain */
-
-  bogus = strdup(lam_mpi_version);
-  free(bogus);
-
   /* Initialize LAM. */
 
   name = (pargc && pargv && (*pargc > 0)) ? **pargv : 0;
@@ -276,10 +271,24 @@ lam_mpi_init(int *pargc, char ***pargv, int requested, int *pprovided,
   lam_exchange_endian_info(tmp_procs, tmp_nprocs);
   free(tmp_procs);
 
+  /*
+   * make sure we do any memory pinning initialization we might need
+   * to do, before we load the ssi modules.  Needs to be here to
+   * guarantee the linker doesn't get stupid.
+   */
+  lam_register_free_handler_init();
+
+  /* Initialize SSI */
+
+  if (lam_ssi_base_open(lam_mpi_ao) == LAMERROR)
+    fail_stop("mpi-init", "ssi-fail-open");
+
   /* Find out what SSI RPI modules are available */
   
   if (lam_ssi_rpi_base_open(lam_mpi_ao) == LAMERROR)
     fail_stop("mpi-init", "ssi-rpi-fail-open");
+  if (lam_ssi_rpi_base_query() == LAMERROR)
+    fail_stop("mpi-init", "ssi-rpi-fail-query");
 
   /* Initialize MPI pre-defined "stuff". The order below is important,
      so don't hack without significant care. */
@@ -292,11 +301,15 @@ lam_mpi_init(int *pargc, char ***pargv, int requested, int *pprovided,
 
   if (lam_ssi_coll_base_open(lam_mpi_ao) == LAMERROR)
     fail_stop("mpi-init", "ssi-coll-fail-open");
+  if (lam_ssi_coll_base_query() == LAMERROR)
+    fail_stop("mpi-init", "ssi-coll-fail-query");
 
   /* Find out which SSI crmpi modules are available */
 
   if (lam_ssi_crmpi_base_open(lam_mpi_ao) == LAMERROR)
     fail_stop("mpi-init", "ssi-crmpi-fail-open");
+  if (lam_ssi_crmpi_base_query() == LAMERROR)
+    fail_stop("mpi-init", "ssi-crmpi-fail-query");
 
   /* Now that all SSI modules have been opened and we have lists of
      available modules, we can decide on which modules will be
@@ -478,6 +491,13 @@ lam_mpi_init(int *pargc, char ***pargv, int requested, int *pprovided,
 			  blk, lam_mkerr(MPI_ERR_OTHER, errno)));
   }
 
+  /* Now that MPI_INIT* is effectively done, invoke any callbacks that
+     were registered. */
+
+  if (lam_ssi_base_mpi_init_callbacks_invoke() != MPI_SUCCESS)
+    return (lam_errfunc(MPI_COMM_WORLD,
+			blk, lam_mkerr(MPI_ERR_OTHER, errno)));
+
   /* All done */
 
   return (MPI_SUCCESS);
@@ -548,8 +568,14 @@ lam_send_selected_ssi_modules(void)
    *
    * It is possible that no CR modules were selected. So handle that case. 
    */
+#if 0
   if (lam_ssi_crmpi_base_available != NULL)
     module = (lam_ssi_module_t *) al_top(lam_ssi_crmpi_base_available);
+#else
+  /* JMS, for the moment, due to bug 682, we're just going to skip
+     checking cr modules. */
+  module = NULL;
+#endif
 
   if ((NULL == lam_ssi_crmpi_base_available) || 
       (NULL == module)) {
@@ -588,8 +614,11 @@ init_cr(void)
   int have_cr = 1;
   lam_ssi_module_t *cr_module, *module, *next;
   lam_ssi_crmpi_t *cr;
-  lam_ssi_coll_t *coll;
+  lam_ssi_t *coll;
+  lam_ssi_coll_1_0_0_t *coll100;
+  lam_ssi_coll_1_1_0_t *coll110;
   const lam_ssi_crmpi_actions_t *cra;
+  int coll_has_cr;
 
   /* If we have an empty list, simply return -- there's nothing to
      do */
@@ -621,12 +650,24 @@ init_cr(void)
     for (module = (lam_ssi_module_t*) al_top(lam_ssi_coll_base_available);
          module != NULL; 
          module = al_next(lam_ssi_coll_base_available, module)) {
-      coll = (lam_ssi_coll_t*) module->lsm_module;
-      if (coll->lsc_has_checkpoint == 0) {
+      coll = module->lsm_module;
+      coll_has_cr = 0;
+      if (coll->ssi_kind_major_version == 1 &&
+          coll->ssi_kind_minor_version == 0 &&
+          coll->ssi_kind_release_version == 0) {
+        coll100 = (lam_ssi_coll_1_0_0_t *) coll;
+        coll_has_cr = coll100->lsc_has_checkpoint;
+      } else if (coll->ssi_kind_major_version == 1 &&
+                 coll->ssi_kind_minor_version == 1 &&
+                 coll->ssi_kind_release_version == 0) {
+        coll110 = (lam_ssi_coll_1_1_0_t *) coll;
+        coll_has_cr = coll110->lsc_has_checkpoint;
+      }
+      if (coll_has_cr == 0) {
         if (lam_ssi_cr_verbose >= 0)
           lam_debug(lam_ssi_cr_did,
                     " CR support disabled; %s coll module does not "
-                    "support checkpoint", coll->lsc_meta_info.ssi_module_name);
+                    "support checkpoint", coll->ssi_module_name);
         have_cr = 0;
         break;
       }
@@ -669,7 +710,12 @@ init_cr(void)
     cr = (lam_ssi_crmpi_t*) next->lsm_module;
     if (cr->lscrm_meta_info.ssi_close_module != NULL)
       cr->lscrm_meta_info.ssi_close_module();
+    lam_ssi_base_module_registry_unuse(next->lsm_module);
   }  
+  if (have_cr == 0) {
+    al_free(lam_ssi_crmpi_base_available);
+    lam_ssi_crmpi_base_available = NULL;
+  }
 
   /* All done */
 
@@ -899,8 +945,34 @@ init_dtype(void)
 	     LAM_ALIGNMENT_INT, TRDTFLOGICAL, "MPI_LOGICAL");
   make_dtype(MPI_F_INTEGER, sizeof(int),
 	     LAM_ALIGNMENT_INT, TRDTFINTEGER, "MPI_INTEGER");
+  make_dtype(MPI_F_INTEGER1, sizeof(LAM_FORTRAN_C_TYPE_INTEGER1),
+             LAM_ALIGNMENT_FORTRAN_INTEGER1, TRDTFINTEGER1, "MPI_INTEGER1");
+  make_dtype(MPI_F_INTEGER2, sizeof(LAM_FORTRAN_C_TYPE_INTEGER2),
+             LAM_ALIGNMENT_FORTRAN_INTEGER2, TRDTFINTEGER2, "MPI_INTEGER2");
+  make_dtype(MPI_F_INTEGER4, sizeof(LAM_FORTRAN_C_TYPE_INTEGER4),
+             LAM_ALIGNMENT_FORTRAN_INTEGER4, TRDTFINTEGER4, "MPI_INTEGER4");
+#if LAM_SIZEOF_FORTRAN_INTEGER8 
+  make_dtype(MPI_F_INTEGER8, sizeof(LAM_FORTRAN_C_TYPE_INTEGER8),
+             LAM_ALIGNMENT_FORTRAN_INTEGER8, TRDTFINTEGER8, "MPI_INTEGER8");
+#else
+  make_dtype(MPI_F_INTEGER8, 0,
+             LAM_ALIGNMENT_INT, TRDTFINTEGER, "MPI_INTEGER8");
+#endif
+
   make_dtype(MPI_F_REAL, sizeof(float),
-	     LAM_ALIGNMENT_FLOAT, TRDTFREAL, "MPI_REAL");
+             LAM_ALIGNMENT_FLOAT, TRDTFREAL, "MPI_REAL");
+  make_dtype(MPI_F_REAL4, sizeof(LAM_FORTRAN_C_TYPE_REAL4),
+             LAM_ALIGNMENT_FORTRAN_REAL4, TRDTFREAL4, "MPI_REAL4");
+  make_dtype(MPI_F_REAL8, sizeof(LAM_FORTRAN_C_TYPE_REAL8),
+             LAM_ALIGNMENT_FORTRAN_REAL8, TRDTFREAL8, "MPI_REAL8");
+#if LAM_SIZEOF_FORTRAN_REAL16
+  make_dtype(MPI_F_REAL16, sizeof(LAM_FORTRAN_C_TYPE_REAL16), 
+             LAM_ALIGNMENT_FORTRAN_REAL16, TRDTFREAL16, "MPI_REAL16"); 
+#else
+  make_dtype(MPI_F_REAL16, 0, 
+             LAM_ALIGNMENT_INT, TRDTFINTEGER, "MPI_REAL16");
+#endif
+
   make_dtype(MPI_F_DOUBLE_PRECISION, sizeof(double),
 	     LAM_ALIGNMENT_DOUBLE, TRDTFDBLPREC, "MPI_DOUBLE_PRECISION");
 
@@ -1384,6 +1456,17 @@ init_env(void)
   if ((err != MPI_SUCCESS) || 
       (key != LAM_MPI_SSI_COLL_REDUCE_CROSSOVER)) 
     return LAMERROR;
+  err = MPI_Comm_create_keyval(LAM_MPI_C_NULL_COPY_FN, 
+			       LAM_MPI_C_NULL_DELETE_FN,
+			       &key, (void *) 0);
+  if ((err != MPI_SUCCESS) || (key != LAM_MPI_SSI_COLL_SHMEM_MESSAGE_POOL_SIZE)) 
+    return LAMERROR;
+  err = MPI_Comm_create_keyval(LAM_MPI_C_NULL_COPY_FN, 
+			       LAM_MPI_C_NULL_DELETE_FN,
+			       &key, (void *) 0);
+  if ((err != MPI_SUCCESS) || (key != LAM_MPI_SSI_COLL_SHMEM_NUM_SEGMENTS)) 
+    return LAMERROR;
+
  
 /*
  * Initialize the attribute values.
@@ -1747,8 +1830,22 @@ init_f77(void)
   lam_F_handles[lam_F_nhdl++] = MPI_F_LOGICAL;
   MPI_F_INTEGER->dt_f77handle = lam_F_nhdl;
   lam_F_handles[lam_F_nhdl++] = MPI_F_INTEGER;
+  MPI_F_INTEGER1->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++] = MPI_F_INTEGER1;
+  MPI_F_INTEGER2->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++]  = MPI_F_INTEGER2; 
+  MPI_F_INTEGER4->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++]  = MPI_F_INTEGER4; 
+  MPI_F_INTEGER8->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++]  = MPI_F_INTEGER8; 
   MPI_F_REAL->dt_f77handle = lam_F_nhdl;
   lam_F_handles[lam_F_nhdl++] = MPI_F_REAL;
+  MPI_F_REAL4->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++] = MPI_F_REAL4;
+  MPI_F_REAL8->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++] = MPI_F_REAL8;
+  MPI_F_REAL16->dt_f77handle = lam_F_nhdl;
+  lam_F_handles[lam_F_nhdl++] = MPI_F_REAL16;
   MPI_F_DOUBLE_PRECISION->dt_f77handle = lam_F_nhdl;
   lam_F_handles[lam_F_nhdl++] = MPI_F_DOUBLE_PRECISION;
   MPI_F_COMPLEX->dt_f77handle = lam_F_nhdl;
