@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2002 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,8 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *
- * $Id: lamreduce.c,v 6.6.2.3 2003/01/27 01:40:50 jsquyres Exp $
+ * $Id: lamreduce.c,v 6.14 2002/12/16 16:23:31 jsquyres Exp $
  *
  *	Function:	- all predefined reduce operations
  *			- min, max, sum, product
@@ -211,7 +212,7 @@ lam_max(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -376,7 +377,7 @@ lam_min(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -393,6 +394,7 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	double		*pdin, *pdout;
 	struct complex	*pxin, *pxout;
 	struct dblcplex	*pdxin, *pdxout;
+	struct ldblcplex *pldxin, *pldxout;
 #if LAM_SIZEOF_LONG_DOUBLE
 	LAM_LONG_DOUBLE	*pldin, *pldout;
 #else
@@ -469,7 +471,7 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
-	if (*datatype == MPI_F_COMPLEX) {
+	if (*datatype == MPI_F_COMPLEX || *datatype == MPI_CXX_COMPLEX) {
 		pxin = (struct complex *) invec;
 		pxout = (struct complex *) inoutvec;
 		for (i = 0; i < *len; ++i, ++pxin, ++pxout) {
@@ -479,7 +481,8 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
-	if (*datatype == MPI_F_DOUBLE_COMPLEX) {
+	if (*datatype == MPI_F_DOUBLE_COMPLEX ||
+	    *datatype == MPI_CXX_DOUBLE_COMPLEX) {
 		pdxin = (struct dblcplex *) invec;
 		pdxout = (struct dblcplex *) inoutvec;
 		for (i = 0; i < *len; ++i, ++pdxin, ++pdxout) {
@@ -489,6 +492,16 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+	if (*datatype == MPI_CXX_LONG_DOUBLE_COMPLEX) {
+		pldxin = (struct ldblcplex *) invec;
+		pldxout = (struct ldblcplex *) inoutvec;
+		for (i = 0; i < *len; ++i, ++pldxin, ++pldxout) {
+			pldxout->r += pldxin->r;
+			pldxout->i += pldxin->i;
+		}
+		return;
+	}
+
 	if (*datatype == MPI_DOUBLE || *datatype == MPI_F_DOUBLE_PRECISION) {
 		pdin = (double *) invec;
 		pdout = (double *) inoutvec;
@@ -541,7 +554,7 @@ lam_sum(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -560,10 +573,13 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	double		*pdin, *pdout;
 	struct complex	*pxin, *pxout;
 	struct dblcplex	*pdxin, *pdxout;
+	struct ldblcplex *pldxin, *pldxout;
 #if LAM_SIZEOF_LONG_DOUBLE
 	LAM_LONG_DOUBLE	*pldin, *pldout;
+	LAM_LONG_DOUBLE ldxr, ldxi;
 #else
 	double		*pldin, *pldout;
+	double          ldxr, ldxi;
 #endif
 #if LAM_SIZEOF_LONG_LONG
 	long long	*pllin, *pllout;
@@ -636,7 +652,8 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
-	if (*datatype == MPI_F_COMPLEX) {
+	if (*datatype == MPI_F_COMPLEX ||
+	    *datatype == MPI_CXX_COMPLEX) {
 		pxin = (struct complex *) invec;
 		pxout = (struct complex *) inoutvec;
 		for (i = 0; i < *len; ++i, ++pxin, ++pxout) {
@@ -648,7 +665,8 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
-	if (*datatype == MPI_F_DOUBLE_COMPLEX) {
+	if (*datatype == MPI_F_DOUBLE_COMPLEX ||
+	    *datatype == MPI_CXX_DOUBLE_COMPLEX) {
 		pdxin = (struct dblcplex *) invec;
 		pdxout = (struct dblcplex *) inoutvec;
 		for (i = 0; i < *len; ++i, ++pdxin, ++pdxout) {
@@ -660,6 +678,18 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+	if (*datatype == MPI_CXX_LONG_DOUBLE_COMPLEX) {
+	  pldxin = (struct ldblcplex *) invec;
+	  pldxout = (struct ldblcplex *) inoutvec;
+	  for (i = 0; i < *len; ++i, ++pldxin, ++pldxout) {
+	    ldxr = pldxout->r;
+	    ldxi = pldxout->i;
+	    pldxout->r = (ldxr * pldxin->r) - (ldxi * pldxin->i);
+	    pldxout->i = (ldxr * pldxin->i) + (ldxi * pldxin->r);
+	  }
+	  return;
+	}
+
 	if (*datatype == MPI_DOUBLE || *datatype == MPI_F_DOUBLE_PRECISION) {
 		pdin = (double *) invec;
 		pdout = (double *) inoutvec;
@@ -712,9 +742,19 @@ lam_prod(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
+
+/*
+ * Logical And
+ *
+ * Supported by:
+ *   C integer: MPI_INT, MPI_LONG, MPI_SHORT, MPI_UNSIGNED_SHORT,
+ *              MPI_UNSIGNED, MPI_UNSIGNED_LONG
+ *   Logical:   MPI_LOGICAL
+ *   Special C++: MPI_CXX_BOOL
+ */
 void
 lam_land(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 {
@@ -725,6 +765,7 @@ lam_land(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	unsigned short	*pusin, *pusout;
 	unsigned	*puin, *puout;
 	unsigned long	*pulin, *pulout;
+	LAM_CXX_BOOL_TYPE *pboolin, *pboolout;
 #if LAM_SIZEOF_LONG_LONG
 	long long	*pllin, *pllout;
 	unsigned long long *pullin, *pullout;
@@ -816,10 +857,30 @@ lam_land(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+	if (*datatype == MPI_CXX_BOOL) {
+	  pboolin = (LAM_CXX_BOOL_TYPE *) invec;
+	  pboolout = (LAM_CXX_BOOL_TYPE *) inoutvec;
+
+	  for (i = 0 ; i < *len ; ++i, ++pboolin, ++pboolout) {
+	    *pboolout = *pboolout && *pboolin;
+	  }
+	  return;
+	}
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
+
+/*
+ * Logical Or
+ *
+ * Supported by:
+ *   C integer: MPI_INT, MPI_LONG, MPI_SHORT, MPI_UNSIGNED_SHORT,
+ *              MPI_UNSIGNED, MPI_UNSIGNED_LONG
+ *   Logical:   MPI_LOGICAL
+ *   Special C++: MPI_CXX_BOOL
+ */
 void
 lam_lor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 {
@@ -830,6 +891,7 @@ lam_lor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	unsigned short	*pusin, *pusout;
 	unsigned	*puin, *puout;
 	unsigned long	*pulin, *pulout;
+	LAM_CXX_BOOL_TYPE *pboolin, *pboolout;
 #if LAM_SIZEOF_LONG_LONG
 	long long	*pllin, *pllout;
 	unsigned long long *pullin, *pullout;
@@ -921,10 +983,30 @@ lam_lor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+	if (*datatype == MPI_CXX_BOOL) {
+	  pboolin = (LAM_CXX_BOOL_TYPE *) invec;
+	  pboolout = (LAM_CXX_BOOL_TYPE *) inoutvec;
+
+	  for (i = 0 ; i < *len ; ++i, ++pboolin, ++pboolout) {
+	    *pboolout = *pboolout || *pboolin;
+	  }
+	  return;
+	}
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
+
+/*
+ * Logical Or
+ *
+ * Supported by:
+ *   C integer: MPI_INT, MPI_LONG, MPI_SHORT, MPI_UNSIGNED_SHORT,
+ *              MPI_UNSIGNED, MPI_UNSIGNED_LONG
+ *   Logical:   MPI_LOGICAL
+ *   Special C++: MPI_CXX_BOOL
+ */
 void
 lam_lxor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 {
@@ -935,6 +1017,7 @@ lam_lxor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	unsigned short	*pusin, *pusout;
 	unsigned	*puin, *puout;
 	unsigned long	*pulin, *pulout;
+	LAM_CXX_BOOL_TYPE *pboolin, *pboolout;
 #if LAM_SIZEOF_LONG_LONG
 	long long	*pllin, *pllout;
 	unsigned long long *pullin, *pullout;
@@ -1027,8 +1110,19 @@ lam_lxor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		return;
 	}
 
+	if (*datatype == MPI_CXX_BOOL) {
+	  pboolin = (LAM_CXX_BOOL_TYPE *) invec;
+	  pboolout = (LAM_CXX_BOOL_TYPE *) inoutvec;
+
+	  for (i = 0 ; i < *len ; ++i, ++pboolin, ++pboolout) {
+	    *pboolout = (*pboolout || *pboolin) &&
+	      !(*pboolout && *pboolin);
+	  }
+	  return;
+	}
+
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -1143,7 +1237,7 @@ lam_band(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -1258,7 +1352,7 @@ lam_bor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -1373,7 +1467,7 @@ lam_bxor(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -1510,7 +1604,7 @@ lam_maxloc(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 
@@ -1648,7 +1742,7 @@ lam_minloc(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
 
 void
@@ -1671,12 +1765,12 @@ lam_replace(void *invec, void *inoutvec, int *len, MPI_Datatype *datatype)
 		|| *datatype == MPI_LONG_LONG_INT
 		|| *datatype == MPI_UNSIGNED_LONG_LONG) {
 
-		memcpy((char *) inoutvec,
-			(char *) invec, *len * (*datatype)->dt_size);
+		lam_memcpy((char *) inoutvec,
+			   (char *) invec, *len * (*datatype)->dt_size);
 
 		return;
 	}
 
 	(void) lam_errfunc(MPI_COMM_WORLD,
-				lam_getfunc(), lam_mkerr(MPI_ERR_OP, 0));
+			   lam_getfunc(), lam_mkerr(MPI_ERR_OP, EINVAL));
 }
