@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: lamtrace.c,v 6.15.2.2 2003/09/11 20:18:36 jsquyres Exp $
+ * $Id: lamtrace.c,v 6.18 2003/09/11 20:12:40 jsquyres Exp $
  *
  *	Function:	- MPI trace generation
  *
@@ -323,7 +323,7 @@ lam_tr_off()
 /*
  *	lam_tr_on
  *
- *	Function:	- turn on tracing
+ *	Function:	- turn on message tracing
  *			- stores a run time trace and a on/off list trace
  *	Returns:	- previous trace mode or LAMERROR
  */
@@ -491,7 +491,7 @@ lam_tr_msg(int type, double time, int syst, int blkt, int peer, int tag,
 	pmsg->trm_mrank = mrank;
 	pmsg->trm_mtag = mtag;
 	pmsg->trm_seqnum = seqnum;
-	
+
 	mltoti4((int4 *) pmsg, sizeof(struct trmsg) / sizeof(int4));
 }
 
