@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: packsize.c,v 6.3.2.1 2002/10/09 19:49:12 brbarret Exp $
+ * $Id: packsize.c,v 6.5 2002/12/11 19:15:13 jsquyres Exp $
  *
  *	Function:	- get size needed to pack data
  *	Accepts:	- count
@@ -71,22 +69,22 @@ int MPI_Pack_size(int count, MPI_Datatype dtype,
  */
 	if (comm == MPI_COMM_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if (count < 0) {
 		return(lam_errfunc(comm,
-			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_COUNT, 0)));
+			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_COUNT, EINVAL)));
 	}
 
 	if (dtype == MPI_DATATYPE_NULL) {
 		return(lam_errfunc(comm,
-			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_TYPE, 0)));
+			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_TYPE, EINVAL)));
 	}
 
 	if (psize == 0) {
 		return(lam_errfunc(comm,
-			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIPACKSIZE, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Return the size.
