@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: put.c,v 1.2.2.1 2002/10/09 19:49:12 brbarret Exp $
+ * $Id: put.c,v 1.6 2002/12/11 19:15:13 jsquyres Exp $
  *
  *	Function:	- put data into a window
  *	Accepts:	- origin address
@@ -72,19 +70,9 @@ the target process memory.
 
 .N WEB
   @*/
-
 int
-MPI_Put(orgaddr, orgcnt, orgtype, rank, targdisp, targcnt, targtype, win)
-
-void			*orgaddr;
-int			orgcnt;
-MPI_Datatype		orgtype;
-int			rank;
-MPI_Aint		targdisp;
-int			targcnt;
-MPI_Datatype		targtype;
-MPI_Win			win;
-
+MPI_Put(void *orgaddr, int orgcnt, MPI_Datatype orgtype, int rank, 
+	MPI_Aint targdisp, int targcnt, MPI_Datatype targtype, MPI_Win win)
 {
 	struct _osdhdr	*header;		/* 1-sided protocol header */
 	int		err;			/* error code */
@@ -95,7 +83,7 @@ MPI_Win			win;
  * Check arguments.
  */
 	if (win == MPI_WIN_NULL) {
-		return(lam_err_comm(MPI_COMM_WORLD, MPI_ERR_WIN, 0,
+		return(lam_err_comm(MPI_COMM_WORLD, MPI_ERR_WIN, EINVAL,
 			"null handle"));
 	}
 
@@ -126,6 +114,7 @@ MPI_Win			win;
  * Sender protocol header followed by data.
  */ 
 	header = (struct _osdhdr *) malloc(sizeof(struct _osdhdr));
+	LAM_ZERO_ME(*header);
 	if (header == 0) {
 		return(lam_err_win(win, MPI_ERR_OTHER, errno,
 			"allocating header"));
