@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: spawn.c,v 6.46.2.1 2003/06/20 03:38:59 jsquyres Exp $
+ * $Id: spawn.c,v 6.49 2003/09/02 20:48:58 vsahay Exp $
  *
  *	Function:	- spawn MPI program
  *	Accepts:	- program name
@@ -59,7 +59,7 @@ static int		build_app(MPI_Comm comm, char *command, char **argv,
 static void             set_error_codes(int err, LIST *app, int *errs);
 static char 		*locate_aschema(char *filename);
 static int              get_sched_rrobin(struct aschema *proc, char *value,
-					 int maxprocs);
+					 int maxprocs, int flag);
 static int              find_cpu_node(struct route *routes, int num_routes,
 				      int cpu);
 
@@ -187,14 +187,25 @@ given with the value of "c14", and maxprocs is 4, LAM will spawn MPI
 processes on:
 
 .vb
-    CPU  Node  MPI_COMM_WORLD rank
-    ---  ----  -------------------
-    c14  n7    0
-    c15  n7    1
-    c0   n0    2
-    c1   n0    3
+CPU  Node  MPI_COMM_WORLD rank
+---  ----  -------------------
+c14  n7    0
+c15  n7    1
+c0   n0    2
+c1   n0    3
 .ve
 
+'lam_no_root_node_schedule'
+
+This key is used to designate that the spawned processes must not be 
+spawned or scheduled on the "root node" (the node doing the spawn). There 
+is no specific value associated with this key, but it should be given some 
+non-null/non-empty dummy value. 
+
+It is a node-specific key and not a CPU-specific one. Hence if the root node
+has multiple CPUs, none of the CPUs on this root node will take part in the 
+scheduling of the spawned processes.
+
 No keys given
 
 If none of the info keys listed above are used, the value of
@@ -364,6 +375,7 @@ MPI_Comm_spawn(char* command, char** argv, int maxprocs, MPI_Info info,
 /*
  * The root does the process spawning.
  */
+
 		err = build_app(comm, command, argv, maxprocs, info, &app);
 		if (err != MPI_SUCCESS) {
 			return(lam_errfunc(comm, BLKMPICOMMSPAWN, err));
@@ -710,8 +722,18 @@ build_app(MPI_Comm comm, char *command, char **argv, int maxprocs,
     char		value[MPI_MAX_OBJECT_NAME];
     struct appenv       *procenv;		/* process env */
     char                **env = 0;
+    void 		**attrp;     
+    int 		found;
+    int 		max_nodes;
+    int 		*nodes = 0;
+    int 		mynodeid;
+    int 		mynodeindex;
+    int 		actual_mynodetype;
+    struct route 	*routes = 0;
+    int 		sched_round_robin;
+    int 		no_root_node_schedule;
+  
     
-
     if (info) {
 	err = MPI_Info_get(info, "lam_spawn_file", MPI_MAX_OBJECT_NAME, 
 			   schema, &flag);
@@ -749,59 +771,75 @@ build_app(MPI_Comm comm, char *command, char **argv, int maxprocs,
 	proc.asc_srcnode = -1;
 	proc.asc_errno = 0;
 
-        /* Look for the key lam_spawn_sched_round_robin */
+	/* Look for the lam_no_root_node_schedule key */
 
+	no_root_node_schedule = 0;
 	if (info != NULL) {
-	  err = MPI_Info_get(info, "lam_spawn_sched_round_robin", 
-			     MPI_MAX_OBJECT_NAME, value, &flag);
+	  err = MPI_Info_get(info, "lam_no_root_node_schedule", 
+			     MPI_MAX_OBJECT_NAME, value, 
+			     &no_root_node_schedule);
+
 	  if (err != MPI_SUCCESS)
 	    return(lam_errfunc(comm, BLKMPICOMMSPAWN, err));
-	  if (flag == 1)
-	    get_sched_rrobin(&proc, value, maxprocs);
+	}
+
+        /* Look for the key lam_spawn_sched_round_robin */
+	sched_round_robin = 0;
+	if (info != NULL) {
+	  err = MPI_Info_get(info, "lam_spawn_sched_round_robin", 
+			     MPI_MAX_OBJECT_NAME, value, &sched_round_robin);
+	  if (err != MPI_SUCCESS)
+	    return(lam_errfunc(comm, BLKMPICOMMSPAWN, err));
+	  
+	  if (sched_round_robin == 1) {
+	    get_sched_rrobin(&proc, value, maxprocs, no_root_node_schedule);
+	  }
 	}
 	
 	/* This will get set in asc_schedule() later, but for
 	 * purify reasons, we need to set it to something here 
 	 */
+
 	LAM_ZERO_ME(proc.asc_node);
-/*
- * Build the argument vector.
- */
+
+	/*
+	 * Build the argument vector.
+	 */
 	procargv = (struct apparg *) malloc(sizeof(struct apparg));
 	if (procargv == 0) {
-	    return(lam_mkerr(MPI_ERR_OTHER, errno));
+	  return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
 
 	procargv->apa_argv = 0;
 	procargv->apa_argc = 0;
 	procargv->apa_refcount = 1;
 
-	if (argvadd(&procargv->apa_argc, &procargv->apa_argv, command)) {
-	    argvfree(procargv->apa_argv);
-	    free((char *) procargv);
-	    return(lam_mkerr(MPI_ERR_OTHER, errno));
+	if (sfh_argv_add(&procargv->apa_argc, &procargv->apa_argv, command)) {
+	  sfh_argv_free(procargv->apa_argv);
+	  free((char *) procargv);
+	  return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
 
 	if (argv != MPI_ARGV_NULL) {
-	    argv_n = argvcount(argv);
-	    for (i = 0; i < argv_n; i++) {
-		if (argvadd(&procargv->apa_argc,
-		    &procargv->apa_argv, argv[i])) {
-		    argvfree(procargv->apa_argv);
-		    free((char *) procargv);
-		    return(lam_mkerr(MPI_ERR_OTHER, errno));
-		}
+	  argv_n = sfh_argv_count(argv);
+	  for (i = 0; i < argv_n; i++) {
+	    if (sfh_argv_add(&procargv->apa_argc,
+			     &procargv->apa_argv, argv[i])) {
+	      sfh_argv_free(procargv->apa_argv);
+	      free((char *) procargv);
+	      return(lam_mkerr(MPI_ERR_OTHER, errno));
 	    }
+	  }
 	}
 
 	proc.asc_args = procargv;
 
- /*
+/*
  * Build the process's env structure.
  */
 	procenv = (struct appenv *) malloc(sizeof(struct appenv));
 	if (procenv == 0) {
-	  argvfree(procargv->apa_argv);
+	  sfh_argv_free(procargv->apa_argv);
 	  free((char *) procargv);
 	  return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
@@ -819,26 +857,78 @@ build_app(MPI_Comm comm, char *command, char **argv, int maxprocs,
  */
 	*app = al_init(sizeof(struct aschema), (int (*)()) 0);
 	if (*app == 0) {
-	    argvfree(procargv->apa_argv);
-	    free((char *) procargv);
-	    free((char *) procenv);
-	    return(lam_mkerr(MPI_ERR_OTHER, errno));
+	  sfh_argv_free(procargv->apa_argv);
+	  free((char *) procargv);
+	  free((char *) procenv);
+	  return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
-
+	
 	if (al_append(*app, &proc) == 0) {
-	    asc_free(*app);
-	    argvfree(procargv->apa_argv);
-	    free((char *) procargv);
-	    free((char *) procenv);
-	    return(lam_mkerr(MPI_ERR_OTHER, errno));
+	  asc_free(*app);
+	  sfh_argv_free(procargv->apa_argv);
+	  free((char *) procargv);
+	  free((char *) procenv);
+	  return(lam_mkerr(MPI_ERR_OTHER, errno));
 	}
 
+	/* 
+	 * VPS: Make root node non-schedulable in the lamd route info
+	 */
+
+	if ((!sched_round_robin) && (no_root_node_schedule)) {	 
+
+	  MPI_Comm_get_attr(MPI_COMM_WORLD, LAM_UNIVERSE_NNODES, &attrp,
+			    &found);
+	  if (found == 0)
+	    return lam_mkerr(MPI_ERR_INTERN, ENOENT);
+
+	  max_nodes = (int) *attrp;
+
+	  routes = (struct route *) malloc((unsigned) max_nodes *
+					   sizeof(struct route));
+	  if (routes == NULL) {
+	    return lam_mkerr(MPI_ERR_NO_MEM, errno);
+	  }
+    
+	  nodes = (int *) malloc(sizeof(int) * max_nodes);
+	  if (nodes == NULL) {
+	    return lam_mkerr(MPI_ERR_NO_MEM, errno);
+	  }
+  
+	  getnodes(nodes, max_nodes, 0, NT_CAST);
+	  mynodeid = getnodeid();
+	  for (i = 0; i < max_nodes; ++i) {
+	    routes[i].r_nodeid = nodes[i];
+	    getrent(&routes[i]);
+	    if (nodes[i] == mynodeid) {
+	      actual_mynodetype = routes[i].r_nodetype; 
+	      mynodeindex = i;
+	      routes[i].r_nodetype = NT_WASTE;	
+	    }
+	  }
+    
+	  if (rrsetrents(mynodeid, routes, max_nodes))
+	    lampanic("spawn (rrsetrents)");
+	}
     }
-/*
- * Schedule the application.
- */
+    /*
+     * Schedule the application.
+     */
     app_sched = asc_schedule(*app);
     asc_free(*app);
+	  
+    /* 
+     * VPS: Make root back to whatever its nodetype was
+     */
+	  
+    if ((!sched_round_robin) && (no_root_node_schedule)) {	 
+      routes[mynodeindex].r_nodetype = actual_mynodetype;
+    
+      if (rrsetrents(mynodeid, routes, max_nodes))
+	lampanic("spawn (rrsetrents)");
+
+      free((char *) routes);
+    }
 
     *app = app_sched;
     return((*app) ? MPI_SUCCESS : lam_mkerr(MPI_ERR_OTHER, errno));
@@ -887,27 +977,29 @@ locate_aschema(char *filename)
 	char		*appdir;		/* application dir */
 	char		*fullpath;		/* full pathname */
 
+
 	if ((appdir = getenv("LAMAPPLDIR"))) {
-		argvadd(&pathc, &pathv, appdir);
+		sfh_argv_add(&pathc, &pathv, appdir);
 	}
-	argvadd(&pathc, &pathv, "");
-	argvadd(&pathc, &pathv, "$LAMHOME/etc");
-	argvadd(&pathc, &pathv, "$TROLLIUSHOME/etc");
+	sfh_argv_add(&pathc, &pathv, "");
+	sfh_argv_add(&pathc, &pathv, "$LAMHOME/etc");
+	sfh_argv_add(&pathc, &pathv, "$TROLLIUSHOME/etc");
 
 	fullpath = sfh_path_find(filename, pathv, R_OK);
-	argvfree(pathv);
+	sfh_argv_free(pathv);
 	return(fullpath);
 }
 
 
 static int
-get_sched_rrobin(struct aschema* proc, char* value, int maxprocs)
+get_sched_rrobin(struct aschema* proc, char* value, int maxprocs, int flag)
 {
   void **attrp;
   int i, j, start, num_left, max_cpus, max_nodes, found, node;
   struct ndi n;
   int *nodes = 0;
   struct route *routes = 0;
+  int mynodeid;
   
   /* If we got a negative starting number, ignore everything else */
 
@@ -960,10 +1052,18 @@ get_sched_rrobin(struct aschema* proc, char* value, int maxprocs)
 
   /* Query the lamd to get route information */
 
+  mynodeid = getnodeid();
   getnodes(nodes, max_nodes, 0, NT_CAST);
   for (i = 0; i < max_nodes; ++i) {
     routes[i].r_nodeid = nodes[i];
     getrent(&routes[i]);
+
+    /* VPS: If no-root-node-schedule, then make it non-scheduleable in the local
+       copy of routes */
+    
+    if ((flag) && (nodes[i] == mynodeid)) {
+      routes[i].r_nodetype = NT_WASTE;
+    }
   }
   free(nodes);
 
