@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: tgetname.c,v 1.3.2.1 2002/10/09 19:49:14 brbarret Exp $
+ * $Id: tgetname.c,v 1.6 2003/05/31 22:28:50 jsquyres Exp $
  *
  *	Function:	- get datatype name
  *	Accepts:	- datatype
@@ -28,6 +26,7 @@
 #include <blktype.h>
 #include <mpi.h>
 #include <mpisys.h>
+#include <etc_misc.h>
 
 
 /*@
@@ -67,15 +66,15 @@ MPI_Type_get_name(MPI_Datatype type, char *name, int *length)
  */
 	if (type == MPI_DATATYPE_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITGETNAME, lam_mkerr(MPI_ERR_TYPE, 0)));
+			BLKMPITGETNAME, lam_mkerr(MPI_ERR_TYPE, EINVAL)));
 	}
 
 	if ((name == 0) || (length == 0)) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITGETNAME, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPITGETNAME, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
-	strncpy(name, type->dt_name, MPI_MAX_OBJECT_NAME);
+	lam_strncpy(name, type->dt_name, MPI_MAX_OBJECT_NAME);
 	name[MPI_MAX_OBJECT_NAME - 1] = 0;
 	*length = strlen(name);
 
