@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1996 The Ohio State University
- *	RBD
+ * $HEADER$
  *
- *	$Id: topotest.c,v 6.3.2.1 2002/10/09 19:49:14 brbarret Exp $
+ * $Id: topotest.c,v 6.5 2002/12/11 19:15:14 jsquyres Exp $
  *
  *	Function:	- get topology assigned to communicator
  *	Accepts:	- communicator
@@ -62,12 +60,12 @@ int MPI_Topo_test(MPI_Comm comm, int *ptopo)
  */
 	if (comm == MPI_COMM_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITOPOTEST, lam_mkerr(MPI_ERR_COMM, 0)));
+			BLKMPITOPOTEST, lam_mkerr(MPI_ERR_COMM, EINVAL)));
 	}
 
 	if (ptopo == 0) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPITOPOTEST, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPITOPOTEST, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 
 	*ptopo = comm->c_topo_type;
