@@ -10,11 +10,9 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
+ * $HEADER$
  *
- *	$Id: wseterr.c,v 1.1.2.1 2002/10/09 19:49:16 brbarret Exp $
+ * $Id: wseterr.c,v 1.3 2002/12/11 19:15:15 jsquyres Exp $
  *
  *	Function:	- set error handler for window
  *	Accepts:	- communicator
@@ -56,12 +54,12 @@ int MPI_Win_set_errhandler(MPI_Win win,MPI_Errhandler errhdl)
  */
 	if (win == MPI_WIN_NULL) {
 		return(lam_errfunc(MPI_COMM_WORLD,
-			BLKMPIWINSETERRHDL, lam_mkerr(MPI_ERR_WIN, 0)));
+			BLKMPIWINSETERRHDL, lam_mkerr(MPI_ERR_WIN, EINVAL)));
 	}
 
 	if (errhdl == MPI_ERRHANDLER_NULL || !(errhdl->eh_flags & LAM_EHWIN)) {
 		return(lam_errfunc(win->w_comm,
-			BLKMPIWINSETERRHDL, lam_mkerr(MPI_ERR_ARG, 0)));
+			BLKMPIWINSETERRHDL, lam_mkerr(MPI_ERR_ARG, EINVAL)));
 	}
 /*
  * Free previous error handler.
