@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  *
  * $HEADER$
  *
- * $Id: ssi_boot_bproc.c,v 1.23.2.1 2003/06/27 01:46:17 jsquyres Exp $
+ * $Id: ssi_boot_bproc.c,v 1.25 2003/07/11 19:40:25 jsquyres Exp $
  *
  *	Function:	- bproc boot SSI module
  */
@@ -53,11 +53,14 @@
 /*
  * Local variables
  */
+static int param_priority;
+static int param_perm_check;
+static int do_perms_checking;
 static char *bhost = NULL;
 static char *hostname = NULL;
 static int origin = -1;
 static struct lamnode origin_node;
-static const lam_ssi_boot_actions_1_0_0_t module_actions = {
+static const lam_ssi_boot_actions_1_1_0_t module_actions = {
 
   lam_ssi_boot_bproc_parse_options,
   lam_ssi_boot_bproc_allocate_nodes,
@@ -72,6 +75,7 @@ static const lam_ssi_boot_actions_1_0_0_t module_actions = {
   /* The rest of the functions use the generic TCP protocol functions
      that are common to all boot SSI's */
 
+  lam_ssi_boot_base_get_nodeid,
   lam_ssi_boot_base_open_srv_connection,
   lam_ssi_boot_base_send_lamd_info,
   lam_ssi_boot_base_receive_lamd_info,
@@ -100,14 +104,27 @@ static char *nodelist_string(int *nodes, int nnodes);
 extern char **environ;
 
 
+int 
+lam_ssi_boot_bproc_open_module(OPT *ad)
+{
+  param_priority =
+    lam_ssi_base_param_register_int("boot", "bproc", "priority", NULL, 
+                                    LAM_SSI_BOOT_BPROC_DEFAULT ? 75 : 50);
+
+  param_perm_check = 
+    lam_ssi_base_param_register_int("boot", "bproc", "perm_check", NULL,
+                                    LAM_HAVE_BPROC_NODEINFO);
+
+  return 1;
+}
+
+
 /***********************************************************************
  *
  * External interface
  *
- *
  ***********************************************************************/
 
-
 /*
  * lam_ssi_boot_bproc_init
  *
@@ -128,10 +145,12 @@ lam_ssi_boot_bproc_init(lam_ssi_boot_location_t where, int *priority)
 
   /* Check this module's priority before doing anything else */
 
-  if (lam_ssi_boot_base_check_priority("bproc", 50, LAM_SSI_BOOT_BPROC_DEFAULT,
-				       priority) < 0)
+  *priority = lam_ssi_base_param_lookup_int(param_priority);
+  if (*priority < 0)
     return NULL;
 
+  do_perms_checking = lam_ssi_base_param_lookup_int(param_perm_check);
+
   /* Fire up BPROC interface -- but only on the root! */
 
   if (where == LAM_SSI_BOOT_LOCATION_ROOT) {
@@ -307,6 +326,10 @@ lam_ssi_boot_bproc_verify_nodes(struct lamnode *nodes, int nnodes)
   uid_t uid;
   gid_t gid;
   char *tmp;
+#if ! LAM_HAVE_BPROC3_API
+  char node_status[100];
+  int ret;
+#endif
 
   if (lam_ssi_boot_verbose >= 1) {
     lam_debug(lam_ssi_boot_did, "bproc: resolved hosts:");
@@ -339,14 +362,56 @@ lam_ssi_boot_bproc_verify_nodes(struct lamnode *nodes, int nnodes)
       continue;
 
     target_node = local_bproc_resolve(nodes[i].lnd_hname);
-    if (bproc_nodestatus(target_node) != bproc_node_up)
+    if (target_node == BPROC_NODE_NONE) {
+      /* bproc gets really unhappy if NODE_NONE is used in the
+         status calls - make sure that doesn't happen */
+      lam_debug(lam_ssi_boot_did, 
+                "bproc: %s was not resolveable to node number\n",
+                nodes[i].lnd_hname);
       sfh_argv_add(&node_downc, &node_downv, nodes[i].lnd_hname);
+      continue;
+    } else if (target_node == BPROC_NODE_MASTER) {
+      /* master node is up and owned by all if we got this far.
+         Some of the bproc calls don't like NODE_MASTER, so keep
+	 it away from those functions*/
+      lam_debug(lam_ssi_boot_did,
+                "bproc: found master node (%s).  Skipping checks.\n",
+		nodes[i].lnd_hname);
+      continue;
+    }
+   
 
-    /* the master is owned by all.... */
+#if LAM_HAVE_BPROC3_API
+    if (bproc_nodestatus(target_node) != bproc_node_up) {
+      sfh_argv_add(&node_downc, &node_downv, nodes[i].lnd_hname);
+    }
+#else
+    ret = bproc_nodestatus(target_node, node_status, 100);
+    node_status[99] = '\0';
+    if (ret < 0) {
+      lam_debug(lam_ssi_boot_did, "bproc: n%d nodestatus failed (%s)\n",
+                target_node, strerror(errno));
+    } else {
+      lam_debug(lam_ssi_boot_did, "bproc: n%d node status: %s\n",
+                target_node, node_status);
+    }
+    if ((ret < 0) || (strcmp(node_status, "down") == 0)) {
+      lam_debug(lam_ssi_boot_did, "bproc: n%d node status down, failure\n",
+                target_node);
+      sfh_argv_add(&node_downc, &node_downv, nodes[i].lnd_hname);
+    }
+#endif
 
-    else if (target_node != -1){
-      if (bproc_nodeinfo(target_node, &ninfo) < 0)
+    else if (do_perms_checking) {
+      /* BWB - debugging - think bproc_nodeinfo is going insane.  Let's
+         see if this helps with the errno */
+      errno = 0;
+      if (bproc_nodeinfo(target_node, &ninfo) < 0) {
+        lam_debug(lam_ssi_boot_did, "bproc: n%d bproc_nodeinfo failed (%s)\n",
+                  target_node, strerror(errno));
         sfh_argv_add(&node_downc, &node_downv, nodes[i].lnd_hname);
+        continue;
+      }
 
       /* make sure user, group and mode are ok dokay.  To make this
          test simple, look for any "good" condition, and simply see if
@@ -355,8 +420,18 @@ lam_ssi_boot_bproc_verify_nodes(struct lamnode *nodes, int nnodes)
       if (! (((ninfo.user == uid) || (ninfo.mode & S_IXUSR) != 0) ||
              ((ninfo.group == gid) || (ninfo.mode & S_IXGRP) != 0) ||
              (ninfo.mode & S_IXOTH) != 0)) { 
+        lam_debug(lam_ssi_boot_did, "bproc: n%d access not sane\n",
+                  target_node);
         sfh_argv_add(&node_permc, &node_permv, nodes[i].lnd_hname);
+        continue;
       }
+
+      /* Woo!  We passed permissions checks. */
+      lam_debug(lam_ssi_boot_did, "bproc: n%d access rights are sane.\n",
+                target_node);
+    } else {
+      lam_debug(lam_ssi_boot_did, "bproc: n%d access rights not checked.\n",
+                target_node);
     }
   }
 
@@ -789,6 +864,7 @@ start_app_many_nodes(char ***argv, int *argc, int num_procs,
 static int 
 local_bproc_resolve(char *name)
 {
+#if LAM_HAVE_BPROC3_API
   int node;
   struct sockaddr_in s;
   struct hostent *name_resolver;
@@ -809,6 +885,54 @@ local_bproc_resolve(char *name)
   } 
 
   return node;
+#else
+  struct hostent *name_resolver;
+  struct bproc_node_set_t list;
+  struct sockaddr_in master_in;
+  int master_in_len;  
+  int i, j;
+
+  /* add in some shortcut checks */
+  if(!strcmp("master",name)) {
+    return BPROC_NODE_MASTER;
+  } else if(!strcmp("self",name)) {
+    return BPROC_NODE_SELF;
+  }
+
+  name_resolver = gethostbyname(name);
+  if (name_resolver == NULL)
+    return BPROC_NODE_NONE;
+
+  /* see if we are the master */
+  master_in_len = sizeof(master_in);
+  if (bproc_nodeaddr(BPROC_NODE_MASTER, (struct sockaddr*) &master_in, 
+                     &master_in_len) < 0) {
+    perror("bproc_nodeaddr");
+    return BPROC_NODE_NONE;
+  }
+  for (j = 0 ; name_resolver->h_addr_list[j] != NULL ; ++j) {
+    if (memcmp(name_resolver->h_addr_list[j], &master_in.sin_addr, 
+               name_resolver->h_length) == 0) {
+      return BPROC_NODE_MASTER;
+    }
+  }
+
+  if (bproc_nodelist(&list) == -1) {
+    perror("bproc_nodelist");
+    return BPROC_NODE_NONE;
+  }
+
+  for (j = 0 ; name_resolver->h_addr_list[j] != NULL ; ++j) {
+    for (i=0; i < list.size; i++) {
+      struct sockaddr_in *sin = (struct sockaddr_in *)&list.node[i].addr;
+      if (memcmp(name_resolver->h_addr_list[j], &sin->sin_addr, 
+                 name_resolver->h_length) == 0) {
+        return list.node[i].node;
+      }
+    }
+  }
+  return BPROC_NODE_NONE;
+#endif
 }
 
 
