@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: ssi_crlam_close.c,v 1.1 2003/04/02 02:41:55 ssankara Exp $
+ * $Id: ssi_crlam_close.c,v 1.4 2003/12/04 19:54:49 jsquyres Exp $
  *
  */
 
@@ -29,26 +29,29 @@ lam_ssi_crlam_base_close(void)
 {
   int ret = 0;
 
-  /* 
-   * SS: currently, the only module available is blcr, and it requires the
-   * PTHREADS package. However, in future, this condition might have to be
-   * modified in the context of other modules becoming available.
-   */
+  /* SS: currently, the only module available is blcr, and it requires
+     the PTHREADS package. However, in future, this condition might
+     have to be modified in the context of other modules becoming
+     available. */
+
 #if LAM_HAVE_POSIX_THREADS
   /* Call close on the active module */
+
   if (lam_ssi_crlam_base_module.lscrl_meta_info.ssi_close_module != NULL)
     lam_ssi_crlam_base_module.lscrl_meta_info.ssi_close_module();
 
+  /* Close the module if it's dynamic */
+
+  lam_ssi_base_module_registry_unuse((lam_ssi_t *) &lam_ssi_crlam_base_module);
+
   /* Close the debug stream */
+
   if (lam_ssi_cr_verbose >= 1)
     lam_debug(lam_ssi_cr_did, " Closing");
   if (lam_ssi_cr_did >= 0) {
     lam_debug_close(lam_ssi_cr_did);
     lam_ssi_cr_did = -1;
   }
-
-  /* Shut down the SSI glue */
-  lam_ssi_base_close();
 #endif
 
   return ret;
