@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: ssi_crlam_blcr.c,v 1.5.2.3 2003/11/11 13:13:03 jsquyres Exp $
+ * $Id: ssi_crlam_blcr.c,v 1.16 2004/03/07 07:05:53 pcharapa Exp $
  *
  *	Function:	- BLCR crlam module 
  */
@@ -29,6 +29,7 @@
 #include <app_mgmt.h>
 #include <etc_misc.h>
 #include <priority.h>
+#include <freq.h>
 #include <preq.h>
 #include <sfh.h>
 
@@ -38,8 +39,6 @@
 
 #include <libcr.h>
 
-#define signal_puts(S)  write(STDOUT_FILENO, (S), strlen(S))
-
 
 /*
  * local functions
@@ -47,9 +46,11 @@
  static int crlam_thread_callback(void *arg);
  static int crlam_signal_callback(void *arg);
  static int cr_signal_app(struct _gps *, int);
- static char *get_checkpoint_file(int node, int pid);
+ static char *get_checkpoint_file(int node, int pid, const char *suffix);
+ static char *trim_filename(const char *name, const char *suffix);
  static char *get_cr_util(const char* util);
  static char *create_app_schema(struct _gps *, int);
+ static void cleanup(char **fname_arr, struct _gps *world, int n);
 
 
 /*
@@ -63,9 +64,13 @@ static lam_ssi_crlam_actions_t module_actions = {
   lam_ssi_crlam_blcr_finalize,
   lam_ssi_crlam_blcr_init,
   /* use the base utility function for restart action */
-  lam_ssi_crlam_base_do_exec
+  lam_ssi_crlam_base_do_exec,
+  lam_ssi_crlam_blcr_lamcheckpoint,
+  lam_ssi_crlam_blcr_lamrestart
 };
 
+static int param_priority;
+static int param_base_dir;
 static struct _gps *cr_world = NULL;
 static int cr_world_n = -1;
 static int app_session = 0;
@@ -74,6 +79,22 @@ static char *executable;
 static cr_client_id_t client_id;
 
 
+/*
+ * Open the module and register SSI parameters
+ */
+int
+lam_ssi_crlam_blcr_open_module(OPT *ad)
+{
+  param_base_dir = 
+    lam_ssi_base_param_register_string("cr", "blcr", "base_dir", NULL, NULL);
+  param_priority = 
+    lam_ssi_base_param_register_int("cr", "blcr", "priority", NULL,
+                                    LAM_SSI_CRLAM_BLCR_DEFAULT ? 75 : 50);
+  
+  return 1;
+}
+
+
 /*
  * lam_ssi_crlam_blcr_query
  *
@@ -83,8 +104,9 @@ const lam_ssi_crlam_actions_t *
 lam_ssi_crlam_blcr_query(int *priority)
 {
   /* Check this module's priority before doing anything else */
-  if (lam_ssi_cr_base_check_priority("blcr", 50, LAM_SSI_CRLAM_BLCR_DEFAULT,
-                                      priority) < 0)
+
+  *priority = lam_ssi_base_param_lookup_int(param_priority);
+  if (*priority < 0)
     return NULL;
 
   /* If we're verbose, print stuff out */
@@ -200,24 +222,28 @@ int lam_ssi_crlam_blcr_checkpoint(void)
     if (cr_signal_app(cr_world, cr_world_n) != 0)
       kexit(1);
 
-    lam_debug(lam_ssi_cr_did, "Child exiting\n");
+    if (lam_ssi_cr_verbose >= 40)
+      lam_debug(lam_ssi_cr_did, "Child exiting\n");
     kexit(0);
 
   } 
 
   /* Parent process */
 
-  lam_debug(lam_ssi_cr_did, "Parent waiting for pid %d\n", pid);
+  if (lam_ssi_cr_verbose >= 40)
+    lam_debug(lam_ssi_cr_did, "Parent waiting for pid %d\n", pid);
   if (waitpid(pid, &status, 0) != pid) {
     lam_perror("async_checkpoint waitpid");
     lam_debug(lam_ssi_cr_did, "waitpid(%d, ...) failed: %d",
               pid, errno);
     return LAMERROR;
   }
-  lam_debug(lam_ssi_cr_did, "Parent reaped pid %d\n", pid);
-
+  
+  if (lam_ssi_cr_verbose >= 40)
+    lam_debug(lam_ssi_cr_did, "Parent reaped pid %d\n", pid);
+  
   if ((WIFEXITED(status) != 0) && (WEXITSTATUS(status) != 0)) {
-    lam_debug(lam_ssi_cr_did, "Bad exit status from child: %d. Aborting...",
+    lam_debug(lam_ssi_cr_did, "Bad exit status from child: %d.",
               WEXITSTATUS(status));
     return LAMERROR;
   }
@@ -229,7 +255,8 @@ int lam_ssi_crlam_blcr_checkpoint(void)
   chkpnt_app_schema = create_app_schema(cr_world, cr_world_n);
   if (chkpnt_app_schema == NULL) {
     perror("create_app_schema");
-    lam_debug(lam_ssi_cr_did, "Couldn't make app schema!");
+    if (lam_ssi_cr_verbose >= 0)
+      lam_debug(lam_ssi_cr_did, "Couldn't make app schema!");
     return LAMERROR;
   }
   
@@ -292,34 +319,40 @@ crlam_thread_callback(void *arg)
 {
   int ret;
 
-  lam_debug(lam_ssi_cr_did, "Started async C/R handler");
+  if (lam_ssi_cr_verbose >= 0)
+    lam_debug(lam_ssi_cr_did, "Started async C/R handler");
 
   /* call the glue checkpoint function */
   if (lam_ssi_crlam_base_checkpoint(cr_world, cr_world_n) != 0) {
-    lam_debug(lam_ssi_cr_did, "mpirun_ASYNC%d: prepare for chkpt failed.",
+    if (lam_ssi_cr_verbose >= 0)
+      lam_debug(lam_ssi_cr_did, "mpirun_ASYNC%d: prepare for chkpt failed.",
               getpid());
     kexit(1);
   }
 
-  lam_debug(lam_ssi_cr_did, "mpirun_async calling cr_checkpoint()");
+  if (lam_ssi_cr_verbose >= 10)
+    lam_debug(lam_ssi_cr_did, "mpirun_async calling cr_checkpoint()");
   ret = cr_checkpoint(0);
     
   if (ret < 0) {
     /* BWB: error handling */
     /* SS: we should probably continue, instead of aborting. */
-    lam_debug(lam_ssi_cr_did, "mpirun_async failed checkpoint - %d", ret);
+    if (lam_ssi_cr_verbose >= 0)
+      lam_debug(lam_ssi_cr_did, "mpirun_async failed checkpoint - %d", ret);
     kexit(1);
   } else if (ret != 0) {
       /* no-op. restart-work is done only in the signal-handler context */
   } else {
     if (lam_ssi_crlam_base_continue() != 0) {
-      lam_debug(lam_ssi_cr_did, "mpirun_ASYNC%d: prepare for continue "
-                "failed.", getpid());
+      if (lam_ssi_cr_verbose >= 0)
+        lam_debug(lam_ssi_cr_did, "mpirun_ASYNC%d: prepare for continue "
+                  "failed.", getpid());
       kexit(1);
     }
   }
 
-  lam_debug(lam_ssi_cr_did, "Ending mpirun_async C/R handler");
+  if (lam_ssi_cr_verbose >= 0)
+    lam_debug(lam_ssi_cr_did, "Exiting mpirun_async C/R handler");
   return 0;
 }
 
@@ -329,8 +362,9 @@ crlam_signal_callback(void *arg)
 {
   int ret;
 
-  /* nothing to be done before checkpointing. */
+  lam_debug(lam_ssi_cr_did, "Started sync C/R handler");
   
+  /* nothing to be done before checkpointing. */
   ret = cr_checkpoint(0);
 
   if (ret < 0) {
@@ -343,9 +377,11 @@ crlam_signal_callback(void *arg)
       lam_debug(lam_ssi_cr_did, "mpirun_SYNC: restart failed.");
   } else {
     /* no-op. */
-    lam_debug(lam_ssi_cr_did, "mpirun_SYNC: continuing...");
+    if (lam_ssi_cr_verbose >= 10)
+      lam_debug(lam_ssi_cr_did, "mpirun_SYNC: continuing...");
   }
 
+  lam_debug(lam_ssi_cr_did, "Exiting sync C/R handler");
   return 0;
 }
 
@@ -364,10 +400,11 @@ cr_signal_app(struct _gps *cr_world, int cr_world_n)
   int cr_argc = 0;
   struct penv cr_env;
   char pid_arg[sizeof(int) * 8];
-  char *cr_checkpoint, *cr_file;
+  char *cr_checkpoint, **cr_filearr;
   struct _gps       *p;
   int cr_pid, cr_idx;
   int nid, pid, status;
+  char *suffix = ".phase1";
 
   cr_checkpoint = get_cr_util("cr_checkpoint");
   if (cr_checkpoint == NULL) {
@@ -375,6 +412,9 @@ cr_signal_app(struct _gps *cr_world, int cr_world_n)
     return LAMERROR;
   }
 
+  /* Initialize the array of checkpoint-file names */
+  cr_filearr = (char **) malloc(sizeof(char *) * cr_world_n);
+
   for(i = 0, p = cr_world; i < cr_world_n; ++i, ++p) {
     /* setup kenya request */
     cr_env.pe_rank = p->gps_node;
@@ -382,66 +422,138 @@ cr_signal_app(struct _gps *cr_world, int cr_world_n)
     cr_env.pe_envv = 0;
 
     /* Set up program arguments */
+
     sfh_argv_add(&cr_argc, &cr_argv, cr_checkpoint);
 
     snprintf(pid_arg, sizeof(pid_arg), "%d", p->gps_pid);
     sfh_argv_add(&cr_argc, &cr_argv, pid_arg);
 
-    cr_file = get_checkpoint_file(p->gps_node, p->gps_pid);
-    if (cr_file != NULL) {
-      lam_debug(lam_ssi_cr_did, "file: %s\n", cr_file);
+    /*
+     * Saving of checkpoint-files follows a two-phase commit approach.
+     * While the  approach used here gives a reasonable  guarantee of a
+     * consistent set of saved checkpoints, there is atleast one case in
+     * which the global state represented by these checkpoints might not be
+     * consistent.  
+     *
+     * The checkpoints are initially saved with a ".phase1" suffix.  Once it
+     * has been determined that all the ranks saved the checkpoints
+     * properly, the checkpoint-files are renamed by stripping the suffix.
+     *
+     * The rename of checkpoint-files is not atomic.  So, if there is a
+     * failure in mpirun (atleast for now, it's only mpirun) during this
+     * time, the saved set of checkpoints is not guaranteed to be
+     * consistent.
+     */
+
+    cr_filearr[i] = get_checkpoint_file(p->gps_node, p->gps_pid, suffix);
+
+    if (cr_filearr[i] != NULL) {
+      if (lam_ssi_cr_verbose >= 40)
+        lam_debug(lam_ssi_cr_did, "checkpoint file: %s\n", cr_filearr[i]);
       sfh_argv_add(&cr_argc, &cr_argv, "-f");
-      sfh_argv_add(&cr_argc, &cr_argv, cr_file);
+      sfh_argv_add(&cr_argc, &cr_argv, cr_filearr[i]);
     } else {
+      int j;
+      /* cleanup the array of checkpoint-file names */
+      for (j = 0; j < i; ++j)
+        free(cr_filearr[j]);
+      free(cr_filearr);
       lam_perror("Getting checkpoint file");
       return LAMERROR;
     }
-
+    
     /*
      * It is safe to expect cr kernel module to be present on all nodes, so
      * src_node can be the same as dst_node when invoking rploadgo
      */
-    lam_debug(lam_ssi_cr_did, "Invoking %s %d -f %s on node:%d\n",
-              cr_checkpoint, p->gps_pid, cr_file, p->gps_node);
-
+    if (lam_ssi_cr_verbose >= 10)
+      lam_debug(lam_ssi_cr_did, "Invoking %s %d -f %s on node:%d\n",
+                cr_checkpoint, p->gps_pid, cr_filearr[i], p->gps_node);
+    
     if (rploadgov(p->gps_node, p->gps_node, cr_checkpoint, RTF_WAIT, cr_argv,
                   &cr_env, &cr_pid, &cr_idx)) {
       /* We can't do anything more than abort... */
       lam_perror("rploadgov failed.");
+      cleanup(cr_filearr, cr_world, i);
       return LAMERROR;
     }
-
-    free(cr_file);
+    
     free(cr_argv);
     cr_argv = NULL;
     cr_argc = 0;
   }
+  
+  if (lam_ssi_cr_verbose >= 10)
+    lam_debug(lam_ssi_cr_did, "Child rpwaiting\n");
+  
+  /* a stripped down version of the pwait() function used in
+     mpirun. */
 
-  lam_debug(lam_ssi_cr_did, "Child rpwaiting\n");
-
-  /* a stripped down version of the pwait() function used in  mpirun. */
   for (i = 0; i < cr_world_n; ++i) {
     if (rpwait(&nid, &pid, &status)) {
       lam_perror("rpwait failed");
+      cleanup(cr_filearr, cr_world, cr_world_n);
       free(cr_checkpoint);
       return LAMERROR;
     }
 
-    /*
-     * if return status of the child is non-zero, then cr_checkpoint did not
-     * complete successfully.
-     */
+    /* if return status of the child is non-zero, then cr_checkpoint
+       did not complete successfully. */
+
     if (status != 0) {
       lam_perror("rpwait failed");
+      cleanup(cr_filearr, cr_world, cr_world_n);
       free(cr_checkpoint);
       return LAMERROR;
     }
   }
 
-  lam_debug(lam_ssi_cr_did, "Child done pwaiting\n");
+  if (lam_ssi_cr_verbose >= 10)
+    lam_debug(lam_ssi_cr_did, "Child done rpwaiting\n");
 
+  /* Now that we know that all the local checkpoints were successful,
+     we can perform the second phase of the 2-phase commit now.  Loop
+     over renaming the checkpoint-files of all ranks. */
+
+  for (i = 0, p = cr_world; i < cr_world_n; ++i, ++p) {
+
+    char *src, *dest;
+    int len;
+
+    /* Assume nid takes 8 * sizeof(int) chars to print */
+
+    len = strlen(cr_filearr[i]) + sizeof(int) * 8 + 2;
+    src = (char *) malloc(sizeof(char) * len);
+
+    snprintf(src, len, "%d:%s", p->gps_node, cr_filearr[i]);
+    dest = trim_filename(src, suffix);
+
+    if (lam_ssi_cr_verbose >= 40)
+      lam_debug(lam_ssi_cr_did, "Renaming %s to %s on node %d", 
+                src, dest, p->gps_node);
+    
+    if (lam_rfrename(src, dest) < 0) {
+
+      /* Unless this is the first iteration of the loop, the set of
+         saved checkpoints is not consistent anymore.  We are doomed
+         :-( */
+
+      cleanup(cr_filearr, cr_world, cr_world_n);
+      free(src);
+      free(dest);
+      free(cr_checkpoint);
+      return LAMERROR;
+    }
+
+    free(src);
+    free(dest);
+  }
+  
+  /* Unlink phase-1 checkpoints on all nodes and cleanup the array of
+     checkpoint-file names */
+
+  cleanup(cr_filearr, cr_world, cr_world_n);
   free(cr_checkpoint);
-
   return 0;
 }
 
@@ -460,14 +572,14 @@ cr_signal_app(struct _gps *cr_world, int cr_world_n)
  *  caller must free() returned string
  */
 static char*
-get_checkpoint_file(int node, int pid)
+get_checkpoint_file(int node, int pid, const char *suffix)
 {
   char *tmp = NULL;
   char *crdir = NULL;
   int str_len = 0;
   struct stat crd;
 
-  crdir = getenv("LAM_MPI_SSI_cr_base_dir");
+  crdir = lam_ssi_base_param_lookup_string(param_base_dir);
 
   if ((crdir != NULL) && (strlen(crdir) > 0)
       && (0 == stat(crdir, &crd)) && (S_ISDIR(crd.st_mode))) {
@@ -496,13 +608,17 @@ get_checkpoint_file(int node, int pid)
      files.  Assume that p->gps_pid could take sizeof(int) * 8 char to
      print */
 
-  str_len = strlen(crdir) + strlen("/context.") + sizeof(int) * 8 * 3 + 4 ;
+  str_len = strlen(crdir) + strlen("/context.") + sizeof(int) * 8 * 3 + 4;
+  if (suffix != NULL)
+    str_len += strlen(suffix);
   tmp = malloc(sizeof(char) * str_len);
   if (tmp == NULL)
     return NULL;
 
   snprintf(tmp, str_len, "%s/context.%d-n%d-%d",
            crdir, app_session, node, pid);
+  if (suffix != NULL)
+    strcat(tmp, suffix);
   if (lam_ssi_cr_verbose >= 10)
     lam_debug(lam_ssi_cr_did, "setting checkpoint_file to %s", tmp);
 
@@ -510,6 +626,26 @@ get_checkpoint_file(int node, int pid)
 }
 
 
+static char *
+trim_filename(const char *name, const char *suffix)
+{
+  int len;
+  char *dst;
+  char *pos;
+
+  pos = strstr(name, suffix);
+  if (pos == NULL)
+    return strdup(name);
+
+  len = pos - name;
+  dst = (char *) malloc((len + 1) * sizeof(char));
+  strncpy(dst, name, len);
+  dst[len] = 0;
+
+  return dst;
+}
+
+
 static char *
 get_cr_util(const char *util)
 {
@@ -518,9 +654,9 @@ get_cr_util(const char *util)
 #ifdef LAM_LBNL_LIBCR_PATH
   int str_len;
 
-    /* Hard-code directory where to find cr_checkpoint binary.  BWB: This
-     * should be removed after development
-     */
+  /* Hard-code directory where to find cr_checkpoint binary.  BWB:
+     This should be removed after development */
+
   str_len = strlen(LAM_LBNL_LIBCR_PATH) + strlen("/bin/") + strlen(util) + 1;
 
   tmp = malloc(sizeof(char) * str_len);
@@ -528,13 +664,11 @@ get_cr_util(const char *util)
     return tmp;
 
   snprintf(tmp, str_len, "%s%s%s", LAM_LBNL_LIBCR_PATH, "/bin/", util);
-
 #else
-
-    tmp = strdup(util);
+  tmp = strdup(util);
 #endif
 
-    return tmp;
+  return tmp;
 }
 
 
@@ -559,7 +693,7 @@ create_app_schema(struct _gps *cr_world, int cr_world_n)
 
   for (i = 0, p = cr_world; i < cr_world_n; ++i, ++p) {
     /* get file name */
-    cr_file = get_checkpoint_file(p->gps_node, p->gps_pid);
+    cr_file = get_checkpoint_file(p->gps_node, p->gps_pid, NULL);
     if (cr_file == NULL) {
       lam_perror("Getting checkpoint file name");
       kexit(1);
@@ -597,7 +731,8 @@ create_app_schema(struct _gps *cr_world, int cr_world_n)
   chkpnt_app_schema = tmp_as;
 
   if (chkpnt_app_schema == NULL) {
-    lam_debug(lam_ssi_cr_did, "checkpoint app schema build failed");
+    if (lam_ssi_cr_verbose >= 0)
+      lam_debug(lam_ssi_cr_did, "checkpoint app schema build failed");
     return NULL;
   }
 
@@ -605,3 +740,166 @@ create_app_schema(struct _gps *cr_world, int cr_world_n)
 
   return tmp_as;
 }
+
+
+static void
+cleanup(char **fname_arr, struct _gps *world, int n)
+{
+  int i;
+  struct _gps *p;
+
+  /* 
+   * Unlink phase-1 checkpoints of ranks 0 through n-1 and cleanup the
+   * array of checkpoint-file names.
+   */
+  for (i = 0, p = world; i < n; ++i, ++p) {
+    char *fname;
+    int len;
+
+    len = strlen(fname_arr[i]) + sizeof(int) * 8 + 2;
+    fname = (char *) malloc(sizeof(char) * len);
+
+    snprintf(fname, len, "%d:%s", p->gps_node, fname_arr[i]);
+    if (lam_ssi_cr_verbose >= 40)
+      lam_debug(lam_ssi_cr_did, "unlinking %s", fname);
+    lam_rfunlink(fname);
+    free(fname);
+    free(fname_arr[i]);
+  }
+  
+  free(fname_arr);
+  return;
+} 
+
+/*
+ * lam_ssi_crlam_blcr_lamcheckpoint
+ * 
+ * Function - Calls cr_checkpoint 
+ * Accepts - the pid of mpirun
+ */ 
+int
+lam_ssi_crlam_blcr_lamcheckpoint(pid_t pid)
+{
+  char *call_checkpoint;
+  int str_len = 0;
+  char *tmp = NULL;
+  char *crdir = NULL;
+  struct stat crd;
+  
+  crdir = lam_ssi_base_param_lookup_string(param_base_dir);
+  
+  if ((crdir != NULL) && (strlen(crdir) > 0)
+      && (0 == stat(crdir, &crd)) && (S_ISDIR(crd.st_mode))) {
+    
+    /* Empty conditional here makes it easier to make the env variable
+       / SSI parameter the default choice */
+  }
+  
+#ifdef LAM_CHECKPOINT_FILE_DIR
+  else if ((strlen(LAM_CHECKPOINT_FILE_DIR) != 0)
+	   && (0 == stat(LAM_CHECKPOINT_FILE_DIR, &crd))
+	   && (S_ISDIR(crd.st_mode))) {
+    crdir = LAM_CHECKPOINT_FILE_DIR;
+  } 
+#endif
+  
+  /* Otherwise, use the $HOME directory */
+  
+  else
+    crdir = getenv("HOME");
+  
+  if (lam_ssi_cr_verbose >= 10)
+    lam_debug(lam_ssi_cr_did, "setting checkpoint_file_dir to %s", crdir);
+  
+  
+  /* crdir now points to the base directory where to store the context
+     files.  Assume that pid could take sizeof(int) * 8 char to
+     print */
+  
+  str_len = strlen(crdir) + strlen("/context.") + strlen("mpirun.") 
+    + (sizeof(int) * 8) + 1;
+  
+  tmp = malloc(sizeof(char) * str_len);
+  
+  if (tmp == NULL) {
+    lam_perror("mallocing temp");
+    kexit(1);
+  }
+  
+  snprintf(tmp, str_len, "%s/context.%s.%d",
+           crdir, "mpirun", pid);
+  
+  str_len += strlen("cr_checkpoint -f ") + (sizeof(int) * 8) + strlen(" ") + 1;
+  call_checkpoint = malloc(sizeof(char) * str_len);
+  
+  if (call_checkpoint == NULL) {
+    free(tmp);			
+    return -1;
+  }
+  
+  snprintf(call_checkpoint, str_len, "%s%s%s%d", 
+	   "cr_checkpoint -f ", tmp, " ", pid); 
+  
+  if (system(call_checkpoint) == -1) {
+    free(call_checkpoint);
+    free(tmp);
+    return -1;
+  }
+  
+  free(tmp);
+  free(call_checkpoint);
+  return 0;
+}
+
+
+/*
+ * lam_ssi_crlam_blcr_lamrestart
+ * 
+ * Function - Calls cr_restart
+ */ 
+int 
+lam_ssi_crlam_blcr_lamrestart(char *temp)
+{
+  int param_context_file;
+  int str_len = 0;
+  char *context_file;
+  char *tmp;
+
+  /* Register the parameter for the name of the Context file */  
+  param_context_file = lam_ssi_base_param_register_string("cr", "blcr", 
+      "context_file", NULL, NULL);
+
+  /* Lookup the SSI parameter for the name of the Context file */
+  context_file =
+    lam_ssi_base_param_lookup_string(param_context_file);
+
+  if (context_file != NULL && strlen(context_file) != 0) {
+    str_len = strlen("cr_restart ") + strlen(context_file) + 1;
+    
+    tmp = malloc(str_len * sizeof(char));
+    
+    if (tmp == NULL) {
+      lam_perror("mallocing temp");
+      free(context_file);
+      kexit(1);
+    }
+    
+    /* cr_restart takes the name of the Context file as argument */
+    snprintf(tmp, str_len, "%s%s", "cr_restart ", context_file);
+ 
+    if (system(tmp) == -1) {
+      free(tmp);
+      free(context_file);
+      lam_perror("Delivering cr_restart");
+      kexit(1);
+    }
+  } else {
+    /* Nothing we can do, except to abort */
+    show_help("lamrestart", "usage", NULL);
+    kexit(1);
+  }
+
+  free(tmp);
+  free(context_file);
+	return 0;
+}
