@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: ssi_rpi_gm.c,v 1.41.2.13 2004/03/12 01:19:11 vsahay Exp $
+ * $Id: ssi_rpi_gm.c,v 1.72 2004/02/14 16:01:58 jsquyres Exp $
  *
  *	Function:	- GM client-to-client interface
  */
@@ -38,17 +38,16 @@
 
 #include <rpi_gm.h>
 #include <rpi_gm_actions.h>
-#include <rpi_gm_fast.h>
 #include <rpi_gm_interval.h>
 #include <rpi_gm_recv_queue.h>
 #include <rpi_gm_recv_events.h>
 #include <rpi_gm_send_queue.h>
 #include <rpi_gm_tiny.h>
-#include <rpi_gm_short.h>
 #include <rpi_gm_long.h>
 #include <rpi_gm_dreg.h>
 #include <rpi_gm_unexpected.h>
 #include <rpi_gm_util.h>
+#include <rpi_gm_cbuf.h>
 
 
 /*
@@ -71,33 +70,35 @@ int lam_ssi_rpi_gm_did = -1;
 int lam_ssi_rpi_gm_haveadv = 0;
 int lam_ssi_rpi_gm_flblock = 0;
 int lam_ssi_rpi_gm_stokens = 0;
+int lam_ssi_rpi_gm_max_stokens = 0;
 int lam_ssi_rpi_gm_rtokens = 0;
+int lam_ssi_rpi_gm_max_rtokens = 0;
+int lam_ssi_rpi_gm_enqueue_sends = 0;
 int lam_ssi_rpi_gm_max_port_num = LAM_SSI_RPI_GM_MAX_PORT_NUM;
 int lam_ssi_rpi_gm_port_num = LAM_SSI_RPI_GM_PORT;
 int lam_ssi_rpi_gm_dma_env_len;
-int lam_ssi_rpi_gm_dma_short_len;
 int lam_ssi_rpi_gm_tinymsglen = 0;
-int lam_ssi_rpi_gm_shortmsglen = 0;
-int lam_ssi_rpi_gm_can_register_mem = LAM_SSI_RPI_GM_CAN_REGISTER_MEM;
+int lam_ssi_rpi_gm_can_register_mem = 0;
 unsigned int lam_ssi_rpi_gm_env_min_size_for_length = 0;
 unsigned int lam_ssi_rpi_gm_data_min_size_for_length = 0;
 lam_array_t *PUB(procs) = NULL;
 int PUB(req_mid) = -1;
-
-
-/* mapping between (gmID * GM_MAX_PORT_NUM + portnum) and (*_proc) */
-
-int lam_ssi_rpi_gm_map_size = 0;
-struct _proc **lam_ssi_rpi_gm_map = NULL;
+int PUB(param_priority) = -1;
+int PUB(param_cr) = -1;
+int PUB(param_fast) = -1;
+int PUB(cr_available) = 0;
+int PUB(using_fast) = 0;
 
 
 /*
  * private varaibles
  */
+#define WANT_DEBUG_STDOUT 0
+#define WANT_DEBUG_STDERR 0
 static lam_debug_stream_info_t lds = {
-  1, 0, 0, NULL, NULL, 0, 0, 1, 0, NULL
+  1, 0, 0, NULL, NULL, WANT_DEBUG_STDOUT, WANT_DEBUG_STDERR, 1, 0, NULL
 };
-static const lam_ssi_rpi_actions_t module_actions = {
+static lam_ssi_rpi_actions_1_1_0_t module_actions = {
   /* RPI API functions */
 
   lam_ssi_rpi_gm_addprocs,
@@ -116,9 +117,17 @@ static const lam_ssi_rpi_actions_t module_actions = {
   lam_ssi_rpi_gm_alloc_mem,
   lam_ssi_rpi_gm_free_mem,
 
+#if LAM_SSI_RPI_GM_WANT_CHECKPOINT
+  lam_ssi_rpi_gm_interrupt,
+  lam_ssi_rpi_gm_checkpoint,
+  lam_ssi_rpi_gm_continue,
+  lam_ssi_rpi_gm_restart,
+#else
   NULL,
   NULL,
   NULL,
+  NULL,
+#endif
 
   /* Flags */
   /* Totalview support */
@@ -147,8 +156,8 @@ lam_ssi_rpi_gm_query(int *priority, int *thread_min, int *thread_max)
 
   /* Check this module's priority before doing anything else */
 
-  if (lam_ssi_rpi_base_check_priority("gm", 50, LAM_SSI_RPI_GM_DEFAULT,
-				      priority) < 0)
+  *priority = lam_ssi_base_param_lookup_int(PUB(param_priority));
+  if (*priority < 0)
     return 0;
 
   /* See if all the procs are the same endian.  This RPI currently can
@@ -169,15 +178,19 @@ lam_ssi_rpi_gm_query(int *priority, int *thread_min, int *thread_max)
      using the gm RPI, we'll free it shortly (see the module_close
      function, below). */
 
+  myport.rgp_gm_port = NULL;
+  myport.rgp_global_node_id = myport.rgp_local_node_id = 0;
+  myport.rgp_port_id = -1;
+
   if (gm_init() != GM_SUCCESS)
     return 0;
 
-  myport.rgp_gm_port = NULL;
-  myport.rgp_global_node_id = 0;
-  myport.rgp_local_node_id = 0;
-  myport.rgp_port_id = -1;
   if (lam_ssi_rpi_gm_get_port(lam_myproc, &myport) == LAMERROR) {
     gm_finalize();
+    /* reset myport to its original values */
+    myport.rgp_gm_port = NULL;
+    myport.rgp_global_node_id = myport.rgp_local_node_id = 0;
+    myport.rgp_port_id = -1;
     return 0;
   }
 
@@ -186,6 +199,14 @@ lam_ssi_rpi_gm_query(int *priority, int *thread_min, int *thread_max)
   *thread_min = MPI_THREAD_SINGLE;
   *thread_max = MPI_THREAD_SERIALIZED;
 
+  /* If we want CR support, then set the minimum thread level to be
+     MPI_THREAD_SERIALIZED */
+
+  PUB(cr_available) = lam_ssi_base_param_lookup_int(PUB(param_cr));
+  if (PUB(cr_available) == 1)
+    *thread_min = MPI_THREAD_SERIALIZED;
+  PUB(using_fast) = lam_ssi_base_param_lookup_int(PUB(param_fast));
+
   /* If we're verbose, print stuff out */
 
   if (lam_ssi_rpi_verbose >= 5) {
@@ -193,12 +214,13 @@ lam_ssi_rpi_gm_query(int *priority, int *thread_min, int *thread_max)
     lam_debug(lam_ssi_rpi_did, "gm:verbose: %d", 
 	      lam_ssi_rpi_verbose);
     lam_debug(lam_ssi_rpi_did, "gm:priority: %d", *priority);
+    lam_debug(lam_ssi_rpi_did, "gm:fast: %d", PUB(using_fast));
   }
 
   /* Setup a request memory pool */
 
   PUB(req_mid) = lam_mp_init(sizeof(struct lam_ssi_rpi_req),
-                             NULL, NULL, 10, 10);
+                             NULL, NULL, 10, 10, 1);
 
   /* Return 1 saying that we want to be considered */
 
@@ -238,10 +260,8 @@ lam_ssi_rpi_gm_init(struct _proc **procs, int nprocs,
      query(), above. */
 
   proc_init(lam_myproc);
-  lam_myproc->p_rpi->cp_gm_port = myport.rgp_gm_port;
-  lam_myproc->p_rpi->cp_global_node_id = myport.rgp_global_node_id;
-  lam_myproc->p_rpi->cp_local_node_id = myport.rgp_local_node_id;
-  lam_myproc->p_rpi->cp_port_id = myport.rgp_port_id;
+  lam_myproc->p_rpi->cp_gm_port = myport;
+  myport.rgp_gm_port = NULL;
 
   /* Initalize GM stuff.  If we fail, we still have to call
      connect_all() so that we can let the other nodes know of our
@@ -268,10 +288,26 @@ lam_ssi_rpi_gm_init(struct _proc **procs, int nprocs,
   if (lam_ssi_rpi_verbose >= 5) {
     lam_debug(lam_ssi_rpi_did, "gm:max_tiny_size: %d", 
               lam_ssi_rpi_gm_tinymsglen);
-    lam_debug(lam_ssi_rpi_did, "gm:max_short_size: %d", 
-              lam_ssi_rpi_gm_shortmsglen);
     lam_debug(lam_ssi_rpi_did, "gm:memory_pin: %d", 
               lam_ssi_rpi_gm_can_register_mem);
+    lam_debug(lam_ssi_rpi_did, "gm:cr_available: %d", 
+              PUB(cr_available));
+    lam_debug(lam_ssi_rpi_did, "gm:fast: %d", 
+	      PUB(using_fast));
+  }
+
+  /* If we have CR available, disable fast */
+
+  if (PUB(cr_available) == 1) {
+    module_actions.lsra_fastrecv = NULL;
+    module_actions.lsra_fastsend = NULL;
+  } else if (PUB(using_fast) == 1) {
+    module_actions.lsra_fastsend = lam_ssi_rpi_gm_fastsend;
+  } else {
+    module_actions.lsra_interrupt = NULL;
+    module_actions.lsra_checkpoint = NULL;
+    module_actions.lsra_continue = NULL;
+    module_actions.lsra_restart = NULL;
   }
 
   return &module_actions;
@@ -327,7 +363,7 @@ int
 lam_ssi_rpi_gm_build(MPI_Request req)
 {
   req->rq_rpi = lam_mp_malloc(PUB(req_mid));
-  if (req->rq_rpi == 0)
+  if (req->rq_rpi == NULL)
     return LAMERROR;
 
   /* Because of @#%@#$% MPI_Bsend (more specifically, MPI_Ibsend), we
@@ -335,7 +371,6 @@ lam_ssi_rpi_gm_build(MPI_Request req)
      we can call destroy cleanly if so. */
 
   req->rq_rpi->cq_envbuf = NULL;
-  req->rq_rpi->cq_envbuf2 = NULL;
 
   return 0;
 }
@@ -355,6 +390,7 @@ int
 lam_ssi_rpi_gm_start(MPI_Request req_top, MPI_Request req)
 {
   struct lam_ssi_rpi_cbuf_msg *bmsg;
+  struct lam_ssi_rpi_req *rpi = req->rq_rpi;
 
   lam_debug_cond((lam_ssi_rpi_gm_did, 
                   "rpilam_ssi_rpi_start: starting req %p\n", req));
@@ -367,34 +403,50 @@ lam_ssi_rpi_gm_start(MPI_Request req_top, MPI_Request req)
      in a receive request may be a wildcard but will be set to the
      actual peer upon matching with an incoming mesage.  */
 
-  req->rq_rpi->cq_peer = req->rq_rank;
-  req->rq_rpi->is_advancing = 0;
-  req->rq_rpi->send_advance_fn = NULL;
-  req->rq_rpi->recv_advance_fn = NULL;
-  req->rq_rpi->dma_data_buf = NULL;
+  rpi->cq_peer = req->rq_rank;
+  rpi->is_advancing = 0;
+  rpi->send_advance_fn = NULL;
+  rpi->recv_advance_fn = NULL;
+  rpi->dma_data_buf = NULL;
 
-  req->rq_rpi->cq_envbuf2 = NULL;
-  req->rq_rpi->cq_envbuf = lam_ssi_rpi_gm_dma_env_malloc();
-  if (req->rq_rpi->cq_envbuf == NULL)
+  rpi->cq_envbuf = lam_ssi_rpi_gm_dma_env_malloc();
+  if (rpi->cq_envbuf == NULL)
     return LAMERROR;
-  req->rq_rpi->cq_envbuf->ge_env.ce_flags = 0;
-  req->rq_rpi->cq_envbuf->ge_env.ce_tag = req->rq_tag;
-  req->rq_rpi->cq_envbuf->ge_env.ce_cid = req->rq_cid;
-  req->rq_rpi->cq_envbuf->ge_env.ce_seq = -1;
-  req->rq_rpi->cq_envbuf->ge_target = NULL;
+  rpi->cq_envbuf->ge_flags = 0;
+  rpi->cq_envbuf->ge_tag = req->rq_tag;
+  rpi->cq_envbuf->ge_cid = req->rq_cid;
+  rpi->cq_envbuf->ge_target = NULL;
+  rpi->cq_envbuf->ge_src_request = req;
+#if !LAM_SSI_RPI_GM_HAVE_RDMA_GET
+  /* This field is only used if we don't have RDMA get, and therefore
+     use an alternate long message protocol. */
+  rpi->cq_envbuf->ge_dest_request = NULL;
+#endif
 
-  /* receive specific initialization */
+  /* <RECEIVER> specific initialization */
 
   if ((req->rq_type == LAM_RQIRECV) || (req->rq_type == LAM_RQIPROBE)) {
-    req->rq_rpi->cq_state = C2CREAD;
-    req->rq_rpi->cq_envbuf->ge_env.ce_rank = req->rq_rpi->cq_peer;
+    rpi->cq_state = C2CREAD;
+    rpi->cq_envbuf->ge_rank = rpi->cq_peer;
+
+    /* If the message is long, we need to pin or setup a DMA buffer
+       for it.  Tiny and short messages don't need pinning/DMA buffers
+       (tiny's come in the envelope, shorts use commodity/pooled
+       pinned buffers). */
+
+    if (req->rq_packsize > lam_ssi_rpi_gm_tinymsglen) {
+      if (lam_ssi_rpi_gm_setup_request_dma(req) != 0)
+        return LAMERROR;
+    }
 
     /* Check for matching buffered envelope/message. If one is found
        fill in any receive request wildcards and advance the request.  */
 
     if ((bmsg = 
-         lam_ssi_rpi_cbuf_find(&(req->rq_rpi->cq_envbuf->ge_env))) != NULL) {
-      lam_ssi_rpi_gm_fill_wildcards(req, &bmsg->cm_env);
+         lam_ssi_rpi_cbuf_find((struct lam_ssi_rpi_envl *) 
+                               rpi->cq_envbuf)) != NULL) {
+      lam_ssi_rpi_gm_fill_wildcards(req, 
+                                    (lam_ssi_rpi_gm_envl_t *) &bmsg->cm_env);
       return lam_ssi_rpi_gm_unexpected_advance(req, bmsg);
     }
 
@@ -404,27 +456,13 @@ lam_ssi_rpi_gm_start(MPI_Request req_top, MPI_Request req)
     lam_debug_cond((lam_ssi_rpi_gm_did, 
                     "rpilam_ssi_rpi_start: didn't find a buffered "
                     "message to receive\n"));
-
-    /* If the message is long, we need to pin or setup a DMA buffer
-       for it.  Tiny and short messages don't need pinning/DMA buffers
-       (tiny's come in the envelope, shorts use commodity/pooled
-       pinned buffers). */
-
-    if (req->rq_packsize > lam_ssi_rpi_gm_shortmsglen) {
-      if (lam_ssi_rpi_gm_setup_request_dma(req) != 0)
-        return LAMERROR;
-
-      req->rq_rpi->cq_envbuf2 = lam_ssi_rpi_gm_dma_env_malloc();
-      if (req->rq_rpi->cq_envbuf2 == NULL)
-        return LAMERROR;
-    }
   }
 
-  /* send specific initialization */
+  /* <SENDER> specific initialization */
 
   else {
-    req->rq_rpi->cq_envbuf->ge_env.ce_rank = req->rq_comm->c_group->g_myrank;
-    req->rq_rpi->cq_envbuf->ge_env.ce_len = req->rq_packsize;
+    rpi->cq_envbuf->ge_rank = req->rq_comm->c_group->g_myrank;
+    rpi->cq_envbuf->ge_len = req->rq_packsize;
 
     if (req->rq_proc == lam_myproc) {
 
@@ -435,45 +473,28 @@ lam_ssi_rpi_gm_start(MPI_Request req_top, MPI_Request req)
 
       /* send to another process */
 
-      req->rq_rpi->cq_state = C2CWRITE;
+      rpi->cq_state = C2CWRITE;
 
       /* long message protocol */
       /* need to pin or setup DMA buffer */
 
-      if (req->rq_packsize > lam_ssi_rpi_gm_shortmsglen) {
-	req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2CLONG;
-	req->rq_rpi->send_advance_fn = lam_ssi_rpi_gm_long_send;
+      if (req->rq_packsize > lam_ssi_rpi_gm_tinymsglen) {
+	rpi->cq_envbuf->ge_flags |= C2CLONG;
+	rpi->send_advance_fn = lam_ssi_rpi_gm_long_send;
 
-        req->rq_rpi->cq_envbuf2 = lam_ssi_rpi_gm_dma_env_malloc();
-        if (req->rq_rpi->cq_envbuf2 == NULL)
-          return LAMERROR;
         if (lam_ssi_rpi_gm_setup_request_dma(req) != 0)
           return LAMERROR;
       } 
 
-      /* short message protocol */
-      /* need to pin or setup DMA buffer */
-
-      else if (req->rq_packsize > lam_ssi_rpi_gm_tinymsglen) {
-	if (req->rq_type == LAM_RQISSEND) {
-	  req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2CSSEND;
-	  req->rq_rpi->send_advance_fn = lam_ssi_rpi_gm_short_send_sync;
-	} else
-	  req->rq_rpi->send_advance_fn = lam_ssi_rpi_gm_short_send;
-
-        if (lam_ssi_rpi_gm_setup_request_dma(req) != 0)
-          return LAMERROR;
-      }
-      
       /* tiny message protocol */
       /* don't use pinning or DMA buffers */
 
       else {
 	if (req->rq_type == LAM_RQISSEND) {
-	  req->rq_rpi->cq_envbuf->ge_env.ce_flags |= C2CSSEND;
-	  req->rq_rpi->send_advance_fn = lam_ssi_rpi_gm_tiny_send_sync;
+	  rpi->cq_envbuf->ge_flags |= C2CSSEND;
+	  rpi->send_advance_fn = lam_ssi_rpi_gm_tiny_send_sync;
 	} else
-	  req->rq_rpi->send_advance_fn = lam_ssi_rpi_gm_tiny_send;
+	  rpi->send_advance_fn = lam_ssi_rpi_gm_tiny_send;
       }
     }
   }
@@ -523,18 +544,33 @@ lam_ssi_rpi_gm_destroy(MPI_Request req)
   /* (I don't think it can ever be C2CBUFFERED... but just in case!) */
 
   if (req->rq_rpi->cq_envbuf != NULL &&
-      (req->rq_rpi->cq_envbuf->ge_env.ce_flags & C2CBUFFERED) == 0) 
+      (req->rq_rpi->cq_envbuf->ge_flags & C2CBUFFERED) == 0)
     lam_ssi_rpi_gm_dma_env_free(req->rq_rpi->cq_envbuf);
-  if (req->rq_rpi->cq_envbuf2 != NULL)
-    lam_ssi_rpi_gm_dma_env_free(req->rq_rpi->cq_envbuf2);
-  
+  req->rq_rpi->cq_envbuf = NULL;
+
+  /* There are several cases -- all having to do with unexpected
+     receives -- where we may have posted a long receive and DMA
+     malloced a pinned buffer (this can only happen on systems where
+     we cannot register arbitrary memory, such as Solaris), but then
+     received a short message (or anything other than a long message).
+     In this case, the DMA-malloced buffer won't have been freed in
+     the normal chain of events, so we free it here.  It's easier to
+     put this special case code in one place (i.e., here) rather than
+     the 3-4 cases that this can happen in the unexepcted receive. */
+
+  if (req->rq_rpi->dma_data_buf != NULL) {
+    lam_ssi_rpi_gm_dma_free(req->rq_rpi->dma_data_buf,
+                            req->rq_packsize);
+    req->rq_rpi->dma_data_buf = NULL;
+  }
+
   /* Free the extra state that this RPI caches on the request */
 
   lam_debug_cond((lam_ssi_rpi_gm_did, "Rank %d: destroying req %p\n", 
                   lam_myproc->p_gps.gps_grank, req));
-  if (req->rq_rpi != 0)
+  if (req->rq_rpi != NULL)
     lam_mp_free(PUB(req_mid), req->rq_rpi);
-  req->rq_rpi = 0;
+  req->rq_rpi = NULL;
 
   return 0;
 }
@@ -596,6 +632,7 @@ lam_ssi_rpi_gm_advance(MPI_Request req_top, int fl_block)
       lam_debug_cond((lam_ssi_rpi_gm_did, 
                       "advance: calling write advance on req %p - %s", 
                       req, MPIL_Request_get_name(req)));
+      /* XXX: SS: no error-checking done here */
       req->rq_rpi->send_advance_fn(req);
     } 
 
@@ -620,23 +657,16 @@ lam_ssi_rpi_gm_advance(MPI_Request req_top, int fl_block)
 
   /* Check for and process any pending gm events */
 
-  do {
-    lam_debug_cond((lam_ssi_rpi_gm_did, 
-                    "advance: calling lam_ssi_rpi_gm_advance"));
-    if (lam_ssi_rpi_gm_gm_advance() != 0) {
-      lam_debug_cond((lam_ssi_rpi_gm_did, 
-                      "advance: returns LAMERROR after lam_ssi_rpi_gm_advance"));
-      return(LAMERROR);
-    }
-    lam_debug_cond((lam_ssi_rpi_gm_did, 
-                    "advance: done with lam_ssi_rpi_gm_advance"));
-  } while (lam_ssi_rpi_gm_flblock && !lam_ssi_rpi_gm_haveadv);
+  if (PUB(flblock) == 1)
+    lam_ssi_rpi_gm_gm_advance_blocking();
+  else
+    lam_ssi_rpi_gm_gm_advance_nonblocking();
 
   lam_debug_cond((lam_ssi_rpi_gm_did, 
                   "advance: completed: lam_ssi_rpi_gm_haveadv %d", 
                   lam_ssi_rpi_gm_haveadv));
 		
-  return(lam_ssi_rpi_gm_haveadv);
+  return PUB(haveadv);
 }
 
 
@@ -678,6 +708,22 @@ lam_ssi_rpi_gm_iprobe(MPI_Request req)
 }
 
 
+/*
+ *	lam_ssi_rpi_gm_module_open
+ *
+ *	Function:	- open the gm RPI module
+ *	Accepts:	- OPT pointer
+ *	Returns:	- 0 or LAMERROR
+ */
+int
+lam_ssi_rpi_gm_module_open(OPT *ad)
+{
+  lam_ssi_rpi_gm_params_register();
+
+  return 1;
+}
+
+
 /*
  *	lam_ssi_rpi_gm_module_close
  *
@@ -698,8 +744,10 @@ lam_ssi_rpi_gm_module_close(void)
      MPI_Init to close down this module for the rest of the process
      (and free the gm port that we have allocated). */
 
-  if (myport.rgp_gm_port != NULL)
+  if (myport.rgp_gm_port != NULL) {
     lam_ssi_rpi_gm_gm_finalize(&myport);
+    myport.rgp_gm_port = NULL;
+  }
 
   return 0;
 }
@@ -736,9 +784,13 @@ lam_ssi_rpi_gm_finalize(struct _proc *p)
     lam_ssi_rpi_gm_dma_destroy();
     lam_ssi_rpi_gm_interval_destroy();
 
-    /* Close down gm */
+    /* Close down gm.  We pass &lam_myproc->p_rpi->cp_gm_port here
+       instead of &myport because it is possible that the application
+       was checkpointed at some point; in that case, the port
+       information would have changed and the current values will only
+       be in lam_myproc->p_rpi->cp_gm_port (and not in myport). */
 
-    lam_ssi_rpi_gm_gm_finalize(&myport);
+    lam_ssi_rpi_gm_gm_finalize(&lam_myproc->p_rpi->cp_gm_port);
 
     /* Free our aux list of MPI_ALLOC_MEM'ed memory */
 
@@ -759,7 +811,6 @@ lam_ssi_rpi_gm_finalize(struct _proc *p)
 
   }
 
-  free(lam_ssi_rpi_gm_map);
   return 0;
 }
 
@@ -784,8 +835,8 @@ send_to_self(MPI_Request req_top, MPI_Request send)
   for (recv = req_top; recv; recv = recv->rq_next)
     if ((recv->rq_state == LAM_RQSSTART) &&
 	(recv->rq_rpi->cq_state == C2CREAD) &&
-	(!lam_ssi_rpi_envl_cmp(&(send->rq_rpi->cq_envbuf->ge_env), 
-                               &(recv->rq_rpi->cq_envbuf->ge_env)))) {
+	(ENVL_COMPARE(send->rq_rpi->cq_envbuf, 
+                      recv->rq_rpi->cq_envbuf)) == 1) {
       lam_debug_cond((lam_ssi_rpi_gm_did, 
                       "send_to_self: found matching recv posted, "
                       "send req %p, recv req %p", send, recv));
@@ -802,7 +853,19 @@ send_to_self(MPI_Request req_top, MPI_Request send)
      user buffer and the send only completes once a matching receive
      is posted.  */
 
-  msg.cm_env = send->rq_rpi->cq_envbuf->ge_env;
+  msg.cm_extra = malloc(sizeof(struct lam_ssi_rpi_cbuf));
+  if (msg.cm_extra == NULL)
+    return LAMERROR;
+
+  msg.cm_extra->lsrc_src_request = send;
+  msg.cm_env.ce_len = send->rq_rpi->cq_envbuf->ge_len;
+  msg.cm_env.ce_tag = send->rq_rpi->cq_envbuf->ge_tag;
+  msg.cm_env.ce_flags = send->rq_rpi->cq_envbuf->ge_flags;
+  msg.cm_env.ce_rank = send->rq_rpi->cq_envbuf->ge_rank;
+  msg.cm_env.ce_cid = send->rq_rpi->cq_envbuf->ge_cid;
+#if LAM_DISINFECT
+  msg.cm_env.ce_seq = -1;
+#endif
   msg.cm_proc = 0;
   msg.cm_dont_delete = 1;
  
@@ -845,9 +908,9 @@ send_to_self_match(MPI_Request send, MPI_Request recv)
 
     /* The receive is actually a probe so the send is not complete.  */
 
-    lam_ssi_rpi_fill_mpi_status(recv, send->rq_rpi->cq_envbuf->ge_env.ce_rank,
-				send->rq_rpi->cq_envbuf->ge_env.ce_tag, 
-				send->rq_rpi->cq_envbuf->ge_env.ce_len);
+    lam_ssi_rpi_fill_mpi_status(recv, send->rq_rpi->cq_envbuf->ge_rank,
+				send->rq_rpi->cq_envbuf->ge_tag, 
+				send->rq_rpi->cq_envbuf->ge_len);
     
     recv->rq_state = LAM_RQSDONE;
     lam_rq_nactv--;
@@ -865,8 +928,8 @@ send_to_self_match(MPI_Request send, MPI_Request recv)
 
     lam_memcpy(recv->rq_packbuf, send->rq_packbuf, len);
 
-    lam_ssi_rpi_fill_mpi_status(recv, send->rq_rpi->cq_envbuf->ge_env.ce_rank,
-				send->rq_rpi->cq_envbuf->ge_env.ce_tag, len);
+    lam_ssi_rpi_fill_mpi_status(recv, send->rq_rpi->cq_envbuf->ge_rank,
+				send->rq_rpi->cq_envbuf->ge_tag, len);
 
     /* Need to remove these pending receives from the receive queues.
        This is somewhat inefficient, because we're scanning down the
@@ -905,10 +968,11 @@ connect_all(int happy)
   struct nmsg outmsg;
   struct dolink *links = 0;
   int4 nlinks;
-  int4 rport, rnode;
-  int4 maxnode;
+  int4 rport;
   int node;
+  unsigned int rnode;
   int i;
+  lam_ssi_rpi_gm_connect_t *coutdata, *cindata;
 
   LAM_ZERO_ME(inmsg);
   LAM_ZERO_ME(outmsg);
@@ -942,36 +1006,42 @@ connect_all(int happy)
     inmsg.nh_length = 0;
     outmsg.nh_length = 0;
     outmsg.nh_flags = 0;
+    coutdata = (lam_ssi_rpi_gm_connect_t *) &(outmsg.nh_data[0]);
     if (happy) {
-      outmsg.nh_data[0] = (int4) lam_myproc->p_rpi->cp_port_id;
-      outmsg.nh_data[1] = (int4) lam_myproc->p_rpi->cp_global_node_id;
+      coutdata->rpgc_port_id = lam_myproc->p_rpi->cp_gm_port.rgp_port_id;
+      coutdata->rpgc_global_node_id = 
+        lam_myproc->p_rpi->cp_gm_port.rgp_global_node_id;
+      lam_debug_cond((PUB(did), 
+                      "connect_all: sending my (global) node %d, my port %d",
+                      coutdata->rpgc_global_node_id, coutdata->rpgc_port_id));
     } else {
-      outmsg.nh_data[0] = (int4) -1;
-      outmsg.nh_data[1] = (int4) -1;
+      coutdata->rpgc_port_id = -1;
+      coutdata->rpgc_global_node_id = -1;
+      coutdata->rpgc_peer_proc = NULL;
     }
   }
 
   /* Loop through all processes, initializing the process data and
      send connection data to everyone.  */
 
-  maxnode = lam_myproc->p_rpi->cp_local_node_id;
+  cindata = (lam_ssi_rpi_gm_connect_t *) &(inmsg.nh_data[0]);
   for (i = 0; i < lam_arr_size(PUB(procs)); ++i) {
     p = array[i];
       
-    /* Check and see if this proc is already connected */
-
-    if (p->p_mode & LAM_PRPIINIT)
+    /* Check and see if this proc is already initialized and connected */
+    if ((p->p_mode & LAM_PRPIINIT) && (p->p_mode & LAM_PRPICONNECT))
       continue;
 
-    /* initialize the process */
-
-    proc_init(p);
-
+    /* initialize process */
+    if (!(p->p_mode & LAM_PRPIINIT))
+      proc_init(p);
+    
     if (p != lam_myproc) {
 
       /* Send the Data */
       
       fill_sync(lam_myproc, p, &outmsg);
+      coutdata->rpgc_peer_proc = p;
       if (nsend(&outmsg)) 
 	return(LAMERROR);
       
@@ -981,43 +1051,50 @@ connect_all(int happy)
       if (nrecv(&inmsg)) 
 	return(LAMERROR);
       
-      rport = (int) inmsg.nh_data[0];
-      rnode = (int) inmsg.nh_data[1];
+      rport = cindata->rpgc_port_id;
+      rnode = cindata->rpgc_global_node_id;
+      lam_debug_cond((PUB(did), 
+                      "connect_all: received (global) node %d, port %d",
+                      rnode, rport));
        
       /* store data in RPI private area */
 
       node = p->p_gps.gps_node;
-      p->p_rpi->cp_port_id = rport;
-      p->p_rpi->cp_global_node_id = rnode;
+      p->p_rpi->cp_gm_port.rgp_port_id = rport;
+      p->p_rpi->cp_gm_port.rgp_global_node_id = rnode;
+      p->p_rpi->cp_peer_proc = cindata->rpgc_peer_proc;
 
 #if LAM_SSI_RPI_GM_2
       /* If we're using gm 2.x, we need to convert the remote global
          node ID into a local node ID */
 
-      if (gm_global_id_to_node_id(myport.rgp_gm_port,
-                                  p->p_rpi->cp_global_node_id,
-                                  &(p->p_rpi->cp_local_node_id)) !=
-          GM_SUCCESS) {
+      if (gm_global_id_to_node_id(lam_myproc->p_rpi->cp_gm_port.rgp_gm_port, 
+                                  p->p_rpi->cp_gm_port.rgp_global_node_id,
+                                  &(p->p_rpi->cp_gm_port.rgp_local_node_id)) !=
+                                  GM_SUCCESS) {
         lam_debug_cond((lam_ssi_rpi_gm_did, "connect_all: "
                         "Unable to convert local ID to global ID"));
         happy = FALSE;
       }
 #else
-      p->p_rpi->cp_local_node_id = p->p_rpi->cp_global_node_id;
+      p->p_rpi->cp_gm_port.rgp_local_node_id = 
+        p->p_rpi->cp_gm_port.rgp_global_node_id;
 #endif
-      maxnode = LAM_max(maxnode, p->p_rpi->cp_local_node_id);
+
       if (rport == -1 && rnode == -1)
 	happy = FALSE;
       
       lam_debug_cond((lam_ssi_rpi_gm_did, 
                       "connect_all: recv (port %d, id %d)from rank %d on n%d",
-                      rport, rnode,
+                      rport, 
+                      p->p_rpi->cp_gm_port.rgp_local_node_id,
                       p->p_gps.gps_grank,
                       node));
       
       if (node > nlinks) 
 	return(LAMERROR);
     }
+    p->p_mode |= LAM_PRPICONNECT;
   }
 
   /* Did we (or someone else) fail?  If so, clean up any memory, and
@@ -1029,43 +1106,6 @@ connect_all(int happy)
     return LAMERROR;
   }
 
-  /* Now that we know the max id, we can alloc enough space for the
-     hash map, and then store the previous data in the hash map. */
-  /* This can actually be wasteful, because there's nothing saying
-     that the id's are contiguous from zero (in fact, they probably
-     aren't).  So lam_ssi_rpi_gm_map is probably pretty sparse.  But we index
-     this array by id number (for now), so we'll leave it this way. */
-
-  if (maxnode != -1 && ((maxnode + 1) > lam_ssi_rpi_gm_map_size)) {
-    if (lam_ssi_rpi_gm_map_size == 0)
-      lam_ssi_rpi_gm_map = 
-        (struct _proc **) malloc(sizeof(struct _proc *) * 
-                                 (maxnode + 1) * 
-                                 lam_ssi_rpi_gm_max_port_num);
-    else
-      lam_ssi_rpi_gm_map = 
-        (struct _proc **) realloc(lam_ssi_rpi_gm_map, 
-                                  sizeof(struct _proc *) * 
-                                  (maxnode + 1) * 
-                                  lam_ssi_rpi_gm_max_port_num);
-    memset(lam_ssi_rpi_gm_map, 0, 
-	   sizeof(struct _proc**) * (maxnode + 1) * 
-	   lam_ssi_rpi_gm_max_port_num);
-    lam_ssi_rpi_gm_map_size = maxnode;
-  }
-
-  /* Fill the map.  This is somewhat redundant if we're coming through
-     here again after MPI_INIT (e.g., spawn), but it's easier this
-     way. */
-
-  if (lam_ssi_rpi_gm_map_size > 0) {
-    for (i = 0; i < lam_arr_size(PUB(procs)); ++i) {
-      p = array[i];
-      PUB(map)[LAM_SSI_RPI_GM_MAP_HASH(p->p_rpi->cp_local_node_id, 
-                                       p->p_rpi->cp_port_id)] = p;
-    }
-  }
-
   /* Cleanup */
 
   if (lam_arr_size(PUB(procs)) > 1)
@@ -1095,13 +1135,26 @@ proc_init(struct _proc *p)
     return;
 
   p->p_rpi->cp_proc = p;
-  p->p_rpi->cp_bmsg = NULL;
-  p->p_rpi->cp_gm_port = NULL;
+  p->p_rpi->cp_gm_port.rgp_gm_port = NULL;
+  p->p_rpi->cp_in_nbytes = p->p_rpi->cp_out_nbytes = 0;
+  p->p_rpi->num_pending_sends = 0;
+  p->p_rpi->num_dropped_sends = 0;
 
   lam_ssi_rpi_gm_reqlist_init(&(p->p_rpi->cp_pending_recvs));
-  lam_ssi_rpi_gm_reqlist_init(&(p->p_rpi->cp_pending_acks));
 
-  p->p_rpi->cp_current_recv_req = NULL;
+  /* Keep track of incoming long messages that are in a critical state
+     such that if a checkpoint request arrives it would cause a stale
+     pointer upon continue/restart. */
+
+  if (PUB(cr_available) == 1) {
+    p->p_rpi->cp_unexpected_long_recvs = 
+      lam_arr_init(sizeof(struct lam_rpi_ssi_envl *), NULL);
+    p->p_rpi->cp_pending_long_recvs = 
+      lam_arr_init(sizeof(MPI_Request), NULL);
+  } else {
+    p->p_rpi->cp_unexpected_long_recvs = NULL;
+    p->p_rpi->cp_pending_long_recvs = NULL;
+  }
 }
 
 
@@ -1142,6 +1195,20 @@ finalize1(struct _proc *p)
   /* Double check to ensure this proc still needs to be cleaned up */
 
   if (p->p_rpi != NULL) {
+
+    /* Free the checkpoint/restat lists of pending long messages */
+
+    if (PUB(cr_available) == 1) {
+      lam_arr_free(p->p_rpi->cp_unexpected_long_recvs);
+      lam_arr_free(p->p_rpi->cp_pending_long_recvs);
+    }
+
+    /* For debugging */
+
+    LAM_ZERO_ME(*(p->p_rpi));
+
+    /* Free it */
+
     free(p->p_rpi);
     p->p_rpi = NULL;
   }
@@ -1150,214 +1217,6 @@ finalize1(struct _proc *p)
 }
 
 
-#if 0
-/* JMS Currently disabled -- doesn't work at all */
-/*
- *	lam_ssi_rpi_gm_fastsend
- *
- *	Function:	- fast blocking send
- *	Accepts:	- buffer to send
- *			- message count
- *			- message datatype
- *			- destination process rank
- *			- message tag
- *			- message communicator
- *	Returns:	- MPI_SUCCESS or error code
- */
-int
-lam_ssi_rpi_gm_fastsend(char *buf, int count, MPI_Datatype dtype, int dest,
-			int tag, MPI_Comm comm)
-{
-  double		local_rep;		/* local data representation */
-  double		net_rep;		/* net data representation */
-  struct _proc	*destproc;
-  char		*packbuf;
-  int			packsize;
-  int			err;
-
-  printf("lam_ssi_rpi_gm_fastsend not implemented in gm RPI yet\n");
-  return (LAMERROR);
-
-  lam_debug_cond((lam_ssi_rpi_gm_did, 
-                  "fastsend: Starting fastsend to rank %d",
-                  dest));
-  
-  /* Check common arguments.  */
-  
-  if (count < 0)
-    return(lam_mkerr(MPI_ERR_COUNT, 0));
-  
-  if (dtype == MPI_DATATYPE_NULL || (!dtype->dt_commit))
-    return(lam_mkerr(MPI_ERR_TYPE, 0));
-  
-  if (LAM_IS_INTER(comm)) {
-    if ((dest < 0) || (dest >= comm->c_rgroup->g_nprocs))
-      return(lam_mkerr(MPI_ERR_RANK, 0));
-    
-    destproc = comm->c_rgroup->g_procs[dest];
-  } else {
-    if ((dest < 0) || (dest >= comm->c_group->g_nprocs))
-      return(lam_mkerr(MPI_ERR_RANK, 0));
-    
-    destproc = comm->c_group->g_procs[dest];
-  }
-  
-  /* Handle zero length messages.  */
-  
-  if (count == 0 || dtype->dt_size == 0) {
-    packbuf = buf;
-    packsize = 0;
-  } else {
-    local_rep = 1.1;
-    ltotf8(&local_rep, &net_rep);
-    
-    /* If contiguous, use the caller's buffer.  */
-    
-    packsize = count * dtype->dt_size;
-    
-    if ((dtype->dt_flags & LAM_DTNOPACK) &&
-	((dtype->dt_flags & LAM_DTNOXADJ) || count == 1) &&
-	((local_rep == net_rep) || (destproc->p_mode & LAM_PHOMOG))) {
-      packbuf = buf;
-      
-      /* Check for bad buffer.  */
-      
-      if (packbuf == 0)
-	return(lam_mkerr(MPI_ERR_BUFFER, 0));
-    }
-    
-    /* Otherwise allocate a buffer. */
-    
-    else {
-      packbuf = malloc(packsize);
-      if (packbuf == 0)
-	return(lam_mkerr(MPI_ERR_OTHER, errno));
-      
-      /* Pack the Buffer */ 
-      
-      if (lam_pack(buf, count, dtype, packbuf, packsize) < 0)
-	return(lam_mkerr(MPI_ERR_INTERN, errno));
-    }
-  }
-  
-  err = lam_ssi_rpi_gm_fastsend_low(packbuf, packsize,
-				    destproc->p_rpi, dest, tag, comm);
-  
-  if (packbuf != buf)
-    free(packbuf);
-  
-  return(err);
-}
-#endif
-
-
-#if 0
-/* JMS Currently disabled -- doesn't work at all */
-/*
- *	lam_ssi_rpi_gm_fastrecv
- *
- *	Function:	- fast blocking receive
- *	Accepts:	- buffer to receive into
- *			- message count
- *			- message datatype
- *			- source process rank
- *			- message tag (inout)
- *			- message communicator
- *			- status (out)
- *			- seqnum (out)
- *	Returns:	- MPI_SUCCESS or error code
- */
-int
-lam_ssi_rpi_gm_fastrecv(char *buf, int count, MPI_Datatype dtype, int src,
-			int *tag, MPI_Comm comm, MPI_Status *stat, int *seqnum)
-{
-  double		local_rep;		/* local data representation */
-  double		net_rep;		/* net data representation */
-  struct _proc	*srcproc;
-  char		*packbuf;
-  int			packsize;
-  int			err;
-
-  printf("lam_ssi_rpi_gm_fastrecv not implemented in gm RPI yet\n");
-  return (LAMERROR);
-
-  lam_debug_cond((lam_ssi_rpi_gm_did, "fastrecv: Starting fastrecv"));
-  
-  /* Check common arguments.  */
-
-  if (count < 0)
-    return(lam_mkerr(MPI_ERR_COUNT, 0));
-  
-  if (dtype == MPI_DATATYPE_NULL || (!dtype->dt_commit))
-    return(lam_mkerr(MPI_ERR_TYPE, 0));
-  
-  if (LAM_IS_INTER(comm)) {
-    if ((src < 0) || (src >= comm->c_rgroup->g_nprocs))
-      return(lam_mkerr(MPI_ERR_RANK, 0));
-    
-    srcproc = comm->c_rgroup->g_procs[src];
-  } else {
-    if ((src < 0) || (src >= comm->c_group->g_nprocs))
-      return(lam_mkerr(MPI_ERR_RANK, 0));
-    
-    srcproc = comm->c_group->g_procs[src];
-  }
-
-  /* Handle zero length messages.  */
-
-  if (count == 0 || dtype->dt_size == 0) {
-    packbuf = buf;
-    packsize = 0;
-  } else {
-    local_rep = 1.1;
-    ltotf8(&local_rep, &net_rep);
-
-    /* If contiguous, use the caller's buffer.  */
-
-    packsize = count * dtype->dt_size;
-
-    if ((dtype->dt_flags & LAM_DTNOPACK) &&
-	((dtype->dt_flags & LAM_DTNOXADJ) || count == 1) &&
-	((local_rep == net_rep) || (srcproc->p_mode & LAM_PHOMOG))) {
-      packbuf = buf;
-
-      /* Check for bad buffer.  */
-
-      if (packbuf == 0)
-	return(lam_mkerr(MPI_ERR_BUFFER, 0));
-    }
-
-    /* Otherwise allocate a buffer.  */
-
-    else {
-      packbuf = malloc(packsize);
-      if (packbuf == 0)
-	return(lam_mkerr(MPI_ERR_OTHER, errno));
-    }
-  }
-
-  err = lam_ssi_rpi_gm_fastrecv_low(packbuf, &packsize,
-				    srcproc->p_rpi, src, tag, comm, seqnum);
-
-  if (stat != MPI_STATUS_IGNORE) {
-    stat->MPI_SOURCE = src;
-    stat->MPI_TAG = *tag;
-    stat->MPI_ERROR = err;
-    stat->st_length = packsize;
-  }
-
-  if (packbuf != buf) {
-    if (lam_unpack(packbuf, packsize, buf, count, dtype) < 0)
-      return(lam_mkerr(MPI_ERR_INTERN, errno));
-    
-    free(packbuf);
-  }
-  
-  return(err);
-}
-#endif
-
-
 /*
  *	lam_ssi_rpi_gm_alloc_mem
  *
@@ -1370,7 +1229,6 @@ int
 lam_ssi_rpi_gm_alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
 {
   allocmem_t entry;
-  int need_to_pin;
   entry.am_length = size;
 
   /* Ensure the list is initialized */
@@ -1387,36 +1245,32 @@ lam_ssi_rpi_gm_alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr)
     entry.am_base = *((void **) baseptr);
     entry.am_registered = 1;
 
-    need_to_pin = 1;
+    /* Now register the interval and mark it "LRU not ok" so that it
+       won't get freed by LRU sweeps. */
+
+    if (lam_ssi_rpi_gm_interval_use(*((void **) baseptr), 
+                                    size, 0, 1) == LAMERROR) {
+      free(*((void **) baseptr));
+      *((void **) baseptr) = NULL;
+      return lam_mkerr(MPI_ERR_NO_MEM, errno);
+    }
   }
 
-  /* Otherwise, use the gm function to allocate pinned memory */
+  /* Otherwise, use the gm function to allocate pinned memory.  This
+     will register the interval for us, so it's not necessary to do
+     anything after calling lam_ssi_rpi_gm_dma_malloc(). */
 
   else {
     entry.am_base = *((void **) baseptr) = lam_ssi_rpi_gm_dma_malloc(size);
     entry.am_registered = 0;
-
-    need_to_pin = 0;
   }
 
-  /* Ensure that we got something back */
+  /* Ensure that we got something back. */
+  /* JMS Bad error handling here -- this doesn't clean up
+     properly... */
 
   if (entry.am_base == NULL)
-    return(lam_mkerr(MPI_ERR_NO_MEM, ENOMEM));
-
-  /* Now register the interal and mark it "LRU not ok" so that it
-     won't get freed by LRU sweeps. */
-
-  if (lam_ssi_rpi_gm_interval_use(*((void **) baseptr), 
-                                  size, 0, need_to_pin) == LAMERROR) {
-    if (need_to_pin == 1)
-      free(*((void **) baseptr));
-    else
-      lam_ssi_rpi_gm_dma_free(*((void **) baseptr));
-
-    *((void **) baseptr) = NULL;
-    return(lam_mkerr(MPI_ERR_NO_MEM, errno));
-  }
+    return lam_mkerr(MPI_ERR_NO_MEM, ENOMEM);
 
   /* All done */
 
@@ -1446,20 +1300,26 @@ lam_ssi_rpi_gm_free_mem(void *base)
   entry = al_find(mpi_alloced_mem, &find);
   if (entry != NULL) {
 
-    /* Mark the interval as not being used (i.e., decrease the
-       reference count) */
+    /* Was this memory registered and the interval manually used above? */
 
-    lam_ssi_rpi_gm_interval_unuse(base, entry->am_length);
+    if (entry->am_registered == 1) {
 
-    /* If it was simply registered above, then just free it */
+      /* Mark the interval as not being used (i.e., decrease the
+         reference count) */
+
+      lam_ssi_rpi_gm_interval_unuse(base, entry->am_length);
+
+      /* Now actually free the memory */
 
-    if (entry->am_registered == 1)
       free(base);
+    }
 
-    /* Otherwise, use the gm function to free it */
+    /* Otherwise, use the gm function to free it.  This will
+       automatically unuse the interval as well. */
 
-    else
-      lam_ssi_rpi_gm_dma_free(base);
+    else {
+      lam_ssi_rpi_gm_dma_free(base, entry->am_length);
+    }
 
     al_delete(mpi_alloced_mem, entry);
   }
