@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,11 +11,8 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	NJN
  *
- *	$Id: ssi_rpi_sysv_common.c,v 1.4 2002/12/16 16:24:57 jsquyres Exp $
+ * $Id: ssi_rpi_sysv_common.c,v 1.5 2003/07/11 19:48:18 jsquyres Exp $
  *
  *	Function:	- common shared memory transport low-level routines
  */
@@ -132,6 +129,11 @@ int lam_ssi_rpi_sysv_poolsize = LAM_SHMPOOLSIZE; /* size of long msg pool */
 int lam_ssi_rpi_sysv_maxalloc = LAM_SHMMAXALLOC; /* max allocation of shmem */
 int lam_ssi_rpi_sysv_short = LAM_SHMSHORTMSGLEN; /* short message len */
 
+int lam_ssi_rpi_sysv_param_poll_yield;
+int lam_ssi_rpi_sysv_param_poolsize;
+int lam_ssi_rpi_sysv_param_maxalloc;
+int lam_ssi_rpi_sysv_param_short;
+
 
 /*
  * private variables
