@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2002 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2003 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -11,11 +11,8 @@
  * LAM/MPI source distribution.
  * 
  * $HEADER$
- *	Ohio Trollius
- *	Copyright 1997 The Ohio State University
- *	GDB
  *
- *	$Id: rpdoom.c,v 6.5 2002/10/09 21:00:47 brbarret Exp $
+ * $Id: rpdoom.c,v 6.6 2003/08/21 17:55:20 jsquyres Exp $
  * 
  *	Function:	- delivers a signal to a remote process
  *			- based on Trollius 2.0 Copyright 1990
@@ -45,19 +42,15 @@
 
 
 int
-rpdoom(nodeid, sflags, svalue, signal)
-
-int4			sflags;		/* process select flags */
-int4			svalue;		/* process select value */
-int4			nodeid;		/* target node address */
-int4			signal;		/* signal number */
-
+rpdoom(int4 nodeid, int4 sflags, int4 svalue, int4 signal)
 {
 	struct nmsg	nhead;		/* network nhead */
 	struct preq	request;	/* kenya request */
 	struct preply	reply;		/* kenya reply */
 	int		mask;
 
+        LAM_ZERO_ME(nhead);
+        LAM_ZERO_ME(request);
 	request.pq_sel_method = ltot(sflags);
 	request.pq_sel_value = ltot(svalue);
 	request.pq_src_node =
