@@ -10,12 +10,14 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *
  *	$Log: getbuf.c,v $
- *	Revision 6.2.2.1  2002/10/09 19:49:56  brbarret
- *	* Updated license header to match current requirements
- *	
+ *	Revision 6.3  2002/10/09 21:00:53  brbarret
+ *	* Update tree to match the latest, greatest, license for LAM/MPI.  And this
+ *	  one is even done properly :)
+ *
  *	Revision 6.2  1999/05/27 20:07:04  kmeyer1
  *	added copyright
  *	
