@@ -10,11 +10,12 @@
  * information, see the LICENSE file in the top level directory of the
  * LAM/MPI source distribution.
  * 
+ * $HEADER$
  *	Ohio Trollius
  *	Copyright 1997 The Ohio State University
  *	SBD/JRV
  *
- *	$Id: tprint.c,v 6.7.2.1 2002/10/09 19:49:56 brbarret Exp $
+ *	$Id: tprint.c,v 6.8 2002/10/09 21:00:54 brbarret Exp $
  * 
  *	Function:	- tstdio output formatting rountines
  */
@@ -396,7 +397,7 @@ char			**storepp;
 static int
 getnum(stringpp, flagp)
 
-char			**stringpp;
+CONST char			**stringpp;
 int			*flagp;		/* width and precision star flags */
 
 {
