@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2003 The Trustees of Indiana University.  
+ * Copyright (c) 2001-2004 The Trustees of Indiana University.  
  *                         All rights reserved.
  * Copyright (c) 1998-2001 University of Notre Dame. 
  *                         All rights reserved.
@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- *	$Id: hboot.c,v 6.32.2.1 2004/01/02 00:20:23 jsquyres Exp $
+ * $Id: hboot.c,v 6.35 2004/01/02 00:20:46 jsquyres Exp $
  *
  *	Function:	- boots OTB operating system
  */
@@ -40,7 +40,7 @@
 #include <etc_misc.h>
 
 #define MAXCLOSEFD	32			/* max close-on-exec fdecs. */
-
+#define LAM_MAXPATHLEN     255                      /* max path name len */
 /*
  * external functions
  */
@@ -48,10 +48,7 @@ extern int		_lam_few(char**);
 extern int		psc_parse();
 extern struct psc	*psc_find();
 extern struct psc       *psc_fill();
-/*
- * external variables
- */
-extern char **environ;
+
 
 /*
  * local functions
@@ -89,6 +86,7 @@ main(int argc, char *argv[])
 	char            *prefix = NULL;
 	char            *opt_prefix;
 	char            *tkill_path;
+	char            *path_env;
 	int             prefix_len;
 	int             fl_prefix;
 	int             tkill_len;
@@ -197,6 +195,23 @@ main(int argc, char *argv[])
 	    return LAMERROR;
 	  }
 	  snprintf(prefix, prefix_len, "%s/bin/", opt_prefix);
+
+	  /* Prepend the prefix to the existing env path, so that lamd
+	     can call the correct tkill during the lamhalt, by
+	     looking at the path  */	
+
+	  if ((path_env = getenv("PATH")) == NULL) {
+	      show_help(NULL, "lib-call-fail", "getenv", NULL);
+	      exit(errno);
+	  }
+
+	  snprintf(path_env, strlen(path_env) + LAM_MAXPATHLEN + 1,
+		   "%s:%s", prefix, path_env);
+	
+	  if (putenv(path_env) < 0) {
+	      show_help(NULL, "lib-call-fail", "putenv", NULL);
+	      exit(errno);
+	  }
 	}
 /*
  * Bail out here, if pretending.
@@ -224,25 +239,25 @@ main(int argc, char *argv[])
 		    return LAMERROR;
 	  	  }
 		  snprintf(tkill_path, tkill_len, "%s%s", prefix, t_tkill);
-		  argvadd(&ac_cmd, &av_cmd, tkill_path);
+		  sfh_argv_add(&ac_cmd, &av_cmd, tkill_path);
 		  free(tkill_path);
 		  free(prefix);
 		}
 		else
-		  argvadd(&ac_cmd, &av_cmd, t_tkill);
+		  sfh_argv_add(&ac_cmd, &av_cmd, t_tkill);
 
 		if (ao_taken(ad, "sessionprefix")) {
-		  argvadd(&ac_cmd, &av_cmd, "-sessionprefix");
-		  argvadd(&ac_cmd, &av_cmd, 
+		  sfh_argv_add(&ac_cmd, &av_cmd, "-sessionprefix");
+		  sfh_argv_add(&ac_cmd, &av_cmd, 
 			  ao_param(ad, "sessionprefix", 0, 0));
 		}
 		if (ao_taken(ad, "sessionsuffix")) {
-		  argvadd(&ac_cmd, &av_cmd, "-sessionsuffix");
-		  argvadd(&ac_cmd, &av_cmd, 
+		  sfh_argv_add(&ac_cmd, &av_cmd, "-sessionsuffix");
+		  sfh_argv_add(&ac_cmd, &av_cmd, 
 			  ao_param(ad, "sessionsuffix", 0, 0));
 		}
 		if (fl_debug) {
-		  argvadd(&ac_cmd, &av_cmd, "-d");
+		  sfh_argv_add(&ac_cmd, &av_cmd, "-d");
 		  printf("hboot: ");
 		  for (i = 0; i < ac_cmd; i++)
 		    printf("%s ", av_cmd[i]);
@@ -266,7 +281,6 @@ main(int argc, char *argv[])
 	  show_help(NULL, "lib-call-fail", "putenv", NULL);
 	  exit(errno);
 	}
-
 #if 1
 	/* Comment this out to make the TM extensions to PBS work
            nicely -- everything will be in one session, so TM can kill
