@@ -274,6 +274,19 @@ main(int argc, char *argv[])
 
 		if (errno == ESRCH) {
 		    lam_debug(debug, " already dead");
+                } else if (errno == EPERM) {
+                    /* 
+                     * So if something happens to a process and its
+                     * pid ends up being reused and we try to kill it
+                     * and it's someone else's process, tkill used to
+                     * get annoyed and complain on stderr.  The
+                     * problem is that lamboot takes that to mean
+                     * something really bad happens and aborts the
+                     * lamboot.  Since that seems silly, we are
+                     * instead going to print a warning and continue
+                     * on.
+                     */
+                    lam_debug(debug, " permission denied (ignoring pid)");
 		} else {
 		    lam_debug(debug, "\n");
 		    perror("tkill (kill)");
