@@ -12,7 +12,7 @@
  * 
  * $HEADER$
  *
- * $Id: wipe.c,v 6.46 2003/04/01 13:32:53 jsquyres Exp $
+ * $Id: wipe.c,v 6.50 2003/11/24 19:33:07 jsquyres Exp $
  *
  *	Function:	- fully-connected LAM booting tool
  *			- boots Trollius on a network of UNIX hosts
@@ -23,12 +23,14 @@
 #include <stdio.h>
 #include <signal.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <args.h>
 #include <lamnet.h>
 #include <etc_misc.h>
 #include <lam-ssi.h>
 #include <lam-ssi-boot.h>
+#include <sfh.h>
 
 
 /*
@@ -47,13 +49,17 @@ main(int argc, char *argv[])
   struct lamnode *lamnet;	/* network description array */
   int unusedc;
   char **unusedv;
+  char *lamprefix = NULL;
+  char **cmdv = NULL;
+  int cmdc;
+  int cmdv0_len;
 
   /* Parse the command line */
 
   ad = ao_init();
   if (ad == 0) {
     ret = errno;
-    perror("wipe (ao_init)");
+    perror("lamwipe (ao_init)");
     exit(ret);
   }
   ao_setflags(ad, AOPRESERVE_ARGV);
@@ -63,11 +69,15 @@ main(int argc, char *argv[])
   ao_setopt(ad, "w", 0, 1, AOINT);
   ao_setopt(ad, "sessionprefix", 0, 1, 0);
   ao_setopt(ad, "sessionsuffix", 0, 1, 0);
+  ao_setopt(ad, "prefix", 0, 1, 0);
+  ao_setopt(ad, "withlamprefixpath", 0, 0, 0);
 
+  lam_ssi_base_open(ad);
   lam_ssi_base_ao_setup(ad);
 
   if (ao_parse(ad, &argc, argv)) {
-    show_help("wipe", "usage", NULL);
+    show_help("lamwipe", "usage", NULL);
+    lam_ssi_base_close();
     exit(EUSAGE);
   }
 
@@ -78,6 +88,45 @@ main(int argc, char *argv[])
   if (ao_taken(ad, "d"))
     putenv("LAM_MPI_SSI_boot_verbose=level:1000,stderr");
 
+  /* Check for -withlamprefixpath. If not supplied and prefix supplied
+     then we need to call the prefix wipe */
+
+  if (!ao_taken(ad, "withlamprefixpath") && ao_taken(ad, "prefix")) {
+
+    /* We have got the prefix, so launch the prefix wipe */
+    lamprefix = ao_param(ad, "prefix", 0, 0);
+      
+    /* Build the command to be passed to wipe -- add the option 
+       -with-lam-prefix-path to make wipe know that it has been called 
+       and has been supplied a prefix for locating other binaries */
+
+    cmdv = sfh_argv_dup(argv);
+    cmdc = argc;
+
+    /* Replace cmdv[0] with the new wipe path */
+    free(cmdv[0]);
+
+    cmdv0_len = strlen("/bin/wipe") + strlen(lamprefix) + 2;
+    cmdv[0] = (char *) malloc (cmdv0_len);
+
+    if (cmdv[0] == NULL) {
+      show_help(NULL, "system-call-fail", "malloc", NULL);
+      return LAMERROR;
+    }
+    snprintf(cmdv[0], cmdv0_len, "%s%s", lamprefix, "/bin/wipe");
+
+    /* Check to see if path is valid and you got permissions */
+    if (access(cmdv[0], X_OK) == -1) {
+      show_help("boot","invalid-path-or-no-permission", lamprefix, "wipe");
+      return LAMERROR;
+    }
+    sfh_argv_add(&cmdc, &cmdv, "-withlamprefixpath");
+
+    /* Fork the prefix wipe */
+    _lam_few(cmdv); 
+    return (0);
+  }
+
   /* Compatability arguments, from pre-SSI days (and it's just a
      heckuva lot shorter to type!) */
 
@@ -90,13 +139,17 @@ main(int argc, char *argv[])
      its own error message if necessary, so exiting upon failure is
      sufficient here. */
 
-  if (lam_ssi_base_ao_process_args(ad) != 0)
+  if (lam_ssi_base_ao_process_args(ad) != 0) {
+    lam_ssi_base_close();
     exit(errno);
+  }
 
   /* Open the boot SSI */
 
-  if (lam_ssi_boot_open(ad, LAM_SSI_BOOT_LOCATION_ROOT) != 0) {
+  if (lam_ssi_boot_open(ad) != 0 ||
+      lam_ssi_boot_select(LAM_SSI_BOOT_LOCATION_ROOT) != 0) {
     show_help("ssi-boot", "unable-to-init", NULL);
+    lam_ssi_base_close();
     exit(1);
   }
 
@@ -104,6 +157,7 @@ main(int argc, char *argv[])
 
   if (ao_taken(ad, "V")) {
     lam_show_version(1);
+    lam_ssi_base_close();
     exit(0);
   }
 
@@ -112,6 +166,7 @@ main(int argc, char *argv[])
 #ifndef LAM_ALLOW_RUN_AS_ROOT
   if (getuid() == 0 || geteuid() == 0) {
     show_help(NULL, "deny-root", NULL);
+    lam_ssi_base_close();
     exit(EACCES);
   }
 #endif
@@ -120,12 +175,14 @@ main(int argc, char *argv[])
 
   ao_unused(ad, &unusedc, &unusedv);
   if ((errno = (unusedc <= 2) ? 0 : EUSAGE)) {
-    show_help("wipe", "usage", NULL);
+    show_help("lamwipe", "usage", NULL);
+    lam_ssi_base_close();
     exit(errno);
   }
 
   if (ao_taken(ad, "h")) {
-    show_help("wipe", "usage", NULL);
+    show_help("lamwipe", "usage", NULL);
+    lam_ssi_base_close();
     exit(0);
   }
 
@@ -135,26 +192,34 @@ main(int argc, char *argv[])
   /* Let the boot SSI parse the command line.  Relevant errors will be
      displayed before returning from this function. */
 
-  if (lam_ssi_boot.lsba_parse_options(ad, 1) != 0)
+  if (lam_ssi_boot.lsba_parse_options(ad, 1) != 0) {
+    lam_ssi_base_close();
     exit(1);
+  }
 
   /* Let the boot SSI generate a list of nodes.  Relevant errors will
      be displayed before returning from this function. */
 
-  if (lam_ssi_boot.lsba_allocate_nodes(&lamnet, &nlamnet, &iorigin) != 0)
+  if (lam_ssi_boot.lsba_allocate_nodes(&lamnet, &nlamnet, &iorigin) != 0) {
+    lam_ssi_base_close();
     exit(1);
+  }
 
   /* Let the boot SSI do an error check on the nodes.  Relevant errors
      will be displayed before returning from this function. */
 
-  if (lam_ssi_boot.lsba_verify_nodes(lamnet, nlamnet) != 0)
+  if (lam_ssi_boot.lsba_verify_nodes(lamnet, nlamnet) != 0) {
+    lam_ssi_base_close();
     exit(1);
+  }
 
   /* Let the boot SSI prepare to boot.  Relevant errors will be
      displayed before returning from this function. */
 
-  if (lam_ssi_boot.lsba_prepare_boot() != 0)
+  if (lam_ssi_boot.lsba_prepare_boot() != 0) {
+    lam_ssi_base_close();
     exit(1);
+  }
 
   /* Do the whole file, or just part of it? */
 
@@ -164,15 +229,20 @@ main(int argc, char *argv[])
   /* Now actually have the boot SSI boot the nodes.  Relevant errors
      will be displayed before returning from this function. */
 
-  if (lam_ssi_boot.lsba_start_rte_procs(lamnet, nlamnet, LAM_SSI_BOOT_PROC_WIPE,
-				       &num_started) != 0 ||
-      num_started != nlamnet)
+  if (lam_ssi_boot.lsba_start_rte_procs(lamnet, nlamnet, 
+                                        LAM_SSI_BOOT_PROC_WIPE,
+                                        &num_started) != 0 ||
+      num_started != nlamnet) {
+    lam_ssi_base_close();
     exit(1);
+  }
 
   /* Free up resources */
 
-  if (lam_ssi_boot.lsba_deallocate_nodes(&lamnet, &nlamnet) != 0)
+  if (lam_ssi_boot.lsba_deallocate_nodes(&lamnet, &nlamnet) != 0) {
+    lam_ssi_base_close();
     exit(1);
+  }
 
   /* All done with the boot SSI module (don't care about errors at
      this point) */
@@ -182,5 +252,7 @@ main(int argc, char *argv[])
 
   /* All done */
 
+  ao_free(ad);
+  lam_ssi_base_close();
   return ret;
 }
